################################################################################
#
# Variable window 
#
################################################################################


################################################################################
#
# Needed procedures
#
################################################################################

proc VariableSystemAddVariable {name pos} {
    global globvar entrywidth varcard varcanvref vartablist varWin
    
    frame $varWin.canvas.l_$name
    frame $varWin.canvas.r_$name
    message $varWin.canvas.l_$name.msg -width 150 -text "$name"
    entry $varWin.canvas.r_$name.en -width $entrywidth -relief sunken \
	-textvariable globvar($name)

    pack $varWin.canvas.l_$name.msg -side left -anchor w
    pack $varWin.canvas.r_$name.en -side left -anchor w
    
    # append var in global tab list
    set vartablist [linsert $vartablist [expr $pos - 1] "$varWin.canvas.r_$name.en"]

    VariableSystemEntryBindtag $varWin.canvas.r_$name.en 
    
    # move all variables above pos one position.
    for {set i $varcard} {$i >= $pos} {incr i -1} {
	# set new position and alter varcanfref
	$varWin.canvas.c move $varcanvref($i) 0 25
	set varcanvref([expr $i + 1]) $varcanvref($i)
    }
    
    $varWin.canvas.c create window  15 [expr 10 + 25 * $pos] -window $varWin.canvas.l_$name -anchor w -tags _$name
    $varWin.canvas.c create window 150 [expr 10 + 25 * $pos] -window $varWin.canvas.r_$name -anchor w -tags _$name
    bindtags $varWin.canvas.l_$name.msg \
	[concat [bindtags $varWin.canvas.l_$name.msg]  GlobalVariableDragging]
    set varcanvref($pos) _$name
    
    incr varcard
    set max [expr 10 + 25 * [expr $varcard + 1]]
    $varWin.canvas.c configure -scrollregion "0 0 0 $max"
     unset max
}

proc VariableSystem_next {enteredFocus list} {
    global varWin varcanvref varcard ignore_enter

    set i [lsearch $list $enteredFocus]
    incr i
    if {$i==-1} {puts $enteredFocus; return; }
    if {$i>=$varcard} {
	set i 0
	set ignore_enter 1
	$varWin.canvas.c yview moveto 0
    } else {
	set canvmax [expr 10+25.0*($varcard+1) + 1]
	set entryheight 25
	set top_of_entry [expr 25 + 25.0*$i]
	set bottom_of_entry [expr $top_of_entry+$entryheight]
	set top [lindex [$varWin.canvas.c yview] 0]
	set bottom [lindex [$varWin.canvas.c yview] 1]
	set onepage [expr $bottom - $top]
	
	if {($bottom_of_entry-2) / $canvmax > $bottom } {
	    set ignore_enter 1
	    $varWin.canvas.c yview moveto \
	           [ expr  ($bottom_of_entry-2) / $canvmax  - $onepage ]
	}
    }
    tkTabToWindow [lindex $list $i]
}

proc VariableSystem_prev {enteredFocus list} {
    global varWin varcanvref varcard ignore_enter

    set i [lsearch $list $enteredFocus]
    if {$i==-1} {puts $enteredFocus; return; }
    incr i -1
    set top [lindex [$varWin.canvas.c yview] 0]
    if {$i<0} then {
	set i [expr $varcard - 1]
	set ignore_enter 1 
	$varWin.canvas.c yview moveto 1
    } else {
	set canvmax [expr 10+25.0*($varcard+1) + 1]
	set top_of_entry [expr 25 + 25.0*$i]

	if { ($top_of_entry-2) / $canvmax < $top } {
            set ignore_enter 1
	    $varWin.canvas.c yview moveto [expr ($top_of_entry-2) / $canvmax]
	}
    }
    tkTabToWindow [lindex $list $i]
}

### contruct bindings for manipulation of variable name labels
proc VariableSystemLabelBinding {} {
    # Ins: createvar (which name?) here
    # Del: deletevar this
    # Drag: show copy of namelabel, leave it under pointer while canvas moves
    # Drop: varprops: move variable to this position, destroy copy
    # Ins+Del on selection variable: add+remove subselection pseudovariables
}
VariableSystemLabelBinding
proc VariableSystemLabelBindtag {en} {
    # emacs-flavour key bindings are already defined in bindtag 'Entry'
    bindtags $en [concat VariableLabelEntry [bindtags $en]]
}

### construct key bindings for variable Entry windows
proc VariableSystemEntryBinding {} {
    set update {
	VariableSystemUpdateVariable $cursorFocus; set cursorFocus %W}
    bind VariableSystemEntry <ButtonRelease-1> $update
    bind VariableSystemEntry <Return> $update
    
    set next {
	VariableSystemUpdateVariable $cursorFocus
	VariableSystem_next $cursorFocus $vartablist
	set cursorFocus [focus]
    }
    regsub -- next $next prev prev
    bind VariableSystemEntry  <Down> $next
    bind VariableSystemEntry    <Up> $prev
    # Emacs up/down disabled because they aren't bound in dialog windows yet
    # (And because Elsbeth history commands are bound to them)
    # bind VariableSystemEntry <Control-n> $next
    # bind VariableSystemEntry <Control-p> $prev
    unset next prev
    
    # War im toplevel tag
    bind VariableSystemEntry <Leave> {
	VariableSystemUpdateVariable $cursorFocus ;# focus $cursorFocus
    }
    
    global focus_follows_mouse; set focus_follows_mouse 0
    bind VariableSystemEntry <Enter> {
	if {$focus_follows_mouse} { # buggy
	    if {$ignore_enter} {
		set ignore_enter 0 ;# Tab scroll moved entry %W under pointer
	    } {
		set cursorFocus %W; tkTabToWindow %W
	    }
	}
    }
    # suppress tk.tcl default: bind all <Tab> {tkTabToWindow [tk_focusNext %W]}
    bind VariableSystemEntry       <Tab> [concat \
					  [bind VariableSystemEntry <Down>] { ;
					  break } ]
    bind VariableSystemEntry <Shift-Tab> [concat \
					  [bind VariableSystemEntry   <Up>] { ;
				          break }]
}
VariableSystemEntryBinding
proc VariableSystemEntryBindtag {en} {
    # emacs-flavour key bindings are already defined in bindtag 'Entry'
    bindtags $en [concat VariableSystemEntry [bindtags $en]]
}

proc VariableSystemUpdateVariable {enteredFocus} {
    global vartablist varcanvref globvar

    set i [lsearch $vartablist $enteredFocus]
    if {$i != -1} {
	# counts from 0, varcanvref counts from 1.
	incr i   
	set name [string trim $varcanvref($i) _]
	variabletool setvalue $name $globvar($name)
	# variabletool must infer correct selection var contents
    }
}

proc VariableSystemDelVariable {name pos} {
    global globvar varcard varcanvref vartablist varWin

    #delete variable from variable tab list
    set index [lsearch $vartablist $varWin.canvas.r_$name.en]
    if {$index>-1} {
	set vartablist [lreplace $vartablist $index $index]
    }
    
    update
    $varWin.canvas.c delete $varcanvref($pos)
    
    # delete variable from varcanvref
    for {set i $pos} {$i < $varcard} {incr i} {
	set varcanvref($i) $varcanvref([expr $i + 1])
	$varWin.canvas.c move $varcanvref($i) 0 -25
    }
    unset varcanvref($varcard)
    incr varcard -1

    update
    destroy $varWin.canvas.l_$name
    destroy $varWin.canvas.r_$name

    set max [expr 10 + 25 * [expr $varcard + 1]]
    $varWin.canvas.c configure -scrollregion "0 0 0 $max"
    unset max

}

proc VariableSystemDelAllVariables {} {
    global varcard varcanvref vartablist varWin

    # delete all displayed items
    for {set i 1} {$i <= $varcard} {incr i} {
	update
	$varWin.canvas.c delete $varcanvref($i)
	update
	destroy $varWin.canvas.l$varcanvref($i)
	destroy $varWin.canvas.r$varcanvref($i)
    }

    # unset all variables
    unset vartablist
    for {set i 1} {$i <= $varcard} {incr i} {
	unset varcanvref($i)
    }

    unset varcard

    #initialize all variables
    set varcard 0
    set vartablist [list]
    set varcanvref() ""
    unset varcanvref()
}

proc VariableSystemSetValue {name value} {
    global globvar
    if {[string compare $value (null)]==0} {
	puts stderr "VariableSystemSetValue: value is (null)"
	set globvar($name) ""
    } else {
	set globvar($name) $value
    }
}


#####################################################################################
#
# Globals
#
#####################################################################################

set varWin .dspvar
set varcard 0
set vartablist [list]

# HACK fuer das Feld globvar. Globvar muss ein globales Feld sein, 
# welches vor dem Aufruf von updatedisplay erzeugt sein muss. 
# Primitive Form der Erzeugung eines leeren Feldes:
#     Einfuegen eines Elementes, welches direkt wieder geloescht wird.
#     Als Nebeneffekt hiervon exisiert ein leeres Feld namens globvar.
# Falls jemand eine bessere Initialisierung eines leeren Feldes kennt, 
# bitte aendern oder mich (Kai@i3.informatik.rwth-aachen.de) benachrichtigen.
set globvar() ""
unset globvar()

set varcanvref() ""
unset varcanvref()

set ignore_enter 0 ;# 1 if Entry just moved beneath the pointer during <Tab>
set cursorFocus "" ;# last VariableSystem Entry widget being in focus

# Initialize variable window
  toplevel $varWin -class Display -takefocus 0
  wm iconname $varWin "Global Variables"
  wm title $varWin "Global Variables"
  wm minsize $varWin 420 10
  wm geometry $varWin 420x200+0+0
  wm maxsize $varWin 420 1500
  wm iconposition $varWin 20 100
  wm iconify $varWin

# build frame for menu
frame $varWin.menu -relief raised -bd 2
pack $varWin.menu -side top -fill x

# menu skeleton
proc VariableSystemTestMenu {} {
    global varWin
    set menubar {Do Var Help}
    foreach word $menubar {
	set button $varWin.menu.[string tolower $word]
	set popup $button.menu
	menubutton $button -text $word -underline 0 -menu $popup
	menu $popup -title $word
	set $word $popup
	pack $button -side [expr ("$popup"=="help")?"right":"left"]
    }
    
    $Do add command -label "allv" -command {
	variabletool setallvisible
	ffSetColorsForWin $varWin
    }
    $Do add command -label "sort" -command {
	variabletool sortanddisplay
	ffSetColorsForWin $varWin
    }
    $Var add command -label "createvar" -command {
	variabletool createvar hugo 1
	ffSetColorsForWin $varWin
    }
    $Var add command -label "varprops" -command {
	variabletool varprops hugo foo 5
	ffSetColorsForWin $varWin
    }
    $Var add command -label "deletevar" -command { 
	variabletool deletevar foo
	ffSetColorsForWin $varWin
    }
    $Var add command -label "setselection" -command {
	variabletool setselection Selection [variabletool getvalue foo]
	puts stderr "Wurden die Knoten und Kanten des graphWin farblich markiert?"
    }

    $Help add command -label "rudimental" -command {
puts stderr "Use Button 1 on the left side to drag the sheet vertically.\
\nUse Emacs keys in the value fields and traverse them with (Shift-) Tab.\
\n"
    }
    $Help add command -label "verbose" -state disabled 
}
# VariableSystemTestMenu

 # build frame for canvas
 frame $varWin.canvas -relief flat
 pack $varWin.canvas -fill both

 #build scrollbar and canvas
 scrollbar $varWin.canvas.vscroll -relief sunken \
    -command "$varWin.canvas.c yview" -takefocus 0
 pack $varWin.canvas.vscroll -side right -fill y
 canvas $varWin.canvas.c -scrollregion {0 0 0 10} -width 400 -height 600\
    -yscrollcommand "$varWin.canvas.vscroll set" ;# not -yscrollincrement 25
 pack $varWin.canvas.c -side left -fill both

 variabletool showvariables

bind $varWin     <Enter> {set ignore_enter 0}
bind $varWin       <Tab> {break}
bind $varWin <Shift-Tab> {break}

# binding to enable dragging of variable window canvas
bind GlobalVariableDragging <ButtonPress-1>    {
    $varWin.canvas.c scan mark %X %Y}
bind GlobalVariableDragging <Button1-Motion>  {+
    $varWin.canvas.c scan dragto 0 %Y}

bindtags $varWin.canvas.c [concat \
			       [bindtags $varWin.canvas.c] \
			       GlobalVariableDragging      ]

return ok
