###############################################################################
#
# canvas.tcl
# 
# 
# Canvas-Prozeduren fuer das Haupt-TCL-Fenster
#
###############################################################################

# Procedure buildMainCanvas
proc buildMainCanvas {} {
  # build widget .frame0
  frame .frame0 \
    -background {gainsboro} \
    -relief {raised}

  global aktuCanvas

  # build widget .frame0.scrollbar1
  scrollbar .frame0.scrollbar1 \
      -command {global aktuCanvas; $aktuCanvas yview} \
      -bg {gold} \
      -relief {raised}

  # build widget .frame0.scrollbar3
  scrollbar .frame0.scrollbar3 \
      -command {global aktuCanvas; $aktuCanvas xview} \
      -bg {gold} \
      -orient {horizontal} \
      -relief {raised}


  # build widget .frame0.canvas2
  canvas .frame0.canvas2 \
    -background {white} \
    -height {207} \
    -insertbackground {black} \
    -insertofftime {600} \
    -relief {raised} \
    -selectbackground {black} \
    -selectforeground {gainsboro} \
    -width {300} \
    -xscrollcommand {.frame0.scrollbar3 set} \
    -yscrollcommand {.frame0.scrollbar1 set}

  # Aufspannen des Canvas auch bei kleinen Graphen, weil sonst das Ziehen von
  # Scrollschieber oder Canvasflaeche selbst jenseits der Scrollregion kein Feedback
  # bietet und der Benutzer mit der Maus leicht zwischen den Endanschlaegen pendelt.
  .frame0.canvas2 create rectangle -1500 -1500 2500 2500 \
      -outline [.frame0.canvas2 cget -bg]

  global dragItem
  set dragItem(.frame0.canvas2) [.frame0.canvas2 create rectangle 0 0 0 0]
  .frame0.canvas2 itemconfigure $dragItem(.frame0.canvas2) \
    -outline [.frame0.canvas2 cget -bg]
  
  global isDirected
  set isDirected(.frame0.canvas2) {YES}
  
  global tk_version
  if {$tk_version < 4} {
    .frame0.canvas2 xview 300 
    .frame0.canvas2 yview 300 
  } {
    .frame0.canvas2 xview scroll 300 units
    .frame0.canvas2 yview scroll 300 units
  }

  pack append .frame0 \
    .frame0.scrollbar1 {right frame center filly} \
    .frame0.canvas2 {top frame center expand fill} \
    .frame0.scrollbar3 {top frame center fillx} 

}


# Procedure addCanvasBindings
proc addCanvasBindings {canv } {
  global symbolicName
  if { $symbolicName(graphView) == $canv} {
    set befehl recall 
  } {
    set befehl SubRecall.$canv
  }        

  # bindings
  
  set body [concat {
    global draging
    global workingMode
    
    switch $workingMode {Navigate}} "{" {
      global dragBoxX;
      global dragBoxY;
    } $befehl { moveEye [expr [%W canvasx %x] - $dragBoxX] [expr [%W canvasy %y] - $dragBoxY];
      set dragBoxX [%W canvasx %x];
      set dragBoxY [%W canvasy %y];
    } "}" { {default} {
      if {$draging == 1} {
        drawDragBox %W [%W canvasx %x] [%W canvasy %y]
      }
    }
  }]
  
  bind $canv <B1-Motion> $body
  
  bind $canv <Button-1> {

    global dragBoxX
    global dragBoxY
    global draging  

    set draging 0  
    set overlapping_items [%W find overlapping \
                             [%W canvasx %x] [%W canvasy %y] [%W canvasx %x] \
                             [%W canvasy %y]] 

    if {$overlapping_items == ""} {    
      set dragBoxX [%W canvasx %x]
      set dragBoxY [%W canvasy %y]
      global dragItem 
      drawDragBox %W $dragBoxX $dragBoxY
      %W itemconfigure $dragItem(%W) -outline black
      set draging 1
    }
  }

  set body [concat {
    global dragBoxX
    global dragBoxY
    global draging
    
    if {$draging == 1} } "{" {
      global workingMode
      
      switch $workingMode {Create}} "{" {
      } $befehl { unsetCurrent
        set overlapping_items [%W find overlapping $dragBoxX \
                                 $dragBoxY [%W canvasx %x] [%W canvasy %y]] 
        foreach i $overlapping_items } "{" {
          if {[lsearch [%W gettags $i] nodetag] != -1}} "{
           $befehl" { multiSelect $i
          } "}
          }
      } {Zoom} {" {        
        
        scan "[%W cget -scrollregion]" {%%d %%d %%d %%d} sx1 sy1 sx2 sy2
        scan "[%W xview]" {%%f %%f} fraqX1 fraqX2
        scan "[%W yview]" {%%f %%f} fraqY1 fraqY2 
        set fraqX [expr $fraqX2-$fraqX1]
        set fraqY [expr $fraqY2-$fraqY1]
        set breite [expr ($sx2 - $sx1)*$fraqX]
        set hoehe [expr ($sy2 - $sy1)*$fraqY]
        
        set centX [expr $dragBoxX + ([%W canvasx %x]-$dragBoxX)/2 ]
        set centY [expr $dragBoxY + ([%W canvasx %Y]-$dragBoxY)/2 ]
        
        centerCanvas %W $centX $centY;
        
        } $befehl { boxscale [expr ($breite/([%W canvasx %x]-$dragBoxX))] \
          [expr ($hoehe/([%W canvasy %y]-$dragBoxY))]
            
        } "} {Navigate} {
    } " {
      set dragBoxX 0
      set dragBoxY 0
      drawDragBox %W 0 0
      global dragItem
      %W itemconfigure $dragItem(%W) -outline [%W cget -bg]
      set draging 0
    } "}" ]

  bind $canv <ButtonRelease-1> $body



#  set body [concat {
#    set overlapping_items [%W find overlapping \
#                             [%W canvasx %x] [%W canvasy %y] [%W canvasx %x] \
#                             [%W canvasy %y]] 
#    set node_exist 0
#    foreach i $overlapping_items {
#      if {[lsearch [%W gettags $i] nodetag] != -1} { 
#        set node_exist 1
#        break
#      }
#    }
#    if {$node_exist == 0}} "{
#      $befehl" { newNode 1 [%W canvasx %x] [%W canvasy %y] %W
#      } "}" ]
  

  bind $canv <Control-Button-1> ""
    #JK alt
    #$body

  set body [concat "
    $befehl" { addInstantPoint [%W canvasx %x] [%W canvasy %y] 
    }]

  bind $canv <Control-Shift-Button-1> $body

  set body [concat "
    $befehl" { removeInstantPoint [%W canvasx %x] [%W canvasy %y]
    }]

  bind $canv <Control-Shift-Button-2> $body

  bind $canv <Double-Button-1> "
    $befehl unsetCurrent
  "
  
  bind $canv <Enter> {focus %W}
  bind $canv <Mod1-Key> "$befehl modPress %A"
  bind $canv <KeyPress> "$befehl keyPress %K"

# Folgende Zeilen wurden am 13.02.98 deaktiviert, da bei Shift-Button-1 auf das Canvas
# (nicht auf einen Knoten oder eine Kante !!) eine Segmentation Violation erzeugt wird.
# Der Fehler scheint in selectIfNot() zu liegen - habe ihn jedoch nicht weiter verfolgt.
# Das Deselektieren bereits selektierter Canvas-Elemente erfolgt weiterhin korrekt.
# Weitere Gruende fuer den Aufruf der Funktion sind mir nicht bekannt.
# @Rolf
#    set body [concat "
#    $befehl" {selectIfNot [%W find withtag current] [%W canvasx %x] \
#		   [%W canvasy %y] 
#    }]

# current item will be searched in nodeMap, leading to selectIfNot(NULL)
# -> stdGraph /selectIfNot/
 
#  bind $canv <Shift-Button-1> $body

    # Provide accelerated dragging of canvas  in addition to
    # using scrollbar sliders in opposite edges of the browser window.
    # Acceleration can be parameterized in tk-dash and possibly tk8.1 .

    bind $canv <ButtonPress-2>  "+ $canv scan   mark %X %Y "
    bind $canv <Button2-Motion> "+ $canv scan dragto %X %Y "
}

# Procedure: drawDragBox
proc drawDragBox {canv x2 y2} {
  global dragBoxX
  global dragBoxY
  global dragItem
  global symbolicName
  eval $canv coords $dragItem($canv) $dragBoxX \
    $dragBoxY $x2 $y2 
  
}

# Procedure centerCanvas
proc centerCanvas {dasCanvas x y} {
  scan "[$dasCanvas cget -scrollregion]" {%d %d %d %d} sx1 sy1 sx2 sy2
  
  scan "[$dasCanvas xview]" {%f %f} fraqX1 fraqX2
  scan "[$dasCanvas yview]" {%f %f} fraqY1 fraqY2
  set fraqX [expr $fraqX2-$fraqX1]
  set fraqY [expr $fraqY2-$fraqY1]
  
  set breite [winfo width $dasCanvas]
  set hoehe [winfo height $dasCanvas]
  
  $dasCanvas xview moveto [expr ($x- $breite/2-$sx1)/($sx2 - $sx1)];
  $dasCanvas yview moveto [expr ($y- $hoehe/2-$sy1)/($sy2 - $sy1)];
}


# Procedure: findGraph
proc findGraph {} {
  global symbolicName
  set dasCanvas $symbolicName(graphView)
  
  if { [$dasCanvas gettags nodetag] != ""} {
    set x [lindex  [$dasCanvas coords nodetag] 0];
    set y [lindex  [$dasCanvas coords nodetag] 1];
    
    centerCanvas $dasCanvas $x $y
  }
}

# Procedure: moveStdLabel
proc moveStdLabel { c x y} {
global symbolicName
[set symbolicName(graphView)] coords $c [expr $x+5] $y
}

# Procedure: moveLabel
proc moveLabel {canv c x y} {
    $canv coords $c [expr $x+5] $y
}

# Procedure: moveStdLine
proc moveStdLine { c args} {
global symbolicName
eval [set symbolicName(graphView)] coords $c $args
}

# Procedure: moveLine
proc moveLine {canv c args} {
eval $canv coords $c $args
}

# Procedure: removeItem
proc removeItem { canv x} {
$canv delete $x
destroy $canv.$x ;# remove widgets created in add{Box,Class,Package}Node -kmf
}

# Procedure: removeStdLabel
proc removeStdLabel { x} {
global symbolicName
return [removeItem [set symbolicName(graphView)] $x]
}

# Procedure: removeStdLine
proc removeStdLine { x} {
global symbolicName
return [removeItem [set symbolicName(graphView)] $x]
}

# Procedure: removeStdNode
proc removeStdNode { x} {
global symbolicName
return [removeItem [set symbolicName(graphView)] $x]
}


# Procedure setAktuCanvas
proc setAktuCanvas { {canv "."} } {
  global symbolicName
  global aktuCanvas
  global oldAktuCanvas
  global aktuCommand

  if {$canv == "."} {
    set canv $symbolicName(graphView) 
  }

  set oldAktuCanvas $aktuCanvas
  set aktuCanvas $canv

  if { $aktuCanvas == $symbolicName(graphView) } {
     set aktuCommand recall
  } {
     set aktuCommand SubRecall.$aktuCanvas
  }

  $oldAktuCanvas configure -xscrollcommand {} -yscrollcommand {}
  $aktuCanvas configure     -xscrollcommand {.frame0.scrollbar3 set} \
    -yscrollcommand {.frame0.scrollbar1 set}

  adjWindow $aktuCanvas 30
}  

# Procedure hideallLabels
proc hideallLabels {} {
  global symbolicName
  global showLabelsFlag
  if { $showLabelsFlag == 1 } {
    showallLabels
  } {
    set canv $symbolicName(graphView)
    foreach i [$canv find withtag cfLabelText] {
      $canv itemconfigure $i -window {}
    }
  }
}

# Procedure showallLabels
proc showallLabels {} {
  global symbolicName
  set canv $symbolicName(graphView)
  foreach i [$canv find withtag cfLabelText] {
    if { [ $canv.$i cget -text ] != "" } {
      $canv itemconfigure $i -window $canv.$i
    }
  }
}

# Procedure: addText
proc addText { canv x y t} {
  global showLabelsFlag
  set cfTmpTag [$canv create window [expr $x+5] $y]
  label $canv.$cfTmpTag -padx 2 -pady 2 -bd 0 -relief flat\
    -bg [$canv cget -bg] -fg black -text $t -anchor center
  $canv itemconfigure $cfTmpTag -anchor w

  if { ($t != "") && ($showLabelsFlag == 1) } {
    $canv itemconfigure $cfTmpTag -window $canv.$cfTmpTag
    set tmp [$canv bbox $cfTmpTag]
    set width [expr [lindex $tmp 2] - [lindex $tmp 0]]
    set height [expr [lindex $tmp 3] - [lindex $tmp 1]]
    $canv itemconfigure $cfTmpTag -width $width -height $height
  } {
    $canv.$cfTmpTag configure -padx 0 -pady 0
    $canv itemconfigure $cfTmpTag -width 0 -height 0
    $canv itemconfigure $cfTmpTag -window {}
  }
  
#     set cfTmpTag [$canv create text $x $y]
  $canv addtag cfLabelText withtag $cfTmpTag
  $canv raise $cfTmpTag all
#     $canv itemconfigure $cfTmpTag -anchor {w} -justify {left}\
#             -font -*-Courier-Bold-R-Normal--*-120-* -text $t
    
  return $cfTmpTag
}


# Procedure: addStdText
proc addStdText { x y t} {
  global symbolicName
  return [addText [set symbolicName(graphView)] $x $y $t]
}

# Procedure: changeStdText
# proc changeStdText { t y} {
#     global symbolicName
#     [set symbolicName(graphView)].$t configure -text $y
#   set tmp [$canv bbox $tmpTag]
#   set width [expr [lindex $tmp 2] - [lindex $tmp 0]]
#   set height [expr [lindex $tmp 3] - [lindex $tmp 1]]
#   $canv itemconfigure $tmpTag -width $width -height $height


# }

# Procedure: changeStdText
# proc changeStdText { t y} {
# global symbolicName
# [set symbolicName(graphView)] itemconfigure $t -text $y
# }

# Procedure: changeText
proc changeText {canv t y} { 
  $canv.$t configure -text {}
  $canv itemconfigure $t -window {}
  $canv.$t configure -padx 0 -pady 0
  $canv itemconfigure $t -width 0 -height 0

  $canv itemconfigure $t -window $canv.$t

  if {$y != ""} {
    $canv.$t configure -text $y
    set tmp [$canv bbox $t]
    set width [expr [lindex $tmp 2] - [lindex $tmp 0]]
    set height [expr [lindex $tmp 3] - [lindex $tmp 1]]
    $canv itemconfigure $t -width $width -height $height
  }
}


# Procedure: adjStdXY
proc adjStdXY { x y x2 y2} {
    global symbolicName
    [set symbolicName(graphView)] configure -scrollregion "$x $y $x2 $y2"
}

# Procedure: adjXY
proc adjXY { canv x y x2 y2} {
    $canv configure -scrollregion "$x $y $x2 $y2"
}

# Procedure: adjWindow
proc adjWindow { {fenster "."} {offset "0"} } {
  if {$fenster == "."} {
    global symbolicName
    set fenster [set symbolicName(graphView)] 
  }
  
  if { [scan [$fenster bbox all] "%d %d %d %d" x y w h] != -1 } {
    $fenster configure -scrollregion "[expr $x-$offset] [expr $y-$offset] \
 		[expr $w+$offset] [expr $h+$offset]"
  }
}

return ok
