###############################################################################
#
# changeToIcon.tcl
# 
# 
# Prozeduren zur Knotendarstellung ICON
#
###############################################################################


# Procedure: changeToIcon
proc changeToIcon {item icon selIcon {canv "."}} {
  global ICON_LIB_DIR
  if {$canv == "."} {
    global symbolicName
    set canv [set symbolicName(graphView)]
  }
  # puts "Change to Icon $icon $selIcon"
  focus $canv
  
  if {[lsearch [$canv gettags $item] iconNode] != -1} {
    global tk_version
    if { $tk_version < 4 } {
      global icons
      $canv itemconfigure $item  -bitmap \
        @$ICON_LIB_DIR/xpm/[set icon]N.xpm
      set icons($item) $icon
    } {
      global selectImage
      global unselectImage
      global iconType    
      global iconSelType    
      
      set tmpCreate [createIcon $icon $selIcon]
      
      if {$tmpCreate == ""} {
        return ""
      }
      
      scan $tmpCreate "%s %s %s %s" \
        unselectImage($item) \
        selectImage($item) iconType($item) \
        iconSelType($item)
      
      $canv itemconfigure $item -image $unselectImage($item)
    }
    return ""
  } {
    scan [$canv coords $item] {%f %f} x y
    removeItem $canv $item
    return [addNode $canv $x $y 15 {black} $icon $selIcon]
  }
}

# Procedure createIcon
proc createIcon { iconFile {selIconFile "__NoIcon__"} } {
  global ICON_LIB_DIR
#     puts "Create Icon"
#     puts $iconFile
#     puts $selIconFile
  
  set bild [file rootname $iconFile]
  set typ [file extension $iconFile]	
  if { $typ == "" } {
    set typ xpm
  } {
    set typ [string trimleft $typ {.}]
  }
  
  switch $typ {xbm} {
    set createTyp bitmap
  } {xpm} {
    set createTyp pixmap
  } {default} {
    set createTyp photo
  }       

  if { $selIconFile != "__NoIcon__" } {
    set selBild [file rootname $selIconFile]
    set selTyp [file extension $selIconFile]	
    if { $selTyp == "" } {
      set selTyp xpm
    } {
      set selTyp [string trimleft $selTyp {.}]
    }

    switch $selTyp {xbm} {
      set selCreateTyp bitmap
    } {xpm} {
      set selCreateTyp pixmap
    } {default} {
      set selCreateTyp photo
    }       
  } {
    set selTyp $typ
    set selBild $bild
    set selCreateTyp $createTyp
  }
  
  # Wir haben das Bild und evlt. auch ein Select Bild
  # Wenn kein Select Bild => Bild := Select Bild
  if { [file exists $ICON_LIB_DIR/$typ/[set bild].$typ] } {
    set normalBild $ICON_LIB_DIR/$typ/[set bild].$typ
    if { ($selBild != "") &&   \
           [file exists $ICON_LIB_DIR/$selTyp/[set selBild].$selTyp]} {
      set selectBild $ICON_LIB_DIR/$selTyp/[set selBild].$selTyp
    } {
      set selectBild $ICON_LIB_DIR/$typ/[set bild].$typ
    }
  } { 
  # Wir haben kein Bild gefunden und suchen nach der alten *N *S 
    if { [file exists $ICON_LIB_DIR/$typ/[set bild]N.$typ] } {
      set normalBild $ICON_LIB_DIR/$typ/[set bild]N.$typ
      if { [file exists $ICON_LIB_DIR/$typ/[set bild]S.$typ] } {
        set selectBild $ICON_LIB_DIR/$typ/[set bild]S.$typ 
      } {
        set selectBild $ICON_LIB_DIR/$typ/[set bild]N.$typ
      }
      set selCreateTyp $createTyp
    } {
      puts stderr " Couldn't find icon \"$iconFile\" !!! "
      return ""
    }
  }
 
  set cfTmpImage [image create $createTyp -file $normalBild ]
  set cfTmpSelImage [image create $selCreateTyp -file $selectBild]

#   puts "$cfTmpImage $cfTmpSelImage $typ $selTyp"
  return "$cfTmpImage $cfTmpSelImage $typ $selTyp"
}


# Procedure: changeMask
proc changeMask { item mask {selMask "__NoMask__"} {fenster "." } } {
  global ICON_LIB_DIR
    
  if {$fenster == "." } {
    global symbolicName
    set fenster $symbolicName(graphView)
  }
  
  if {[lsearch [$fenster gettags $item] iconNode] != -1} {
    global tk_version
    if { $tk_version < 4 } {
      global icons
      $fenster itemconfigure $item  -mask \
        @$ICON_LIB_DIR/xbm/[set mask].xbm
    } {
      global selectImage
      global unselectImage
      global iconType    
      global iconSelType    
      
      set maskBild [file rootname $mask]
      if { $selMask != "__NoMask__" } {
        set selMaskBild [file rootname $selMask]
        set selMaskType $iconSelType($item)
      } {
        set selMaskBild $maskBild
        set selMaskType $iconSelType($item)  
        # ??? obere Zeile evtl. nicht korrekt
      }
      if { $mask != "__NoMask__" } {
        switch $iconType($item) {xbm} {
          $unselectImage($item) configure  -maskfile \
            $ICON_LIB_DIR/xbm/$maskBild.xbm
        } {xpm} {
          $unselectImage($item) configure  -maskfile \
            $ICON_LIB_DIR/xbm/$maskBild.xbm
        } {default} {
          puts stderr "Masks ($mask) not supported for type $iconType($item)"
        }
      }
      
      if {$selMaskBild != "__NoMask__"} {
        switch $selMaskType {xbm} {
          $selectImage($item) configure  -maskfile \
            $ICON_LIB_DIR/xbm/$selMaskBild.xbm
        } {xpm} {
          $selectImage($item) configure  -maskfile \
            $ICON_LIB_DIR/xbm/$selMaskBild.xbm
        } {default} {
          if { $mask != "__NoMask__" } {
            # puts $selMaskType
            puts stderr "Masks ($mask) not supported for type $selMaskType"
          }
        }
      }
    }
  }
}

# Procedure makeIconCanvas
proc makeIconCanvas { frame {name ""} } {
  frame $frame \
    -background {gainsboro} \
    -relief {raised}
  
  if {$name != ""} {
    entry $frame.l -relief flat -bd 0
    $frame.l insert 0 $name
    $frame.l configure -state disabled
    pack append $frame $frame.l {top frame nw}
  }

  canvas $frame.canvas \
    -xscrollcommand "$frame.scrollbar3 set" \
    -yscrollcommand "$frame.scrollbar1 set" \
    -width 0
  
  # build widget .frame0.scrollbar1
  scrollbar $frame.scrollbar1 \
    -command "$frame.canvas yview" \
    -bg {gold} 
  
  # build widget .frame0.scrollbar3
  scrollbar $frame.scrollbar3 \
    -command "$frame.canvas xview" \
    -bg {gold} \
    -orient {horizontal} 
    
  pack append $frame \
    $frame.scrollbar1 {right frame center filly} \
    $frame.scrollbar3 {bottom frame center fillx} \
    $frame.canvas {top frame center expand fill} 
}

# Procedure fillMaskCanvas
proc fillMaskCanvas { canv fromCanv args } {
  set x 10
  set y 10
  set maxY 0
  
  $canv delete all
#   puts $args
  foreach i [lindex $args 0] {
    
    set tmpImage [$fromCanv itemcget $i -image]
    set tmpTag [$canv create image $x $y -image $tmpImage]
     
    scan [$canv bbox $tmpTag] "%f %f %f %f" dx dy dw dh
    set dh [expr $dh - $dy]
    set dw [expr $dw - $dx]
    
    if { $dh > $maxY } {set maxY $dh}
    
    set x [expr $x +$dw+5]
    if { $x > 400 } {
      set x 10
      set y [expr $y +$maxY + 5]
      set maxY 0
    }

    $canv bind $tmpTag <Button-1> {
      global iconBox
      set tmpTag [%W find withtag current]
      scan [%W bbox $tmpTag] {%%f %%f %%f %%f} tx ty tw th
      %W coords aktuFrame [expr $tx-1] [expr $ty-1] [expr $tw+1] \
        [expr $th+1]
      switch %W { 
        .topIcon.frameLeft.framem0.canvas {
          recall setCurrentMask $iconBox([%W itemcget $tmpTag -image])
        } 
        .topIcon.frameRight.framem1.canvas {
          recall setCurrentSelMask $iconBox([%W itemcget $tmpTag -image])
        }
      }
    }
  }
  set tmpFrame [$canv create rectangle 0 0 0 0 -outline blue]
  $canv addtag aktuFrame withtag $tmpFrame

  adjWindow $canv 5
}

# Procedure: showIconBox
proc showIconBox {} {
  toplevel .topIcon
  global tk_version
  global ICON_LIB_DIR
  global iconBox
  
  wm positionfrom .topIcon ""
  wm sizefrom .topIcon ""
  wm maxsize .topIcon 1000 1000
  wm minsize .topIcon 10 10
  wm title .topIcon {Icons: }
  wm geometry .topIcon 350x360
  
  frame .topIcon.frameLeft
  frame .topIcon.frameRight

  makeIconCanvas .topIcon.frameLeft.frame0 "Icons"
  makeIconCanvas .topIcon.frameRight.frame1 "Select Icon"
  makeIconCanvas .topIcon.frameLeft.framem0 "Mask"
  makeIconCanvas .topIcon.frameRight.framem1 "Select Mask"
    
  set canv .topIcon.frameLeft.frame0.canvas
  set selCanv .topIcon.frameRight.frame1.canvas
  set maskCanv .topIcon.frameLeft.framem0.canvas
  set selMaskCanv .topIcon.frameRight.framem1.canvas
  $maskCanv configure -height 30
  $selMaskCanv configure -height 30

  pack append .topIcon.frameRight \
    .topIcon.frameRight.frame1 {top frame center expand fill} \
    .topIcon.frameRight.framem1 {bottom frame center fillx} 

  pack append .topIcon.frameLeft \
    .topIcon.frameLeft.frame0 {top frame center expand fill} \
    .topIcon.frameLeft.framem0 {bottom frame center fillx} 

  pack append .topIcon \
    .topIcon.frameRight {right frame center expand fill} \
    .topIcon.frameLeft {right frame center expand fill} 
  
  ffSetColorsForWin .topIcon

  set x 10
  set y 10
  set maxY 0
  
  foreach i "[glob -nocomplain $ICON_LIB_DIR/xpm/*.xpm] \
  [glob -nocomplain $ICON_LIB_DIR/gif/*.gif] \
  [glob -nocomplain $ICON_LIB_DIR/ppm/*.ppm] \
  [glob -nocomplain $ICON_LIB_DIR/xbm/*.xbm] " {
    
    set bild [file tail $i]
    set typ [file extension $i]	
    
    if { $typ == "" } {
      set typ xpm
    } {
      set typ [string trimleft $typ {.}]
    }
    
    switch $typ {xbm} {
      set createTyp bitmap
    } {xpm} {
      set createTyp pixmap
    } {default} {
      set createTyp photo
    }       
    
    set tmpTag [$canv create image $x $y -anchor nw]
    set tmpSelTag [$selCanv create image $x $y -anchor nw]

    set neuesImage [image create $createTyp -file $i]

    $canv itemconfigure $tmpTag -image $neuesImage
    $selCanv itemconfigure $tmpSelTag -image $neuesImage
        
    set iconBox($neuesImage) $bild
    
    # puts $bild
    # puts "Setting $i at ($x,$y)"
    foreach i "$canv $selCanv" {
      $i bind $tmpTag <Button-1> {
        global iconBox
        set tmpTag [%W find withtag current]
        scan [%W bbox $tmpTag] {%%f %%f %%f %%f} tx ty tw th
        %W coords aktuFrame [expr $tx-1] [expr $ty-1] [expr $tw+1] \
          [expr $th+1]
        switch %W { 
            .topIcon.frameLeft.frame0.canvas {
                recall setCurrentIcon $iconBox([%W itemcget $tmpTag -image])
                recall setNoMask
                if { [file extension $iconBox([%W itemcget $tmpTag -image])]
                     == ".xpm" } {
                    fillMaskCanvas .topIcon.frameLeft.framem0.canvas %W \
                        [%W find withtag [expr $tw-$tx]x[expr $th-$ty]xxbm]
                } {
                    fillMaskCanvas .topIcon.frameLeft.framem0.canvas %W ""
                }
          } 
          .topIcon.frameRight.frame1.canvas {
            recall setCurrentSelIcon $iconBox([%W itemcget $tmpTag -image])
            recall setNoSelMask
              if { [file extension $iconBox([%W itemcget $tmpTag -image])]
                   == ".xpm" } {
                  fillMaskCanvas .topIcon.frameRight.framem1.canvas %W \
                      [%W find withtag [expr $tw-$tx]x[expr $th-$ty]xxbm]
              } {
                  fillMaskCanvas .topIcon.frameRight.framem1.canvas %W ""
              }
          }
        }
      }
    }

    scan [$canv bbox $tmpTag] "%f %f %f %f" dx dy dw dh
    set dh [expr $dh - $dy]
    set dw [expr $dw - $dx]
    
    if { $dh > $maxY } {set maxY $dh}
    
    $canv addtag [set dw]x[set dh]x[set typ] withtag $tmpTag
    $selCanv addtag [set dw]x[set dh]x[set typ] withtag $tmpSelTag    
    
    set x [expr $x +$dw+5]
    if { $x > 150 } {
      set x 10
      set y [expr $y +$maxY + 5]
      set maxY 0
    }
  }

  foreach i "$canv $selCanv $maskCanv $selMaskCanv" {
      bind $i <Double-Button-1> {
      %W coords aktuFrame 0 0 0 0
      switch %W { 
        .topIcon.frameLeft.frame0.canvas {
          recall setNoIcon
          recall setNoSelIcon
          .topIcon.frameLeft.frame0.canvas coords aktuFrame 0 0 0 0 
          .topIcon.frameRight.frame1.canvas coords aktuFrame 0 0 0 0 
          fillMaskCanvas .topIcon.frameLeft.framem0.canvas %W ""
          fillMaskCanvas .topIcon.frameRight.framem1.canvas %W ""
        } 
        .topIcon.frameRight.framem0.canvas {
          recall setNoMask
        } 
        .topIcon.frameLeft.framem1.canvas {
          recall setNoSelMask
        } 
        .topIcon.frameRight.frame1.canvas {
          recall setNoSelIcon
          .topIcon.frameRight.frame1.canvas coords aktuFrame 0 0 0 0 
          fillMaskCanvas .topIcon.frameRight.framem1.canvas %W ""
        }
      }
    }
  }

  adjWindow $canv 5
  adjWindow $selCanv 5
  
  set tmpFrame [$canv create rectangle 0 0 0 0 -outline blue]
  $canv addtag aktuFrame withtag $tmpFrame

  set tmpFrame [$selCanv create rectangle 0 0 0 0 -outline blue]
  $selCanv addtag aktuFrame withtag $tmpFrame

  set tmpFrame [$maskCanv create rectangle 0 0 0 0 -outline blue]
  $maskCanv addtag aktuFrame withtag $tmpFrame

  set tmpFrame [$selMaskCanv create rectangle 0 0 0 0 -outline blue]
  $selMaskCanv addtag aktuFrame withtag $tmpFrame

  adjWindow $maskCanv 5
  adjWindow $selMaskCanv 5

#     puts $aktuNormFrame
#     puts $aktuSelFrame 
        
}

# Procedure: setBoring
proc setBoring { c {bild "-1"}} {

}


# Procedure: unsetBoring
proc unsetBoring { c} {
}

return ok