###############################################################################
#
# node.tcl
# 
# 
# Prozeduren fuer die Behandlung von Knoten im Haupt-TCL-Fenster
#
###############################################################################
global nodeTypeName
set nodeTypeName ""
global edgeTypeName
set edgeTypeName ""

# Procedure: addNode
# erzeugt neuen Knoten im Canvas
# Parameter: Canvas, Position (x,y), Radius (fuer Circle-Darstellung), Farbe, Icons (fuer Icon-Darstellung)
proc addNode { canv x y r col {iconFile "__NoIcon__"} {selIconFile "__NoIcon__"} } {

  switch $iconFile {
      # circle
      __NoIcon__   { 
                    set cfTmpTag [addCircleNode $canv $x $y $r $col]
                   }
      # icon
      default { 
		global ICON_LIB_DIR
		global tk_version
		
		if { $tk_version < 4 } {
		    global icons
		    set bild [file rootname $iconFile]
		    
		    set cfTmpTag [$canv create bitmap $x $y \
				      -bitmap @$ICON_LIB_DIR/xpm/[file rootname $bild]N.xpm]
		    set icons($cfTmpTag) $bild
		} {
		    global selectImage
		    global unselectImage
		    global iconType    
		    global iconSelType    
		    
		    set tmpCreate [createIcon $iconFile $selIconFile]

		    if {$tmpCreate == ""} {
			return [addNode $canv $x $y $r $col]
		    }
		    
		    set cfTmpTag [$canv create image $x $y]

		    scan  $tmpCreate "%s %s %s %s" \
			unselectImage($cfTmpTag) \
			selectImage($cfTmpTag) iconType($cfTmpTag) \
			iconSelType($cfTmpTag)
		    
		    $canv itemconfigure $cfTmpTag -image $unselectImage($cfTmpTag)
		}
		$canv addtag iconNode withtag $cfTmpTag
                addNodeBindings $canv $cfTmpTag
              }
  }

  $canv addtag nodetag withtag $cfTmpTag

  return $cfTmpTag
}


# Procedure: addStdNode
# proc addStdNode { x y radius col {were "0"}} {
# global symbolicName
# return [addNode [set symbolicName(graphView)] $x $y $radius $col $were]
# }


# Procedure: moveStdNode
proc moveStdNode { c x y r} {
  global symbolicName
  moveNode $symbolicName(graphView) $c $x $y $r
}

# Procedure: moveNode
proc moveNode { canv c x y r} {
  if {[lsearch [$canv gettags $c] circleNode] != -1} {
    $canv coords $c [expr ($x-$r)]  [expr ($y+$r)] \
      [expr ($x+$r)] [expr ($y-$r)] 
      #puts "movenode: Circle"
    return
  }

  if {[lsearch [$canv gettags $c] classNode] != -1} {
    $canv coords $c $x $y
     # puts "movenode: Class"
    return
  }

  if {[lsearch [$canv gettags $c] packageNode] != -1} {
    $canv coords $c $x $y
      #puts "movenode: Package"
    return
  }

  if {[lsearch [$canv gettags $c] windowNode] != -1} {
    $canv coords $c $x $y
      #puts "movenode: Box"
    return
  }

  if {[lsearch [$canv gettags $c] dotNode] != -1} {
    $canv coords $c [expr ($x-2)]  [expr ($y+2)] \
      [expr ($x+2)] [expr ($y-2)] 
      #puts "movenode: Dot"
    return
  }

  if {[lsearch [$canv gettags $c] iconNode] != -1} {
    $canv coords $c $x $y
    return
  }

}

proc buildNodeMenu {} {
# wird noch irgendwo beim Prototyp-Start gebunden
# Aufruf auskommentiert
puts stderr "fixme: buildNodeMenu called"
}


# Procedure buildSpecificNodeMenu
proc buildSpecificNodeMenu {c} {

  # falls vorhanden, Menu zerstoeren - damit jeder Knotentyp sein eigenes erhaelt
  catch {destroy .frame0.canvas2.menu0 ; return 0 } error_code
  if { $error_code != 0 } then { 
      #puts "Menu nicht zerstoert!"
  }

  tixPopupMenu .frame0.canvas2.menu0 -height 1

  set m [.frame0.canvas2.menu0 subwidget menu]


    global nodeTypeName
    recall "getNodeTypeName"

    if { [string compare $nodeTypeName ""] != 0 } {
	$m add separator
	$m add command \
	    -label "NodeType $nodeTypeName" \
	    -state {disabled} 

	$m add separator
#@ vorerst disabled, da Settingsfenster ueberarbeitet wird
#@ muss nach Fertigstellung ueberarbeitet werden - RS
	$m add command \
	    -command "configureNode" \
	    -label {Configure NodeType} \
	    -state {disabled} 


	$m add cascade -label {Change NodeStyle} -menu $m.menu2
	menu $m.menu2 
	$m.menu2 add command -label "Label only" -command {selectNodeStyle LABEL}
	$m.menu2 add command -label Box          -command {selectNodeStyle BOX}
	$m.menu2 add command -label "Round Box"  -command {selectNodeStyle RBOX}
	$m.menu2 add command -label "Double Box" -command {selectNodeStyle DBOX}
	$m.menu2 add command -label "Frame Box"  -command {selectNodeStyle FBOX}
	$m.menu2 add command -label Circle  -command {selectNodeStyle CIRCLE}
	$m.menu2 add command -label Class   -command {selectNodeStyle CLASS}
	$m.menu2 add command -label Dot     -command {selectNodeStyle DOT}
	$m.menu2 add command -label Package -command {selectNodeStyle PACKAGE}
    } {
	$m add command \
	    -label "Nodes Menu" \
	    -state {disabled} 
    }

#@  $m add separator
  $m add command \
      -command { showAttrFromSel }\
      -label {Show Attributes}
#  $m add command \
#      -command "selectNode" \
#      -label {change style}

#@  $m add separator
#@  $m add command \
#@      -command "unAvailable" \
#@      -label {Select All} \
#@      -state {disabled} 
#@  $m add command \
#@      -command "unAvailable" \
#@      -label {Unselect All} \
#@      -state {disabled} 
}

proc selectStyle {style} {
    showMessage "you've chosen style $style"    
}

# Procedure: changeNodeColor
proc changeNodeColor { node col {canv "."}} {
    global symbolicName
    if { $canv == "."} {
        set canv $symbolicName(graphView)
    }
    if {[lsearch [.frame0.canvas2 gettags $node] classNode] != -1} {
        return
    }
    if {[lsearch [.frame0.canvas2 gettags $node] packageNode] != -1} {
        return
    }
    if {[lsearch [.frame0.canvas2 gettags $node] windowNode] != -1} {
        return
    }
    if {[lsearch [.frame0.canvas2 gettags $node] dotNode] != -1} {
        return
    }
    if {[lsearch [.frame0.canvas2 gettags $node] iconNode] != -1} {
	# $canv itemconfigure $node -foreground $col
    } {
        $canv itemconfigure $node -outline $col -fill $col
    }
}

# Procedure: unsetSelected
proc unsetSelected { c {canv "."} {col "black"} } {
	global symbolicName
	global typeArray
	global edgeTypeName
	global isNode

	if {$canv == "."} {
		set canv $symbolicName(graphView) 
	}
    
	if {[lsearch [$canv gettags $c] classNode] != -1} {
		# Farbe gemaess Definition in addClassWindow
		#[ansgar]
		#set col {gainsboro}
		set col [$canv cget -background]
 		$canv.$c configure -bg $col 
		#-bd {1}
		$canv.$c.canvas2.f1.canv3 configure -bg $col
		$canv.$c.canvas2.f1 configure -bg $col
		set col {lightgrey}
		$canv.$c.canvas2 configure -bg $col
		return
	}
    
	if {[lsearch [$canv gettags $c] packageNode] != -1} {
		# Farbe gemaess Definition in addPackageWindow
		set col #ebebeb
		$canv.$c configure -bg $col -bd {1}
		$canv.$c.canvas2 configure -bg $col
		$canv.$c.canvas2.f1 configure -bg $col
		set col {lightgrey}
		$canv.$c.canvas2.f1.nb subwidget page1 configure -bg $col
		#@ setzt aktuelles Canvas vom deselektierten WindowNode wieder zurueck
		#@ auf Hauptcanvas (sh. setSelected) - wird bei Subgraphen benoetigt
		#@ setAktuCanvas $symbolicName(graphView) 
    	return
    }
	
	if {[lsearch [$canv gettags $c] dotNode] != -1} {
		$canv itemconfigure $c -fill $col
		return
	}
    
	global CSets
	if {[lsearch [$canv gettags $c] windowNode] != -1} {
		if $CSets(used_for_FFSel) {
			unsetMarked $c $canv Unselected
			return
		} else {
			# Farbe gemaess Definition in addBoxWindow
			set col {lightgrey}
			$canv.$c configure -bg $col -bd {3}
			$canv.$c.canvas2 configure -bg $col
			$canv.$c.canvas2.f1 configure -bg $col
			catch { $canv.$c.canvas2.f1.l1 configure -bg $col ; return 0 } error_code
			#@ setzt aktuelles Canvas vom deselektierten WindowNode
			#@ wieder zurueck auf Hauptcanvas (s. proc SetSelected) -
			#@ wird bei Subgraphen benoetigt
			#@ setAktuCanvas $symbolicName(graphView)
			return
		}
	}
    
	if {[lsearch [$canv gettags $c] iconNode] != -1} {
		global tk_version
		if { $tk_version < 4 } {
			global icons
			global ICON_LIB_DIR
			set tmpMap $icons($c)N.xpm
			$canv itemconfigure $c  -bitmap @$ICON_LIB_DIR/$tmpMap
		} {
			global unselectImage
			$canv itemconfigure $c -image $unselectImage($c)
		}
	} {
		$canv itemconfigure $c -fill $col
	}

	if {[lsearch [$canv gettags $c] edgetag] != -1} {
		recall "getEdgeIsNode";
		if { $isNode == 0 } then {
			recall "getEdgeTypeName";
			# [ansgar]
			changeEdgeDisplay $c $edgeTypeName
		}
	} 
}

# Procedure: setSelected
proc setSelected { c {canv "."}} {
  global symbolicName

  if {$canv == "."} {
    set canv $symbolicName(graphView) 
  }

  if {[lsearch [$canv gettags $c] classNode] != -1} {
    $canv.$c.canvas2.f1.canv3 configure -bg {gold}

    return
  }
    
  if {[lsearch [$canv gettags $c] packageNode] != -1} {
    #$canv.$c configure -bg {red} -bd {1}
    #$canv.$c.canvas2.f1 configure -bg {gold}
    $canv.$c.canvas2.f1.nb subwidget page1 configure -bg {gold}

    #@ deaktiviert - setzt aktuelles Canvas auf selektierten WindowNode
    #@ Folge: Scrollbars werden entsprechend dem Window angepasst -
    #@ ist bei einigen Spezifikationen unerwuenscht  
    #@ z.B. ist lt. Andy Ausgabe einer Spezifikation eine Menge
    #@ selektierter Knoten ==> kein Scrolling im Canvas moeglich
    #@ wird jedoch fuer Scrollen bei Subgraphen in SubFenstern benoetigt
    #@    setAktuCanvas $canv.$c.canvas2 

    return
  }
    
  if {[lsearch [$canv gettags $c] dotNode] != -1} {
      $canv itemconfigure $c -fill {gold}
      return
  }
  
  global CSets
  if {[lsearch [$canv gettags $c] windowNode] != -1} {
      if $CSets(used_for_FFSel) {
	  setMarked $c $canv Selected
	  return
      } else  {
	  $canv.$c configure -bg {red} -bd {3}
	  $canv.$c.canvas2.f1 configure -bg {gold}
	  catch { $canv.$c.canvas2.f1.l1 configure -bg {gold} ; return 0 } error_code
	  #@ deaktiviert - setzt aktuelles Canvas auf selektierten
	  #@ WindowNode.  Folge: Scrollbars werden entsprechend dem
	  #@ Window angepasst - ist bei einigen Spezifikationen
	  #@ unerwuenscht, z.B. ist lt. Andy Ausgabe einer Spezifikation
	  #@ eine Menge selektierter Knoten ==> kein Scrolling im
	  #@ Canvas moeglich. Wird jedoch fuer Scrollen bei Subgraphen
	  #@ in SubFenstern benoetigt
	  #@ setAktuCanvas $canv.$c.canvas2

    return
      }
  }
  
  if {[lsearch [$canv gettags $c] iconNode] != -1} {
    global tk_version
    if { $tk_version < 4 } {
      global icons
      global ICON_LIB_DIR
      $canv itemconfigure $c  -bitmap @$ICON_LIB_DIR/xpm/$icons($c)S.xpm 
    } {
      global selectImage
      $canv itemconfigure $c -image $selectImage($c)
    }
      return
  }
  
  if {[lsearch [$canv gettags $c] edgetag] != -1} {
      $canv itemconfigure $c -fill {red}
  } {
    $canv itemconfigure $c -fill {gold}
  } 
}

#####
## general coloring procedure for windowNodes
## suitable for calls from FF setSelected(), proc changeNodeColor
## and proc setMarked which can pass more parameters from sel_tool

proc MarkWindowNode { c {canv "."} {bgcol {}} {bdcol {}} {bd {}} } {
    puts_debug "MarkWindowNode node $c, bgcol $bgcol, bdcol $bdcol, bd $bd\n"
    if {$canv == "."} {set canv [SN graphView]}

    if {[lsearch [$canv gettags $c] windowNode] != -1} {
	if {$bdcol != {}} {
	    $canv.$c configure -bg $bdcol
	}
	if {$bd != {}} {
	    $canv.$c configure -bd $bd
	}
	if {$bgcol != {}} {
	    $canv.$c.canvas2    configure -bg $bgcol
	    $canv.$c.canvas2.f1 configure -bg $bgcol
	    catch {
		$canv.$c.canvas2.f1.l1 configure -bg $bgcol
		return 0
	    } error_code
	}
	return 0
    } else {
	puts_debug "$c isn't a windowNode.\n"
	return 1
    }
}

# vgl. changeEdgeColor
proc MarkEdge { c {canv "."} {col {}} } {
    if {$canv == "."} {
        set canv [SN graphView]
    }
    if {$col != {}} {
	$canv itemconfigure $c -fill $col 
    } else {
	$canv itemconfigure $c -fill black ;# WRONG
	# use color from general settings
    }
}

proc setColorSchemes {{config {}}} {
    global mark
    global CSets

    # Colors from setSelected, unsetSelected (windowNodes)
    set mark(UnSelected,traditional) {lightgrey lightgrey}
    set mark(Selected,traditional)   {gold red}
    set mark(UnMarked,traditional)   {lightgrey lightgrey}

    # background colored by selections, border by ColorSets
    set mark(UnSelected,orthogonal)  {lightgrey}
    set mark(Selected,orthogonal)    {gold}
    set mark(UnMarked,orthogonal)    {{} lightgrey}

    # orthogonal with thick borders iff in ColorSet
    set mark(UnSelected,framed)  {lightgrey}
    set mark(Selected,framed)    {gold}
    set mark(UnMarked,framed)    {{} lightgrey 3}

    # background gold if somehow selected, thick colorborders,
    # no interoperation between old and new Selections
    set mark(UnSelected,QED)  {lightgrey lightgrey {}}
    set mark(Selected,QED)    {gold lightgrey {}}
    set mark(UnMarked,QED)    {lightgrey lightgrey 3}

    # no globally set colors yet

    set mark(UnSelected,edge) black
    set mark(Selected,edge) red
    set mark(UnMarked,edge) black

    foreach {set col} $config {
	set mark($set,traditional)  [list $col red]
	set mark($set,orthogonal)   [list {} $col]
	set mark($set,framed)       [list {} $col \$CSets(thickness)]
	set mark($set,QED)	    [list gold $col 5]
	set mark($set,edge)         $col
    }
}

# debug lines?
set CSets(verbose) 0
proc puts_debug {text} {
    global CSets
    if {$CSets(verbose)} {puts stderr $text}
}

# declare arrays
set mark() {}; unset mark()
set CSets() {}; unset CSets()

# ColorScheme "framed"
set CSets(thickness) 5 ;# to be tuned

# Set-Color association
set CSets(DefaultColors) {
    ColorSetA blue  ColorSetB red  ColorSetC seagreen
      BlueSet blue     RedSet red   GreenSet seagreen
}
proc resetColorSchemes {} {
    global CSets
    setColorSchemes $CSets(DefaultColors)
}
resetColorSchemes

# Use Colorset procs to set windowNode selected
set CSets(used_for_FFSel) 1

# maintenance menu entries
set CSets(checkbuttons) {
    "select by marking" CSets(used_for_FFSel)
}
set CSets(ColorSchemeList) {
    traditional traditional
    orthogonal orthogonal
    framed framed
    QED QED
}

# "orthogonal" Scheme allows to use both Selections and ColorSets

set CSets(ColorScheme) QED

set CSets(menubutton) 0 ;# no maintenance menu

proc CSets_menubutton {frame {name CSets}} {
    global CSets

    if {! $CSets(menubutton)} return 

    set mb $frame.menubutton_demo
    set m $mb.m

    menubutton $mb -text $name -menu $mb.m
    menu $m
    foreach {label var} $CSets(checkbuttons) {
	$m add check -label $label -variable $var
    }
    foreach {label val} $CSets(ColorSchemeList) {
	$m add radio -label $label -variable CSets(ColorScheme) -value $val
    }
    $m add separ
    $m add check -label {verbose tcl} -variable CSets(verbose)
    $m add comm -label {verbose C++  on} -command {selectiontool_debug debug 1}
    $m add comm -label {verbose C++ off} -command {selectiontool_debug debug 0}
    $m add comm -label {unsetColorSets} -command {selectiontool unsetColorSets}
    
    ## recoloring only necessary if menu is defined after window.tcl
    #  ffSetColorsForWin $mb       
    #  $mb configure -background \#ebebeb
    pack $mb -side right
}

# usual coloring coloring for graphic elements
proc setSelectedVariant {c {canv "."}} {
    global mark
    global CSets
    if {[lsearch [$canv gettags $c] windowNode] != -1} {
	eval {MarkWindowNode $c $canv} $mark(Selected,$CSets(ColorScheme))
    } else {
	MarkEdge $c $canv $mark(Selected,edge)
    }
}
proc unsetSelectedVariant {c {canv "."}} {
    global mark
    global CSets
    if {[lsearch [$canv gettags $c] windowNode] != -1} {
	eval {MarkWindowNode $c $canv} $mark(UnSelected,$CSets(ColorScheme))
    } else {
	MarkEdge $c $canv {}
    }
}

# mark graphic elements in set "setname" as defined in current ColorScheme
# or graphic elements in old variabletool Selection lists ("setSelected")
proc setMarked {c {canv .} {setname Selected}} {
    global mark
    global CSets
    
    if {$canv == "."} {
        set canv [SN graphView]
    }
    
    if {[lsearch [$canv gettags $c] windowNode] != -1} {
	# build command, split parameter list from mark() into parameters
	eval {MarkWindowNode $c $canv} $mark($setname,$CSets(ColorScheme))
    } else {
	MarkEdge $c $canv $mark($setname,edge)
    }
}
proc unsetMarked {c {canv .} {setname Unselected}} {
    # graphic element c must be in set setname or be selected by setSelected
    global mark
    global CSets

    if {$canv == "."} {
        set canv [SN graphView]
    }
    
    if {[lsearch [$canv gettags $c] windowNode] != -1} {
	eval {MarkWindowNode $c $canv} $mark(UnMarked,$CSets(ColorScheme))
    } else {
	MarkEdge $c $canv {}
    }
}

#####

proc unAvailable {} {
    showMessage "under construction"
}

proc configureNode {} {
    global nodeTypeName
    global typeArray
    global actualType

    set actualType $nodeTypeName
    nodeSettings
}

proc configureEdgeNode {} {
    global nodeTypeName
    global typeArray
    global actualType
    recall "getEdgeNodeTypeName"

    if { [string compare $nodeTypeName ""] != 0 } {
	set actualType $nodeTypeName
	nodeSettings
    } {
	showMessage " Too many nodes selected ! "
    }
}

proc getNodeInfo {} {
    global nodeTypeName
    global typeArray
    global actualType
    recall "getNodeTypeName"

    if { [string compare $nodeTypeName ""] != 0 } {
	.statusbar.status1 configure -text "NodeType: $nodeTypeName     Style: $typeArray($nodeTypeName,STYLE)"
    } {
	.statusbar.status1 configure -text "Too many nodes selected! For node information please select only one node."
    }
}

proc selectNode {} {
    global nodeTypeName
    global typeArray
    global actualType
    recall "getNodeTypeName"
#	showMessage "NodeType: $nodeTypeName Style: $typeArray($nodeTypeName,STYLE)"

    if { [string compare $nodeTypeName ""] != 0 } {
	set actualType $nodeTypeName
	nodeStyleWin
    } {
	showMessage " Too many nodes selected ! "
    }
}

proc selectNodeStyle {s} {
    #--------------------------------------------------------------
    # entsprechend ausgewaehltem Style Optionen anbieten (s.dort)
    #--------------------------------------------------------------
    global typeArray
    global actualType
    global nodeTypeName

    recall "getNodeTypeName"

    if { [string compare $nodeTypeName ""] != 0 } {
	set actualType $nodeTypeName
	set typeArray($actualType,STYLE) $s
	saveSettings APPLY
    } {
	showMessage " More than one node selected ! "
    }
}

# Procedure addNodeBindings
# Ereignisbehandlung fuer Knoten (Circle, Box etc.)
proc addNodeBindings {canv cfTmpTag} {
    global symbolicName
    if { $symbolicName(graphView) == $canv} {
	set befehl recall
    } {
	set befehl SubRecall.$canv
    }        
  
    # Knoten selektieren und Popup-Menu anzeigen
    $canv bind $cfTmpTag      <Button-3> "recall setCurrentNode $cfTmpTag %x %y; buildSpecificNodeMenu $cfTmpTag; .frame0.canvas2.menu0 post .frame0.canvas2 %x %y"
  
    # Knoten selektieren und Info in Statusbar anzeigen
    $canv bind $cfTmpTag      <Shift-Button-3> "recall setCurrentNode $cfTmpTag %x %y; getNodeInfo"

    $canv bind $cfTmpTag <ButtonRelease-3> {
#	.frame0.canvas2.menu0 invoke active; 
#	.frame0.canvas2.menu0 unpost;
	.statusbar.status1 configure -text " "
    }

  # Knoten selektieren
    set body { setCurrentNode [%W find withtag current] [%W canvasx %x] [%W canvasy %y] }
    $canv bind $cfTmpTag <Button-1> "$befehl $body "    
  $canv bind $cfTmpTag <Shift-Button-1> "break"    
  
  # selektierte Knoten bewegen
  global tt
  set body {moveCurrent [%W find withtag current] [%W canvasx %x] \
		[%W canvasy %y]; set tt [%W find withtag current] }
  $canv bind $cfTmpTag <B1-Motion> "$befehl $body"
  
  # reine FFGRAPH-Funktionen - entfallen
  # neue Kante erzeugen
  #set body  {newEdge [%W find withtag current] [%W canvasx %x] \
               [%W canvasy %y]}  
  #$canv bind $cfTmpTag <Control-Button-1> "$befehl $body$"  
  $canv bind $cfTmpTag <Control-B1-Motion> break;
  # Knoten loeschen  
  #set body2 {removeNode [%W find withtag current]}
  #$canv bind $cfTmpTag <Button-2> "$befehl $body2"
  
  #Knoten selektieren und Inspector aufrufen
  set body1 {setCurrentNode [%W find withtag current] %x %y;}
  set body2 {modPress i}
  $canv bind $cfTmpTag <M1-Button-1> "$befehl $body1; $befehl $body2"
}

# Parameters of node binding install procedures:

#      item - widget whose events will be bound to these standard handlers
#             not an item; to be renamed 
#  cfTmpTag - tag of canvas item (windownode) associated with the widget
#             Carsten Friedrich Tmp Tag ?
#        cN - useless, will be removed until 31.01.1999 -kmf

# - only .frame0.canvas2 is supported as parameter canv isn't used * FIX ME *

# - currentNode is a global variable set and used during event handling time
#   it is not used during calls to the node binding install procedure
#   its value is not accessed from the C++ side
#   its value is not used after processing the event handler code

# - Mouse events to be bound to specific node widgets. Named after their
#   meaning in some stable release. Use bind-advise.tcl to change bindings.

proc node_B1_press {item cfTmpTag {cN {don't care}}} {

    # Selektierung

  # CurrentNode enthaelt zur Laufzeit den aktuellen Knoten (= Nummer), der hier
  # mit $cfTmpTag als Konstante eingesetzt wird. Die Variable currentNode wird
  # nicht in Seiteneffekten genutzt. Die Verbindung zu FFGraph erfolgt mit
  # proc setCurrentNode. Unbekannte Items werden munter als *(NULL) aufgefasst.
  # Die Event-Koordinaten muessen jedoch beim Prozedurdurchlauf gesetzt werden
  # beachte: die absoluten Screen-Koordinaten %X und %Y erlauben, Gesten im
  # Canvas zu starten und ueber die Fenstergrenzen hinaus fortzusetzen.
  # Im Gegensatz dazu beim Button-3 Event: dort wird alles zur Laufzeit ermittelt
  # beachte hier: die Canvas-Koordinaten %x und %y (Kleinschreibung!)
  # Die Funktionalitaet ist ansonsten die gleiche

  set body { setCurrentNode $currentNode [.frame0.canvas2 canvasx %X] [.frame0.canvas2 canvasy %Y] }
  set body2 { moveCurrent [.frame0.canvas2 find withtag current] [.frame0.canvas2 canvasx %X] [.frame0.canvas2 canvasy %Y] }
  bind $item <Button-1> "set currentNode $cfTmpTag ; recall $body ; recall $body2 ; unset currentNode"
}

proc node_B1_shift_press {item cfTmpTag {cN {don't care}}} {
    set body { selectIfNot $currentNode [.frame0.canvas2 canvasx %X] [.frame0.canvas2 canvasy %Y] }
    bind $item <Shift-Button-1> "set currentNode $cfTmpTag ; recall $body; unset currentNode"  
}

proc node_B1_motion {item {cfTmpTag {}}} {
    set body { moveCurrent [.frame0.canvas2 find withtag current] [.frame0.canvas2 canvasx %X] [.frame0.canvas2 canvasy %Y] }
    bind $item <B1-Motion> "recall $body"
}

proc node_B1_release {item {cfTmpTag {}}} {
    bind $item <ButtonRelease-1> {
	.statusbar.status1 configure -text " "
    }
}

proc node_B3_press {item cfTmpTag} {
    bind $item <Button-3> "recall setCurrentNode $cfTmpTag %x %y; buildSpecificNodeMenu $cfTmpTag; .frame0.canvas2.menu0 post .frame0.canvas2.$cfTmpTag %x %y"
}

proc node_B3_shift_press {item cfTmpTag} {
    bind $item <Shift-Button-3> "recall setCurrentNode $cfTmpTag %x %y; getNodeInfo"
}

proc node_B3_motion {item cfTmpTag} {
# unnoetig
#    bind $item <B3-Motion> {
#	catch {	MenuPopupHandle .frame0.canvas2.menu0 .frame0.canvas2.$cfTmpTag %X %Y; return 0 } error_code
#	if { $error_code != 0 } then { 
#	    puts "error in MenuPopHandle (node_B3_motion)"
#	}
#    }
}

proc node_B3_release {item {cfTmpTag {}}} {
    bind $item <ButtonRelease-3> {
	.statusbar.status1 configure -text " "
    }
}

return ok
