###############################################################################
#
# setWindow.tcl
# 
# 
# Erzeugung des Notebooks fuer Global Settings
#
# Die Inhalte der einzelnen Tabs werden in createPage... definiert
# Jede Definition steht zwecks Uebersichtlichkeit mit den dazugehoerigen 
# Prozeduren in einem eigene Skript.
#
# setVisibilities.tcl
# setNode.tcl
# setEdgeType.tcl
# setLayout.tcl
#
###############################################################################


proc createNoteBook {w} {
    global typeArray
    global typeList
    global actualType
    global nodeStyle
    global h1
    global h2
    global c1
    global c2
    global c3
    global visi_value
    global main_window
    global nodeLabel
    global edgelabel
    global iconposition
    global win_path_for_types
    global win_path_for_tree

#### noteBook #####################################################
#                                                                 #
    set name [tixOptionName $w]
    option add *$name*TixControl*entry.width 10
    option add *$name*TixControl*label.width 18
    option add *$name*TixControl*label.anchor e

    tixNoteBook $w.nb -ipadx 6 -ipady 6
    $w config -bg gray
    $w.nb subwidget nbframe config -backpagecolor gray

    $w.nb add nodes      -label "Type Settings"     -underline 0
    $w.nb add visinodes  -label "Type Visibilities" -underline 5
    $w.nb add layout     -label "Layout Settings"   -underline 0

    pack $w.nb -expand yes -fill both -padx 5 -pady 5 -side top
    
#                                                                 #
#### noteBook #####################################################

#### Use a ButtonBox to hold the buttons ##########################
    create_ButtonBox

#### pages ########################################################
#                                                                 #

    #--------------------------------------------------------------
    # Create page Visibilities                                    =
    #--------------------------------------------------------------
    createPage_Visibilities

    #--------------------------------------------------------------
    # Create page Nodes                                           =
    #--------------------------------------------------------------
    createPage_Nodes

    #--------------------------------------------------------------
    # Create page Layout                                          =
    #--------------------------------------------------------------
    createPage_Layout


}

proc create_ButtonBox {} {
   # Use a ButtonBox to hold the buttons.
    global main_window
    global typeArray
    global win_path_for_types

    tixButtonBox $main_window.box -orientation horizontal

    if { [string compare $main_window ".settings"] == 0 } {
	# Aktionen fuer settings-Fenster
	$main_window.box add ok -text Ok -command "saveSettings OK_LAYOUT" \
	    -width 10 -background grey -activebackground green
	$main_window.box add apply -text Apply -command "saveSettings APPLY_LAYOUT" \
	    -width 10 -background grey -activebackground yellow
    } {
	# Aktionen fuer nodeStyle-und nodeSettings-Fenster
	$main_window.box add ok -text Ok -command "saveSettings OK" \
	    -width 10 -background grey -activebackground green
	$main_window.box add apply -text Apply -command "saveSettings APPLY" \
	    -width 10 -background grey -activebackground yellow
    }
    $main_window.box add cancel -text Cancel -command "saveSettings CANCEL" \
	-width 10 -background grey -activebackground red
    $main_window.box add help -text Help -command "helpSettings" \
	-width 10 -background grey -activebackground lightblue
    pack $main_window.box -side bottom -fill x

    # put some little help to the buttons
    tixBalloon $main_window.balloon
    $main_window.balloon bind $main_window.box.ok -balloonmsg "Close window and save settings"
    $main_window.balloon bind $main_window.box.apply -balloonmsg "Save settings"
    $main_window.balloon bind $main_window.box.cancel -balloonmsg "Close window and discard changes"
    $main_window.balloon bind $main_window.box.help -balloonmsg "Get some information about the settings window"
}

proc helpSettings {} {
    toplevel .topA 

    # Window manager configurations
    global tk_version
    wm positionfrom .topA ""
    wm sizefrom .topA ""
    wm maxsize .topA 1000 1000
    wm minsize .topA 10 10
    showHelpWindow {.topA}
}

proc saveSettings {status} {
    # allgemeine Routine zum Speichern der Settings
    # wird neben dem Settings-Fenster auch im Popup-Menu verwendet!!!
    # gueltige Werte fuer Status: (s.o. create_ButtonBox)
    # CANCEL:       kein Speichern, aber zugehoeriges Fenster zerstoeren
    # APPLY:        nur Speichern, Fenster (soweit vorhanden) wird beibehalten
    # OK:           Speichern + zugehoeriges Fenster zerstoeren
    # APPLY_LAYOUT: Zusatzinfos zu Layout updaten + Speichern
    # OK_LAYOUT:    Zusatzinfos zu Layout updaten + Speichern + Fenster zerstoeren

    global main_window
    global typeArray
    global visi_value
    # !! wichtig !! actualType wird in C-Prozedur (s. config2CPP) benoetigt
    global actualType    

    setBusy
    if {[string compare $status "CANCEL"] == 0} {
#	set typeArray(RESULT) CANCEL
#	puts "SaveSettings: MainWindow=$main_window"
	destroy $main_window
    } else {
	if {[string compare $status "OK"] == 0 || [string compare $status "OK_LAYOUT"] == 0} {
	    destroy $main_window
	}
	if {[string compare $status "APPLY_LAYOUT"] == 0 || [string compare $status "OK_LAYOUT"] == 0} {
	    dynamite:update dummy
	    sugiyama:update dummy
	    set typeArray(VISIATTRVALUE) $visi_value
	}
	TCL_config2CPP
    }
    unsetBusy

}
 
# Procedure: showHelpWindow
proc showHelpWindow {wInfo} {
    global txtPath
    wm title $wInfo {Settings Help: }

    set textFile $txtPath/settings.txt
    if {[catch "open $textFile r" textInFile]} {
        puts stderr "$textInFile"
        set infoMessage $textInFile
    } else {
        set infoMessage [read $textInFile]
        close $textInFile
    }

    frame $wInfo.bottom
    button $wInfo.bottom.ok -text " Dismiss " -command "destroy $wInfo"
    pack $wInfo.bottom.ok -side left -expand 1 -padx 30

    frame $wInfo.textview
    scrollbar $wInfo.textview.xscroll -command "$wInfo.textview.text xview" -orient horizontal
    scrollbar $wInfo.textview.yscroll -command "$wInfo.textview.text yview"
    text $wInfo.textview.text -relief sunken -bd 2 -xscroll "$wInfo.textview.xscroll set" -yscroll "$wInfo.textview.yscroll set" -wrap none

    pack $wInfo.textview.xscroll -side bottom -fill x
    pack $wInfo.textview.text -side left -fill y 
    pack $wInfo.textview.yscroll -side right -fill y 
    
    pack $wInfo.bottom -side bottom -padx 10 -pady 10
    pack $wInfo.textview -side left -fill y 

    $wInfo.textview.text insert end $infoMessage

    centerWindow $wInfo
}


return ok