#
# fileselect.tcl --
# simple file selector.
#
# Mario Jorge Silva			          msilva@cs.Berkeley.EDU
# University of California Berkeley                 Ph:    +1(510)642-8248
# Computer Science Division, 571 Evans Hall         Fax:   +1(510)642-5775
# Berkeley CA 94720                                 
# 
# Layout:
#
#  file:                  +----+
#  ____________________   | OK |
#                         +----+
#
#  +------------------+    Cancel
#  | ..               |S
#  | file1            |c
#  | file2            |r
#  |                  |b
#  | filen            |a
#  |                  |r
#  +------------------+
#  currrent-directory
#
# Copyright 1993 Regents of the University of California
# Permission to use, copy, modify, and distribute this
# software and its documentation for any purpose and without
# fee is hereby granted, provided that this copyright
# notice appears in all copies.  The University of California
# makes no representations about the suitability of this
# software for any purpose.  It is provided "as is" without
# express or implied warranty.
#


# names starting with "fileselect" are reserved by this module
# no other names used.

# use the "option" command for further configuration

option add *Listbox*font \
    "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1" startupFile
option add *Entry*font \
    "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1" startupFile
option add *Label*font \
    "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1" startupFile



# this is the default proc  called when "OK" is pressed
# to indicate yours, give it as the first arg to "fileselect"

proc fileselect.default.cmd {f} {
    global selectedfile
    # modified: sets global variable to selected file name
    set selectedfile $f
}


# this is the default proc called when error is detected
# indicate your own pro as an argument to fileselect

proc fileselect.default.errorHandler {errorMessage} {
    puts stdout "error: $errorMessage"
    catch { cd ~ }
}

# this is the proc that creates the file selector box

proc fileselect {
    {cmd fileselect.default.cmd} 
    {purpose "Open:"} 
    {w .fileSelectWindow} 
    {errorHandler fileselect.default.errorHandler}} {

    catch {destroy $w}

    toplevel $w
    grab $w
    wm title $w "Select File"


    # path independent names for the widgets
    global fileselect

    set fileselect(entry) $w.file.eframe.entry
    set fileselect(list) $w.file.sframe.list
    set fileselect(scroll) $w.file.sframe.scroll
    set fileselect(ok) $w.bframe.okframe.ok
    set fileselect(cancel) $w.bframe.cancel
    set fileselect(dirlabel) $w.file.dirlabel

    # widgets
    frame $w.file -bd 10 
    frame $w.bframe -bd 10
    pack append $w \
        $w.file {left filly} \
        $w.bframe {left expand frame n}

    frame $w.file.eframe
    frame $w.file.sframe
    label $w.file.dirlabel -anchor e -width 24 -text [pwd] 

    pack append $w.file \
        $w.file.eframe {top frame w} \
	$w.file.sframe {top fillx} \
	$w.file.dirlabel {top frame w}


    label $w.file.eframe.label -anchor w -width 24 -text $purpose
    entry $w.file.eframe.entry -relief sunken 

    pack append $w.file.eframe \
		$w.file.eframe.label {top expand frame w} \
                $w.file.eframe.entry {top fillx frame w} 


    scrollbar $w.file.sframe.yscroll -relief sunken \
	 -command "$w.file.sframe.list yview"
    listbox $w.file.sframe.list -relief sunken \
	-yscroll "$w.file.sframe.yscroll set" 

    pack append $w.file.sframe \
        $w.file.sframe.yscroll {right filly} \
 	$w.file.sframe.list {left expand fill} 

    # buttons
    frame $w.bframe.okframe -borderwidth 2 -relief sunken
 
    button $w.bframe.okframe.ok -text OK -relief raised -padx 10 \
        -command "fileselect.ok.cmd $w $cmd $errorHandler"

    button $w.bframe.cancel -text cancel -relief raised -padx 10 \
        -command "fileselect.cancel.cmd $w"
    pack append $w.bframe.okframe $w.bframe.okframe.ok {padx 10 pady 10}

    pack append $w.bframe $w.bframe.okframe {expand padx 20 pady 20}\
                          $w.bframe.cancel {top}

    # Fill the listbox with a list of the files in the directory (run
    # the "/bin/ls" command to get that information).
    # to not display the "." files, remove the -a option and fileselect
    # will still work
 
    $fileselect(list) insert end ".."
    foreach i [exec /bin/ls -a [pwd]] {
        if {[string compare $i "."] != 0 && \
	    [string compare $i ".."] != 0 } {
            $fileselect(list) insert end $i
        }
    }

   # Set up bindings for the browser.
    bind $fileselect(entry) <Return> {eval $fileselect(ok) invoke}
    bind $fileselect(entry) <Control-c> {eval $fileselect(cancel) invoke}

    bind $w <Control-c> {eval $fileselect(cancel) invoke}
    bind $w <Return> {eval $fileselect(ok) invoke}


    tk_listboxSingleSelect $fileselect(list)


    bind $fileselect(list) <Button-1> {
        # puts stderr "button 1 release"
        %W select from [%W nearest %y]
	update
	$fileselect(entry) delete 0 end
	$fileselect(entry) insert 0 [%W get [%W nearest %y]]
    }

    bind $fileselect(list) <Key> {
        %W select from [%W nearest %y]
	update
        $fileselect(entry) delete 0 end
	$fileselect(entry) insert 0 [%W get [%W nearest %y]]
    }

    bind $fileselect(list) <Double-ButtonPress-1> {
        # puts stderr "double button 1"
        %W select from [%W nearest %y]
	update
	$fileselect(entry) delete 0 end
	$fileselect(entry) insert 0 [%W get [%W nearest %y]]
	$fileselect(ok) invoke
    }

    bind $fileselect(list) <Return> {
        %W select from [%W nearest %y]
	update
	$fileselect(entry) delete 0 end
	$fileselect(entry) insert 0 [%W get [%W nearest %y]]
	$fileselect(ok) invoke
    }

    # set kbd focus to entry widget

    focus $fileselect(entry)

}


# auxiliary button procedures

proc fileselect.cancel.cmd {w} {
    global selectedfile
    # modified: sets global variable to empty string
    set selectedfile ""
    # puts stderr "Cancel"
    update
    destroy $w
}

proc fileselect.ok.cmd {w cmd errorHandler} {
    global fileselect
    set selected [$fileselect(entry) get]

    # some nasty file names may cause "file isdirectory" to return an error
    set sts [catch { 
	file isdirectory $selected
    }  errorMessage ]

    if { $sts != 0 } then {
	$errorHandler $errorMessage
	update
	destroy $w
	return

    }

    # clean the text entry and prepare the list
    update
    $fileselect(entry) delete 0 end
    $fileselect(list) delete 0 end
    $fileselect(list) insert end ".."

    # perform globbing on the selection. 
    # If globing returns an error, return (leaving the file listbox empty)
    # If resulting list length > 1, put the list on the file listbox and return
    # If globing expands to a list of filenames in multiple directories,
    # the indicated regexp is invalid and the error handler is called instead.

#    set sts [catch {
#	set globlist [glob [list $selected]]
#    } errorMessage ]
#
#    if { $sts != 0 } then {
#	return
#    }
#
#    if {[llength $globlist] > 1} {
#	if {[regexp "/" $globlist] != 0} {
#	    $errorHandler [list "Invalid regular expression, " $selected, "."]
#	    destroy $w
#	    return
#	}
#	foreach i $globlist {
#	    if {[string compare $i "."] != 0 && \
#		[string compare $i ".."] != 0} {
#		$fileselect(list) insert end $i
#	    }
#	}
#	return
#    }

    # selection may be a directory. Expand it.

    if {[file isdirectory $selected] != 0} {
	cd $selected
	set dir [pwd]
	$fileselect(dirlabel) configure -text $dir

	foreach i [exec /bin/ls -a $dir] {
	    if {[string compare $i "."] != 0 && \
		[string compare $i ".."] != 0} {
		$fileselect(list) insert end $i
	    }
	}
	return
    }
    update
    destroy $w
    $cmd $selected
}


proc dialog {dlgname text dlgtitle props {checks ""}} {
    #
    # only called from new command pdialog
    # if props has the value propsButton show a properties - button 
    #
    global globvar entrywidth tablist knopf dlgnametest
    
    # Fenster oeffnen & in 2 Haelften unterteilen
    
    toplevel .dlg -class Dialog
    wm title .dlg $dlgtitle
    wm iconname .dlg Dialog
    
    # obere Haelfte in links und rechts unterteilen
    if {[string length $text] != 0} {
	frame .dlg.top -relief flat
	frame .dlg.top.left -relief flat
	frame .dlg.top.right -relief flat
    }
    
    frame .dlg.bot -relief flat
    pack .dlg.bot -side bottom -fill both
    
    # Aufbau der Eingabefelder nur, falls 
    # der Parameter text nicht leer ist.
    
    set i 0
    set tablist {}
    
    if { [string length $text] != 0} {
	
	foreach te $text {
	    # display te
	    message .dlg.top.left.msg$i -width 4i -text $te 
	    pack .dlg.top.left.msg$i -in .dlg.top.left -side top -expand 1 -fill x -padx 5 -pady 3
	    
	    # create entries
	    entry .dlg.top.right.en$i -width $entrywidth -relief sunken -textvariable $dlgname$i
	    
	    # bindings for cursor and emacs-like keys
	    lappend tablist ".dlg.top.right.en$i"
	    Entrybinding ".dlg.top.right.en$i"
	    if {[string length $checks] != 0} {
		bind .dlg.top.right.en$i <Return> {.dlg.bot.def.ok flash;  if {[string compare [CloseDialog pdialog checkdialog $dlgnametest] "ok"] == 0} { set knopf "ok"}}
	    } else {
		bind .dlg.top.right.en$i <Return> {.dlg.bot.def.ok flash; set knopf "ok"}
	    }

	    pack .dlg.top.right.en$i -in .dlg.top.right -side top -expand 1 -fill x -padx 5 -pady 3
	    
	    incr i
	}
    }

    if {[string length $text] != 0} {
	pack .dlg.top.left -in .dlg.top -side left -fill y
	pack .dlg.top.right -in .dlg.top -side right -fill y
	pack .dlg.top -side top -expand 1 -fill both 
    }
    
    
    # 2 (evtl. 3) Buttons anzeigen
    button .dlg.bot.cancel -text "Cancel" -command {set knopf "Abbruch"}
    pack .dlg.bot.cancel -side left -expand 1 -padx 5m -pady 5m -ipadx 2m -ipady 1m
    
    if {[string compare propsButton $props] == 0} {
	button .dlg.bot.props -text "Properties..." -command {set knopf "Properties"}
	pack .dlg.bot.props -side left -expand 1 -padx 2m -pady 1m -ipadx 3m -ipady 1m 
    }
    
    frame .dlg.bot.def -relief sunken -bd 1
    set dlgnametest $dlgname
    if {[string length $checks] != 0} { 
	button .dlg.bot.def.ok -text "OK" -command {.dlg.bot.def.ok flash; if {[string compare [CloseDialog pdialog checkdialog $dlgnametest] "ok"] == 0} { set knopf "ok"}} 
    } else {
	button .dlg.bot.def.ok -text "OK" -command {.dlg.bot.def.ok flash; set knopf "ok"} 
    }
    pack .dlg.bot.def.ok -in .dlg.bot.def -side left -expand 1 -padx 2m -pady 1m -ipadx 3m -ipady 1m 
    pack .dlg.bot.def -side left -expand 1 -padx 1m; # -padx 1m -pady 1m
    
    
    # setze grab & focus
    set oldfocus [focus]
    # grab .dlg
    if {[string length $text] != 0} {
	focus .dlg.top.right.en0
    } else {
	if {[string compare properties $props] == 0} {
	    focus .dlg.middle.right.en0
	} else {
	    focus .dlg.bot.def.ok
	}
    }
    
    
    # Warte auf Ergebnis
    tkwait variable knopf
    update
    destroy .dlg
    focus $oldfocus
    #JK:puts stdout $knopf
    return $knopf
}



proc propertiesdialog {dlgname text dlgtitle} {
    #
    # only called from new command pdialog
    # shows properties 
    #
    global globvar entrywidth tablist knopf dlgnametest

    # Fenster oeffnen & in 2 Haelften unterteilen
    
    toplevel .dlg -class Dialog
    wm title .dlg $dlgtitle
    wm iconname .dlg Dialog

    set dlgnametest $dlgname
    
    # obere Haelfte in links und rechts unterteilen
    if {[string length $text] != 0} {
	frame .dlg.para -relief flat
	frame .dlg.para.left -relief flat
	frame .dlg.para.right -relief flat

	frame .dlg.parasep -relief sunken -borderwidth 2
	frame .dlg.parasep.left -relief sunken -borderwidth 2
	frame .dlg.parasep.right -relief sunken -borderwidth 2
    }
    
    
#@    frame .dlg.menu -relief flat
#@    frame .dlg.menu.left -relief flat
#@    frame .dlg.menu.right -relief flat
    
#@    frame .dlg.menusep -relief sunken -borderwidth 2
#@    frame .dlg.menusep.left -relief sunken -borderwidth 2
#@    frame .dlg.menusep.right -relief sunken -borderwidth 2

    frame .dlg.oper -relief flat
    frame .dlg.oper.left -relief flat
    frame .dlg.oper.right -relief flat
    
    # create titles
    if {[string length $text] != 0} {
	message .dlg.para.msg -width 5i -text "Parameter Properties"
	pack .dlg.para.msg -fill x
    }
    
#@    message .dlg.menu.msg -width 5i -text "Menu Properties"
#@    pack .dlg.menu.msg -fill x

    message .dlg.oper.msg -width 5i -text "Operation Properties"
    pack .dlg.oper.msg -fill x
    
    frame .dlg.bot -relief flat
    pack .dlg.bot -side bottom -fill both
    
    # Aufbau der Eingabefelder nur, falls 
    # der Parameter text nicht leer ist.
    
    set i 0
    set tablist {}
    
    if { [string length $text] != 0} {
	
	foreach te $text {
	    # display te
	    message .dlg.para.left.msg$i -width 4i -text $te 
	    pack .dlg.para.left.msg$i -in .dlg.para.left -side top -expand 1 -fill x -padx 5 -pady 3
	    
	    # create entries
	    entry .dlg.para.right.en$i -width $entrywidth -relief sunken -textvariable props$dlgname$i
	    
	    # bindings for cursor and emacs-like keys
	    lappend tablist ".dlg.para.right.en$i"
	    Entrybinding ".dlg.para.right.en$i"
	    bind .dlg.para.right.en$i <Return> {.dlg.bot.def.ok flash;  \
		    if {[string compare [CheckPropsDialog $dlgnametest] "ok"] ==0} {set knopf "ok"}}

	    pack .dlg.para.right.en$i -in .dlg.para.right -side top -expand 1 -fill x -padx 5 -pady 3
	    
	    incr i
	}
    }

    # Show fields for menuproperties 
#@    set j 0
#@    foreach te { "Toplevel" "Name" "Position"} {
#@	message .dlg.menu.left.msg$i -width 4i -text $te
#@	pack .dlg.menu.left.msg$i -in .dlg.menu.left -side top -expand 1 -fill x -padx 5 -pady 3
	
#@	entry .dlg.menu.right.en$i -width $entrywidth -relief sunken -textvariable propsmenu$dlgname$j
	
#@	puts stdout propsmenu$dlgname$j

	# bindings for cursor and emacs-like keys
#@	lappend tablist ".dlg.menu.right.en$i"
#@	Entrybinding ".dlg.menu.right.en$i"
#@	bind .dlg.menu.right.en$i <Return> {.dlg.bot.def.ok flash;  \
#@		if {[string compare [CheckPropsDialog $dlgnametest] "ok"] ==0} {set knopf "ok"}}

#@	pack .dlg.menu.right.en$i -in .dlg.menu.right -side top -expand 1 -fill x -padx 5 -pady 3
	
#@	incr i
#@	incr j
#@    }
    set j 2
    set propsmenu$dlgname$j 1

    #Show fields for operation properties
    set j 0
    message .dlg.oper.left.msg$i -width 4i -text "Priority"
    pack .dlg.oper.left.msg$i -in .dlg.oper.left -side top -expand 1 -fill x -padx 5 -pady 3
    
    entry .dlg.oper.right.en$i -width $entrywidth -relief sunken -textvariable propsoper$dlgname$j
    
    # bindings for cursor and emacs-like keys
    lappend tablist ".dlg.oper.right.en$i"
    Entrybinding ".dlg.oper.right.en$i"
    bind .dlg.oper.right.en$i <Return> {.dlg.bot.def.ok flash;  \
	    if {[string compare [CheckPropsDialog $dlgnametest] "ok"] ==0} {set knopf "ok"}}
    
    pack .dlg.oper.right.en$i -in .dlg.oper.right -side top -expand 1 -fill x -padx 5 -pady 3


    
    if {[string length $text] != 0} {
	pack .dlg.para.left -in .dlg.para -side left -fill y
	pack .dlg.para.right -in .dlg.para -side right -fill y
	pack .dlg.para -side top -expand 1 -fill both 

	pack .dlg.parasep.left -in .dlg.parasep -side left -fill y
	pack .dlg.parasep.right -in .dlg.parasep -side right -fill y
	pack .dlg.parasep -side top -expand 1 -fill both 
    }
    
#@    pack .dlg.menu.left -in .dlg.menu -side left -fill x 
#@    pack .dlg.menu.right -in .dlg.menu -side left -fill x 
#@    pack .dlg.menu -side top -expand 1 -fill both 

#@    pack .dlg.menusep.left -in .dlg.menusep -side left -fill y
#@    pack .dlg.menusep.right -in .dlg.menusep -side right -fill y
#@    pack .dlg.menusep -side top -expand 1 -fill both 
    
    pack .dlg.oper.left -in .dlg.oper -side left -fill x 
    pack .dlg.oper.right -in .dlg.oper -side left -fill x 
    pack .dlg.oper -side top -expand 1 -fill both 

    # 2 (evtl. 3) Buttons anzeigen
    button .dlg.bot.cancel -text "Cancel" -command {set knopf "Abbruch"}
    pack .dlg.bot.cancel -side left -expand 1 -padx 5m -pady 5m -ipadx 2m -ipady 1m
    
    frame .dlg.bot.def -relief sunken -bd 1
    button .dlg.bot.def.ok -text "OK" -command {.dlg.bot.def.ok flash; \
	    if {[string compare [CheckPropsDialog $dlgnametest] "ok"] ==0} {set knopf "ok"}}
	
    pack .dlg.bot.def.ok -in .dlg.bot.def -side left -expand 1 -padx 2m -pady 1m -ipadx 3m -ipady 1m 
    pack .dlg.bot.def -side left -expand 1 -padx 1m; # -padx 1m -pady 1m
    
    
    # setze grab & focus
    set oldfocus [focus]
    # grab .dlg


#@    if {[string length $text] != 0} {
	focus .dlg.para.right.en0
#@    } else {
#@	focus .dlg.menu.right.en0
#@    }

    
    
    # Warte auf Ergebnis
    tkwait variable knopf
    update
    destroy .dlg
    focus $oldfocus
    ###puts stdout $knopf
    return $knopf
}


proc CloseDialog {command checkproc name} {
    global CheckResult

    $command $checkproc $name
    if {[string length $CheckResult] != 0} {
	ErrorWin $CheckResult
	set CheckResult ""
	return cancel
    }
    return ok
}

proc CheckPropsDialog name {
    if {[string compare [CloseDialog "operationparametertool" "checkpropsdialog" $name] "ok"] == 0} {
	if {[string compare [CloseDialog "prioritytool" "checkprioritydialog" $name] "ok"] == 0} {
	    return "ok"
	}   
    }
    return "cancel"
}

proc ErrorWin {errmsg} {#
  toplevel .errorwin
  wm iconname .errorwin "Error Message"
  wm title .errorwin "Error Message"
  set ErrorWinOk "" 

  frame .errorwin.top -relief raised -bd 1
  frame .errorwin.bottom -relief raised -bd 1
  pack .errorwin.top -fill x -ipadx 5 -ipady 15 -side top
  pack .errorwin.bottom -fill x -ipadx 3c -side top


# Fehlermeldung zeigen
  message .errorwin.top.msg -width 4i -text $errmsg
  pack .errorwin.top.msg -side top -pady 2m

#button basteln
  frame .errorwin.bottom.but -relief sunken -bd 1
  pack .errorwin.bottom.but -pady 8
  button .errorwin.bottom.but.ok -text "OK" -command {set ErrorWinOk "ok"}
  pack .errorwin.bottom.but.ok -padx 2m -pady 1m -ipadx 3m -ipady 1m
  bind .errorwin.bottom.but.ok <Return> {set ErrorWinOk "ok"}
  # Focus & Grab setze, warten  
  set oldfocus [focus]
  grab .errorwin
  focus .errorwin
  tkwait variable ErrorWinOk
  update
  destroy .errorwin
  focus $oldfocus
  return $ErrorWinOk
}



proc Entrybinding en {
    global tablist knopf
    # emacs-like key bindings are defined for the widget class 'Entry'
    # provide additional key bindings for an entry 'en'
    bind $en <Down> {tab $tablist}
    bind $en <Up> {inverstab $tablist}
}


proc tab list {
    # taken from Ousterhouts draft
    set i [lsearch $list [focus]]
    incr i
    if {$i>=[llength $list]} {
	set i 0
    }
    focus [lindex $list $i]
}

proc inverstab list {
    # This proc was implemented due to Jojo Bausch.
    # He convinced me that the inverse function of 'tab' was missing.

    set i [lsearch $list [focus]]
    set i [expr $i-1]
    if {$i<0} {
	set i [llength $list]
	set i [expr $i-1]
    }
    focus [lindex $list $i]
}


set CheckResult ""

return ok
