#################################################################################
#
# Canvas 
#
#################################################################################

proc CreateCanvas {path} {
    #
    # Put up canvas and scrollbars and configure top-down display
    #
    global iter vnodespace hnodespace topd
    canvas $path.c -scrollregion {0 0 800c 800c} -width 800 -height 600 -xscroll "$path.hscroll set" -yscroll "$path.vscroll set"
    scrollbar $path.vscroll -relief sunken -command "$path.c yview"
    scrollbar $path.hscroll -orient horiz -relief sunken -command "$path.c xview"
    pack $path.hscroll -side bottom -fill x
    pack $path.vscroll -side right -fill y
    pack $path.c -expand 1 -fill both
    $path.c graph configure -order $topd -nodespaceh $hnodespace -nodespacev $vnodespace -iterations $iter
}

proc ClearBrowser {path} {
    global taglist

    set taglist [list]
    update
    $path.c graph destroy
    $path.c delete node
    $path.c delete edge
    foreach child [winfo children $path.c] {
	update
	destroy $child 
    }
}


proc Layout {path} {
    global layoutalg topd hnodespace vnodespace iter
    $path.c graph configure -order $topd -nodespaceh $hnodespace -nodespacev $vnodespace -iterations $iter

    $path.c graph layout $layoutalg
    update
}

proc layoutsettings {path} {
  global iter vnodespace hnodespace topd
    toplevel .ldialog
   wm title .ldialog "Layout Options"
    scale .ldialog.it -label "iterations for sugiyama" -from 1 -to 100 -length 10c -orient horizontal -command {set iter}
    .ldialog.it set $iter
    scale .ldialog.vspace -label "vertical nodespace" -from 5 -to 100 -length 10c -orient horizontal -command {set vnodespace}
    .ldialog.vspace set $vnodespace
    scale .ldialog.hspace -label "horizontal nodespace" -from 5 -to 100 -length 10c -orient horizontal -command {set hnodespace}
    .ldialog.hspace set $hnodespace
    frame .ldialog.bottom
    frame .ldialog.middle
    pack .ldialog.it .ldialog.vspace .ldialog.hspace -ipady 2 -side top
    pack .ldialog.middle .ldialog.bottom -side top -pady 4 

    # radio buttons
    frame .ldialog.middle.dir
    frame .ldialog.middle.lab
    pack .ldialog.middle.dir .ldialog.middle.lab
    #
    # Layout direction
    message .ldialog.middle.dir.msg -text "Layout direction:"
    radiobutton .ldialog.middle.dir.top -text "top down" -variable topd -value 1 -relief flat
    radiobutton .ldialog.middle.dir.left -text "left right" -variable topd -value 0 -relief flat
    pack .ldialog.middle.dir.msg .ldialog.middle.dir.top .ldialog.middle.dir.left -side left -padx 2
    #
    # node type
    message .ldialog.middle.lab.msg -text "show node type"
    radiobutton .ldialog.middle.lab.top -text "yes" -variable nodetype -value 1 -relief flat
    radiobutton .ldialog.middle.lab.left -text "no" -variable nodetype -value 0 -relief flat
    pack .ldialog.middle.lab.msg .ldialog.middle.lab.top .ldialog.middle.lab.left -side left -padx 2
    
    # button apply & dismiss
    button .ldialog.bottom.apply -text "Apply" -command {$path.c graph configure -order $topd -nodespaceh $hnodespace -nodespacev $vnodespace -iterations $iter; setlabel showtype $nodetype} 
    button .ldialog.bottom.dis -text "Dismiss" -command {update; destroy .ldialog}
    pack .ldialog.bottom.apply .ldialog.bottom.dis -side left -ipady 2m -ipadx 2m -padx 1c
}

proc GraphOpen {} {
    global selectedfile
    fileselect
    tkwait variable selectedfile
    if {[string length $selectedfile] != 0} {
	ClearLog
	pdialog opengraph $selectedfile
    }
}

proc Leave {} {
    update
    destroy .
}

######################################
#the proc. gets a identity of a node
#and tries to place it in the middle of the window
#showing the disired graph

set scrollinc 10
# srollinc ist harmlos, wird nur hier lokal benutzt!

proc center {id} {
  global path

  CenterNode $path N$id

}

proc CenterNode { path idNode } {

        global scrollinc

        #get the current position of desired node
        set xy [$path.c coords $idNode]
        #isolate the parameters from the xy structure
        set x [lindex $xy 0]
        set y [lindex $xy 1]

        puts "$x $y"

        set w [winfo reqwidth $path.c]
        set h [winfo reqheight $path.c] 

        set newposx [expr int ( ($x - ($w / 2) ) / $scrollinc )]
        set newposy [expr int ( ($y - ($h / 2) ) / $scrollinc )]

        if {$newposx < 0} {set newposx 0}
        if {$newposy < 0} {set newposy 0}

        $path.c xview $newposx
        $path.c yview $newposy
}


#################################################################################
#
# Globals
#
################################################################################

# Create window for graph view
# CreateCanvas $path
#JK brauchen wir nicht! => ffgraph window

return ok