####################################################################
#
# Begin Presentation-Code
#
# written by Jens Holle
#            email: jho@uni-paderborn.de
#
# Version: 30.07.96
#
####################################################################



# CreatePresentationMenu
# Definition in file ./PresentationMenu.tcl
# Else default: 

proc CreatePresentationMenu {preswin type} {
}

if { [file exists "./PresentationMenu.tcl"]} {
  puts "reading PresentationMenu.tcl\n" 
  source PresentationMenu.tcl
}

# CreatePresentationMenu




proc DisplayPresentation {node} {

  set preswin .preswin$node

  textview gettextattr $node

  set text [textview getpresentation $node]

  CreatePresentationWindow $preswin "Presentation of Node $node:"
  InsertPresentation $preswin 1.0 end $text ""

}



proc DisplayRootPresentations {} {

  set allroots [textview gettextroots]

  while {$allroots != ""} {

    set root [lindex $allroots 0]
    set allroots [lreplace $allroots 0 0]
    set preswin .rootpreswin$root

    textview gettextattr $root

    set text [textview getpresentation $root]

    CreatePresentationWindow $preswin [lindex $text 0]
    InsertPresentation $preswin 1.0 end $text $root

    while {[regexp {(%)([0-9]+)(%)} $text f f nextnode]} {

      set sub [textview getpresentation $nextnode]
     
      regexp -indices {(%)([0-9]+)(%)} $text poslist  
      set pos1 "1.0 + [lindex $poslist 0] chars"
      set pos2 "1.0 + [lindex $poslist 1] chars"
      InsertPresentation $preswin $pos1 $pos2 $sub $nextnode

      regsub {(%)([0-9]+)(%)} $text $sub text

    }
 
  } 

}



proc KillPresentation {preswin} {

  if {[winfo exists $preswin]} {
      destroy $preswin
  } 

}



proc CreatePresentationWindow {preswin prestitle} {

  global fontsize

  if ![winfo exists $preswin] {

    toplevel $preswin
    wm minsize $preswin 40 15
    set fontsize 120

    frame $preswin.mbar -relief raised -bd 2
    text $preswin.text -yscrollcommand "$preswin.scroll set"
    scrollbar $preswin.scroll -command "$preswin.text yview"

    pack $preswin.mbar -side top -fill x
    pack $preswin.text -side left -anchor nw -fill y
    pack $preswin.scroll -side right -fill y

    wm title $preswin $prestitle
    wm iconname $preswin $prestitle

    $preswin.text configure -font *-Courier-Medium-R-Normal-*-$fontsize-* \
                                   -foreground #666

    CreateGeneralMenus $preswin
    CreatePresentationMenu $preswin $prestitle

  }

}



proc InsertPresentation {preswin pos1 pos2 text prestag} {
  
  $preswin.text delete $pos1 "$pos2 + 1 chars"
  $preswin.text insert $pos1 $text
  
  if {$prestag != ""} {
  
    set pos2 "$pos1 + [string length $text] chars"

    $preswin.text tag remove [$preswin.text tag names $pos1] $pos1 $pos2

    $preswin.text tag add $prestag $pos1 $pos2

    BindPresentation $preswin $prestag

  }

}



proc BindPresentation {preswin prestag} {

  $preswin.text tag raise $prestag

  $preswin.text tag bind $prestag <Enter> "
    $preswin.text tag configure $prestag -foreground #000
  "

  $preswin.text tag bind $prestag <Leave> "
    $preswin.text tag configure $prestag -foreground #666
  "

  $preswin.text tag bind $prestag <Button-2> "
    BrowserSetSelection1 $preswin $prestag
  "

  $preswin.text tag bind $prestag <Button-3> "
    BrowserSetSelection2 $preswin $prestag
  "

}



proc BrowserSetSelection1 {preswin tag} {

  global taglist

  set path .browse  

  # selecting/deselecting a node
  foreach oldTag $taglist {
    # set normal colors.
    NodeSetColor $path.c.node$oldTag Selection Black White
  }

  ResetPresentationSelection

  # delete old taglist
  set taglist [list]
	   
  #build new selection
  lappend taglist N$tag
  NodeSetColor $path.c.nodeN$tag Selection White Black
  $preswin.text tag configure $tag -font *-Courier-Bold-O-Normal-*-120-*
	   
  #update variable
  UpdateSelectionVariable Selection $taglist

}



proc BrowserSetSelection2 {preswin tag} {

  global taglist

  set path .browse  

  # continue selection, switch selection mode 
  # (selected <--> unselected) of node.
	       
   set index [lsearch $taglist N$tag]
	       
  if {$index == -1} {
    # node not selected.
    lappend taglist N$tag
    NodeSetColor $path.c.nodeN$tag Selection White Black
    $preswin.text tag configure $tag -font *-Courier-Bold-O-Normal-*-120-*
    UpdateSelectionVariable Selection $taglist
  } else {
    # node selected.
    set taglist [lreplace $taglist $index $index]
    NodeSetColor $path.c.nodeN$tag Selection Black White
    $preswin.text tag configure $tag -font *-Courier-Medium-R-Normal-*-120-*
    UpdateSelectionVariable Selection $taglist
  }

}



proc ResetPresentationSelection {} {

  global taglist

  set allroots [textview gettextroots]

  while {$allroots != ""} {

    set root [lindex $allroots 0]
    set allroots [lreplace $allroots 0 0]
    set preswin .rootpreswin$root

    foreach oldTag $taglist {
      set oldTag [string range $oldTag 1 end]
      $preswin.text tag configure $oldTag -font *-Courier-Medium-R-Normal-*-120-*
    }

  } 

}



proc CreateGeneralMenus {preswin} {

  global globvar

  CreateMenu $preswin File
  CreateTclMenuEntry $preswin File Save 0 "SavePresentation $preswin"
  CreateSeparator $preswin File
  CreateTclMenuEntry $preswin File Quit 0 "KillPresentation $preswin"

  CreateMenu $preswin Undo
  CreateTclMenuEntry $preswin Undo Undo 0 "calltransaction Undo"
  CreateTclMenuEntry $preswin Undo Redo 0 "calltransaction Redo"

  CreateMenu $preswin Show
  CreateTclMenuEntry $preswin Show Selection 0 {center $globvar(Selection1)}

  InitMenu $preswin {}

}



proc CreateTclMenuEntry {preswin menuname menuentry key menucommand} {

  set menu [string tolower $menuname]

  $preswin.mbar.$menu.menu add command -label $menuentry -underline $key \
	        -command $menucommand

}



#
# Presentation-Menu Procedures
#

proc CreateMenu {preswin menuname} {

  set menu [string tolower $menuname]

  menubutton $preswin.mbar.$menu -text $menuname -menu $preswin.mbar.$menu.menu
  pack $preswin.mbar.$menu -side left
  menu $preswin.mbar.$menu.menu

}

proc CreateMenuEntry {preswin menuname menuentry key menucommand} {

  set menu [string tolower $menuname]

  $preswin.mbar.$menu.menu add command -label $menuentry -underline $key \
	        -command "calldialog $menucommand Transactions $menucommand"

}

proc CreateSeparator {preswin menuname} {

  set menu [string tolower $menuname]

  $preswin.mbar.$menu.menu add separator

}

proc InitMenu {preswin usermenulist} {

  set presmenu $preswin.mbar  

  set menulist "$presmenu $presmenu.file $presmenu.undo $presmenu.show"
  foreach menu $usermenulist {
    set menu [string tolower $menu]
    set menulist [lappend menulist $presmenu.$menu]
  }
  
  eval tk_menuBar $menulist

  focus $presmenu

}

#
# TextView-Dump
#

proc SavePresentation {preswin} {

  global selectedfile

  fileselect
  tkwait variable selectedfile
  if {[string length $selectedfile] != 0} {
    set file [open $selectedfile w+]
    if {[string first "couldn't open" $file]==-1} {
      puts $file [$preswin.text get 1.0 end]
      close $file
    }
  }
}



########################################################################
#
# End Presentation-Code
#
########################################################################






return ok