#################################################################################
#
# dialog for node end edge types & helping procs
#
#################################################################################

proc selectall {window} {
    .choose.$window.left.bottom.types select from 0
    .choose.$window.left.bottom.types select to end
}

proc transfer {window} {
# transfer the selected listbox entries
    foreach i [.choose.$window.left.bottom.types curselection] {
	set sel [.choose.$window.left.bottom.types get $i]
	#
	# check if selected entry already exists in user listbox
	#
	set text [list]
	for {set disp 0} {$disp<[.choose.$window.right.bottom.types size]} {incr disp} {
	    lappend text [.choose.$window.right.bottom.types get $disp]
	}

	# entry doesn't exist in user listbox
	if {[lsearch $text $sel]==-1} {
	    lappend text $sel
	    set text [lsort $text]
	    # insert sel
	    .choose.$window.right.bottom.types insert [lsearch $text $sel] $sel 
	}
    }
}

proc remove {window} {
    set offset 0
    foreach i [.choose.$window.right.bottom.types curselection] {
	update
	.choose.$window.right.bottom.types delete [expr $i-$offset]
	incr offset
    }
}

proc showlistboxes {window} {

    if {[string compare $window nodetypes]==0} {
	set name "node types"
    } else {
	set name "edge types"
    }
    frame .choose.$window.left
    frame .choose.$window.middle
    frame .choose.$window.right
    pack .choose.$window.left .choose.$window.middle .choose.$window.right -side left
    frame .choose.$window.left.top
    frame .choose.$window.left.bottom
    pack .choose.$window.left.top .choose.$window.left.bottom -side top -fill x
    label .choose.$window.left.bottom.msg -text $name -relief raised -bd 2
    pack .choose.$window.left.bottom.msg -fill x

    listbox .choose.$window.left.bottom.types -yscrollcommand ".choose.$window.left.bottom.scroll set" -relief raised -bd 2
    scrollbar .choose.$window.left.bottom.scroll -command ".choose.$window.left.bottom.types yview"

    pack .choose.$window.left.bottom.types -side left
    pack .choose.$window.left.bottom.scroll -side right -fill y
  
    button .choose.$window.middle.all -text "Select all" -command "selectall $window"  
    button .choose.$window.middle.disp -text "Display -->" -command "transfer $window"
    button .choose.$window.middle.rem -text "Remove" -command "remove $window"
    pack .choose.$window.middle.all .choose.$window.middle.disp .choose.$window.middle.rem -side top -pady 10 -padx 5 -fill x
    
    # right node type box
    frame .choose.$window.right.top
    frame .choose.$window.right.bottom
    pack .choose.$window.right.top .choose.$window.right.bottom -side top -fill x
    label .choose.$window.right.bottom.msg -text "Display" -relief raised -bd 2
    pack .choose.$window.right.bottom.msg -fill x

    listbox .choose.$window.right.bottom.types -yscrollcommand ".choose.$window.right.bottom.scroll set" -relief raised -bd 2
    scrollbar .choose.$window.right.bottom.scroll -command ".choose.$window.right.bottom.types yview"

    pack .choose.$window.right.bottom.types -side left
    pack .choose.$window.right.bottom.scroll -side right -fill y
}


proc chooseTypesToDisplay {} {
    global nodetypes edgetypes NoOfNodetypes NoOfEdgetypes

    toplevel .choose
    wm title .choose "Types to display by browser"
    set ok ""
    frame .choose.toptext -relief raised -bd 2
    frame .choose.nodetypes; # -relief raised -bd 2
    frame .choose.edgetypes; # -relief raised -bd 2
    frame .choose.ok -relief raised -bd 2
    pack .choose.toptext .choose.nodetypes .choose.edgetypes .choose.ok -fill both -side top

    message .choose.toptext.msg -text "Select node types and edge types to display by browser" -width 6i
    pack .choose.toptext.msg
    button .choose.ok.b -text OK -command {set ok bye}
    button .choose.ok.save -text Save -command saveusertypes 
    pack .choose.ok.save -side left -padx 15 -ipadx 5 -ipady 3 -pady 5 -expand 1 -fill x

    pack .choose.ok.b -padx 15 -ipadx 5 -ipady 3 -pady 5 -expand 1 -fill x -side left

    showlistboxes nodetypes
    showlistboxes edgetypes
    showtypes
    
    set oldfocus [focus]
    grab .choose
    focus .choose
    
    tkwait variable ok
    
    set NoOfNodetypes [.choose.nodetypes.right.bottom.types size]
    for {set i 0} {$i<$NoOfNodetypes} {incr i} {
	set nodetypes($i) [.choose.nodetypes.right.bottom.types get $i]
    }
    set NoOfEdgetypes [.choose.edgetypes.right.bottom.types size]
    for {set i 0} {$i<$NoOfEdgetypes} {incr i} {
	set edgetypes($i) [.choose.edgetypes.right.bottom.types get $i]
    }
    update
    destroy .choose
    set focus $oldfocus
    update
}


proc showtypes {} {
    global nodetypes edgetypes NoOfNodetypes NoOfEdgetypes allowedNtypes allowedEtypes
    # erlaubte Typen in die Dialogbox
    for {set i 0} {$i<[llength $allowedNtypes]} {incr i} {
	.choose.nodetypes.left.bottom.types insert end [lindex $allowedNtypes $i]
    }
    for {set i 0} {$i<[llength $allowedEtypes]} {incr i} {
	.choose.edgetypes.left.bottom.types insert end [lindex $allowedEtypes $i]
    }

    # user types
    for {set i 0} {$i<$NoOfNodetypes} {incr i} {
	.choose.nodetypes.right.bottom.types insert end $nodetypes($i)
    }
    for {set i 0} {$i<$NoOfEdgetypes} {incr i} {
	.choose.edgetypes.right.bottom.types insert end $edgetypes($i)
    }
}



proc saveusertypes {} {
    set f [open UserNodeTypes w]
    for {set i 0} {$i<[.choose.nodetypes.right.bottom.types size]} {incr i} {
	set nodetype [.choose.nodetypes.right.bottom.types get $i]
	puts $f $nodetype
    }
    close $f
    set f [open UserEdgeTypes w]
    for {set i 0} {$i<[.choose.edgetypes.right.bottom.types size]} {incr i} {
	set edgetype [.choose.edgetypes.right.bottom.types get $i]
	puts $f $edgetype
    }
    close $f
}


proc initallowedtypes {} {
    # initialisiert die Liste, die die erlaubten Typen enthaelt
    # soll einmal nach dem Graphenoeffnen aufgerufen werden (von inittypes)
    
    global allowedNtypes allowedEtypes

    # delete lists
    set allowedNtypes [list]
    set allowedEtypes [list]

    # load node types
    if {[file exists NodeTypes]==1} {
	#  lade erlaubte typen
	set f [open NodeTypes r]
	while {[gets $f type]>=0} {
	    lappend allowedNtypes $type
	}
	close $f
    } else {
	# frage bei GRAS nach allen Knotentypen
	browser settypes nodes allowedNtypes
    }
    set allowedNtypes [lsort $allowedNtypes]

    if {[file exists EdgeTypes]==1} {
	set f [open EdgeTypes r]
	while {[gets $f type]>=0} {
	    lappend allowedEtypes $type
	}
	close $f
    } else {
	browser settypes edges allowedEtypes
    }
    set allowedEtypes [lsort $allowedEtypes]
}
 
proc initusertypes {} {
    # initialisiert die arrays nodetypes & edgetypes
    # soll einmal nach dem Graphenoeffnen aufgerufen werden (von inittypes)

    global nodetypes edgetypes NoOfNodetypes NoOfEdgetypes allowedNtypes allowedEtypes

    set i 0
    set temp [list]
    if {[file exists UserNodeTypes]==1} {
	# lade UserNodeTypes
	set f [open UserNodeTypes]
	while {[gets $f type]>=0} {
	    lappend temp $type
	}
	set temp [lsort $temp]
	for {set i 0} {$i<[llength $temp]} {incr i} {
	    set nodetypes($i) [lindex $temp $i]
	}
	set NoOfNodetypes [llength $temp]
	close $f
    } else {
	# File gibt es nicht -> nimm alle erlaubten Typen und kopiere sie in
	# die user-listbox
	for {set i 0} {$i<[llength $allowedNtypes]} {incr i} {
	    set nodetypes($i) [lindex $allowedNtypes $i]
	}
	set NoOfNodetypes [llength $allowedNtypes]
    }

    # das gleiche fuer edgetypes in gruen
    #
    if {[file exists UserEdgeTypes]==1} {
	set f [open UserEdgeTypes r]
	set temp ""
	while {[gets $f type]>=0} {
	    lappend temp $type
	}
	close $f
	set temp [lsort $temp]
	for {set i 0} {$i<[llength $temp]} {incr i} {
	    set edgetypes($i) [lindex $temp $i]
	}
	set NoOfEdgetypes [llength $temp]
    } else {
	for {set i 0} {$i<[llength $allowedEtypes]} {incr i} {
	    set edgetypes($i) [lindex $allowedEtypes $i]
	}
	set NoOfEdgetypes [llength $allowedEtypes]
    }
}


proc inittypes {} {
    # initialisiert die Liste mit den erlaubten Typen & die Arrays nodetypes
    # , edgetypes
    # inittypes muss einmal nach dem oeffnen des Graphen aufgerufen werden
    # wird fuer den Browser von C aus aufgerufen in InitBrowser

    initallowedtypes
    initusertypes
}


return ok
