spec CIM

   section Scheme

      node_class OBJECT
         intrinsic
            Name : string;
      end;

      node_class COMPUTER is_a OBJECT end;

      node_class DATABASE is_a OBJECT end;

      node_class PROGRAM is_a OBJECT end;

      edge_type StoredOn : DATABASE -> COMPUTER;

      edge_type RunsOn : PROGRAM -> COMPUTER;

      node_class APPLICATION is_a PROGRAM end;

      node_class CIM_MANAGER is_a PROGRAM end;

      node_class DOCUMENT_BASE is_a DATABASE end;

      node_class DOCUMENT is_a OBJECT
         intrinsic
            File : string := "";
      end;

      edge_type Stores : DOCUMENT_BASE -> DOCUMENT;

      node_class PRIVATE_BASE is_a DOCUMENT_BASE end;

      edge_type Accesses : APPLICATION -> PRIVATE_BASE;

      node_class PUBLIC_BASE is_a DOCUMENT_BASE end;

      edge_type Controls : CIM_MANAGER -> PUBLIC_BASE;

      node_class INTEGRATION_BASE is_a DATABASE end;

      edge_type Integrates : INTEGRATION_BASE -> PUBLIC_BASE;

      edge_type Contains : INTEGRATION_BASE -> META_DOCUMENT;

      node_class META_DOCUMENT is_a OBJECT
         intrinsic
            Locked : boolean := false;
      end;

      edge_type Describes : META_DOCUMENT -> DOCUMENT;

      edge_type DependsOn : META_DOCUMENT -> META_DOCUMENT;

      edge_type Administrates : CIM_MANAGER -> INTEGRATION_BASE;
   end;

   section Operations

      production CreateComputer ( ComputerName : string ; Type : type_in COMPUTER ) =
         (* @CC_LG 7008(8,8)(27,33) @*)
         ::=
         begin
            1' : Type;
            (* @CC_LG 6011(72,28)(59,21) @*)
         end
         (* @CC_LG 7009(8,8)(149,47) @*)
         transfer 1'.Name := ComputerName;
      end;

      production CreateProgram ( ProgramName : string ; Type : type_in PROGRAM ; ComputerName : string )
      =
         begin
            obl_node `1 : COMPUTER;
            (* @CC_LG 6007(59,28)(89,21) @*)
         end
         (* @CC_LG 7008(8,8)(245,47) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(36,28)(57,19) @*)
            2' : Type;
            (* @CC_LG 6011(204,28)(59,21) @*)
            2' -> 1' : RunsOn;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7009(8,8)(245,47) @*)
         condition `1.Name = ComputerName;
         transfer 2'.Name := ProgramName;
      end;

      production CreateIntegrationBase ( BaseName : string ; Type : type_in INTEGRATION_BASE ; CIMManagerName : string
                                                                                                                (* @CC_LT Vertical @*)
                                          ; ComputerName : string )
      =
         begin
            obl_node `1 : CIM_MANAGER;
            (* @CC_LG 6007(264,28)(113,21) @*)
            obl_node `2 : COMPUTER;
            (* @CC_LG 6007(60,28)(89,21) @*)
            `1 -> `2 : RunsOn;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7008(8,8)(329,47) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(84,56)(57,19) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(48,168)(57,19) @*)
            1' -> 2' : RunsOn (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            3' : Type;
            (* @CC_LG 6011(120,112)(59,21) @*)
            3' -> 2' : StoredOn;
            1' -> 3' : Administrates;
         end
         (* @CC_LG 7009(8,8)(162,187) @*)
         condition (`1.Name = CIMManagerName) and (`2.Name = ComputerName);
         transfer 3'.Name := BaseName;
      end;

      production CreatePrivateBase ( BaseName : string ; Type : type_in PRIVATE_BASE ; ApplicationName : string ; ComputerName : string
                                      )
      =
         begin
            obl_node `1 : APPLICATION;
            (* @CC_LG 6007(264,28)(113,21) @*)
            obl_node `2 : COMPUTER;
            (* @CC_LG 6007(60,28)(89,21) @*)
            `1 -> `2 : RunsOn;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7008(8,8)(329,47) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(264,28)(57,19) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(60,28)(57,19) @*)
            1' -> 2' : RunsOn (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            3' : Type;
            (* @CC_LG 6011(168,112)(59,21) @*)
            3' -> 2' : StoredOn;
            (* @CC_LG 3004 @*)
            1' -> 3' : Accesses;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7009(329,145) @*)
         condition (`1.Name = ApplicationName) and (`2.Name = ComputerName);
         transfer 3'.Name := BaseName;
      end;

      production CreatePublicBase ( BaseName : string ; Type : type_in PUBLIC_BASE ; CIMManagerName : string ; IntegrationBaseName : string
                                     ; ComputerName : string )
      =
         begin
            obl_node `1 : CIM_MANAGER;
            (* @CC_LG 6007(288,27)(113,21) @*)
            obl_node `2 : COMPUTER;
            (* @CC_LG 6007(60,27)(89,21) @*)
            obl_node `3 : INTEGRATION_BASE;
            (* @CC_LG 6007(288,98)(155,21) @*)
            `1 -> `3 : Administrates;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7008(8,8)(377,117) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(312,27)(57,19) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(84,27)(57,19) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(312,112)(57,19) @*)
            1' -> 3' : Administrates (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            4' : Type;
            (* @CC_LG 6011(84,112)(59,21) @*)
            3' -> 4' : Integrates;
            (* @CC_LG 3004 @*)
            1' -> 4' : Controls;
            (* @CC_LG 3004 @*)
            4' -> 2' : StoredOn;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7009(365,131) @*)
         condition (`1.Name = CIMManagerName) and (`2.Name = ComputerName) and (`3.Name = IntegrationBaseName);
         transfer 4'.Name := BaseName;
      end;

      production CreateDocument ( DocumentName : string ; Type : type_in DOCUMENT ; MetaType : type_in META_DOCUMENT
                                   ; CIMManagerName : string ; IntegrationBaseName : string ; PublicBaseName : string
                                   )
      =
         begin
            obl_node `1 : CIM_MANAGER;
            (* @CC_LG 6007(216,28)(113,21) @*)
            obl_node `2 : INTEGRATION_BASE;
            (* @CC_LG 6007(84,98)(155,21) @*)
            obl_node `3 : PUBLIC_BASE;
            (* @CC_LG 6007(384,98)(155,21) @*)
            `1 -> `2 : Administrates;
            (* @CC_LG 3004 @*)
            `1 -> `3 : Controls;
            (* @CC_LG 3004 @*)
            `2 -> `3 : Integrates;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7008(473,117) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(216,27)(57,19) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(108,84)(57,19) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(348,84)(57,19) @*)
            1' -> 2' : Administrates (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            1' -> 3' : Controls (* @CC_LG 1002C,BC(0,-9)(72,10) @*);
            (* @CC_LG 3004 @*)
            2' -> 3' : Integrates (* @CC_LG 1002C,BC(0,-9)(60,10) @*);
            (* @CC_LG 3004 @*)
            4' : MetaType;
            (* @CC_LG 6011(108,154)(83,21) @*)
            5' : Type;
            (* @CC_LG 6011(348,154)(59,21) @*)
            4' -> 5' : Describes;
            (* @CC_LG 3004 @*)
            2' -> 4' : Contains;
            (* @CC_LG 3004 @*)
            3' -> 5' : Stores;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7009(8,8)(461,173) @*)
         condition (`1.Name = CIMManagerName) and (`2.Name = IntegrationBaseName) and (`3.Name = PublicBaseName);
         transfer 4'.Name := DocumentName;
                  5'.Name := DocumentName;
      end;

      production FetchDocument ( DocumentName : string ; Write : boolean ; ApplicationName : string ; CIMManagerName : string
                                  ; IntegrationBaseName : string ; PublicBaseName : string ; PrivateBaseName : string
                                  )
      =
         begin
            obl_node `1 : CIM_MANAGER;
            (* @CC_LG 6007(204,28)(113,21) @*)
            obl_node `2 : INTEGRATION_BASE;
            (* @CC_LG 6007(84,98)(155,21) @*)
            obl_node `3 : PUBLIC_BASE;
            (* @CC_LG 6007(336,98)(155,21) @*)
            obl_node `4 : META_DOCUMENT;
            (* @CC_LG 6007(84,168)(125,21) @*)
            obl_node `5 : DOCUMENT;
            (* @CC_LG 6007(336,168)(95,21) @*)
            obl_node `6 : PRIVATE_BASE;
            (* @CC_LG 6007(336,266)(125,21) @*)
            obl_node `7 : APPLICATION;
            (* @CC_LG 6007(84,266)(101,21) @*)
            `1 -> `2 : Administrates;
            (* @CC_LG 3004 @*)
            `1 -> `3 : Controls;
            (* @CC_LG 3004 @*)
            `2 -> `3 : Integrates;
            (* @CC_LG 3004 @*)
            `2 -> `4 : Contains;
            (* @CC_LG 3004 @*)
            `3 -> `5 : Stores;
            (* @CC_LG 3004 @*)
            `4 -> `5 : Describes;
            (* @CC_LG 3004 @*)
            `7 -> `6 : Accesses (* @CC_LG 1002C,BC(0,6)(66,10) @*);
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7008(8,8)(425,285) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(204,28)(57,19) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(120,84)(57,19) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(300,84)(57,19) @*)
            obl_node 4' = `4;
            (* @CC_LG 6007(120,154)(57,19) @*)
            obl_node 5' = `5;
            (* @CC_LG 6007(300,154)(57,19) @*)
            obl_node 6' = `6;
            (* @CC_LG 6007(300,280)(57,19) @*)
            obl_node 7' = `7;
            (* @CC_LG 6007(120,280)(57,19) @*)
            1' -> 2' : Administrates (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            1' -> 3' : Controls (* @CC_LG 1002C,BC(0,-9)(72,10) @*);
            (* @CC_LG 3004 @*)
            2' -> 3' : Integrates (* @CC_LG 1002C,BC(0,-9)(60,10) @*);
            (* @CC_LG 3004 @*)
            2' -> 4' : Contains (* @CC_LG 1002C,BC(0,-9)(48,10) @*);
            (* @CC_LG 3004 @*)
            3' -> 5' : Stores (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            4' -> 5' : Describes (* @CC_LG 1002C,BC(0,-9)(60,10) @*);
            (* @CC_LG 3004 @*)
            7' -> 6' : Accesses (* @CC_LG 1002C,BC(0,20)(66,10) @*);
            (* @CC_LG 3004 @*)
            8' : `5.type;
            (* @CC_LG 6011(300,210)(83,21) @*)
            6' -> 8' : Stores;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7009(8,8)(425,313) @*)
         condition (`1.Name = CIMManagerName) and (`2.Name = IntegrationBaseName) and (`3.Name = PublicBaseName) and (`4.Name = DocumentName)
                                                 and (`5.Name = DocumentName) and (`6.Name = PrivateBaseName) and (`7.Name = ApplicationName)
                                                 and (not (`4.Locked and Write))                                                             ;
         transfer 8'.Name := DocumentName;
                  8'.File := `5.File;
                  4'.Locked := Write;
      end;

      production StoreDocument ( DocumentName : string ; ApplicationName : string ; CIMManagerName : string ; IntegrationBaseName : string
                                  ; PublicBaseName : string ; PrivateBaseName : string )
      =
         begin
            obl_node `1 : CIM_MANAGER;
            (* @CC_LG 6007(216,28)(113,21) @*)
            obl_node `2 : INTEGRATION_BASE;
            (* @CC_LG 6007(84,98)(155,21) @*)
            obl_node `3 : PUBLIC_BASE;
            (* @CC_LG 6007(360,98)(155,21) @*)
            obl_node `4 : META_DOCUMENT;
            (* @CC_LG 6007(84,168)(125,21) @*)
            obl_node `5 : DOCUMENT;
            (* @CC_LG 6007(360,168)(95,21) @*)
            obl_node `6 : PRIVATE_BASE;
            (* @CC_LG 6007(360,280)(125,21) @*)
            obl_node `7 : APPLICATION;
            (* @CC_LG 6007(84,280)(101,21) @*)
            `1 -> `2 : Administrates;
            (* @CC_LG 3004 @*)
            `1 -> `3 : Controls;
            (* @CC_LG 3004 @*)
            `2 -> `3 : Integrates;
            (* @CC_LG 3004 @*)
            `2 -> `4 : Contains;
            (* @CC_LG 3004 @*)
            `3 -> `5 : Stores;
            (* @CC_LG 3004 @*)
            `4 -> `5 : Describes;
            (* @CC_LG 3004 @*)
            `7 -> `6 : Accesses (* @CC_LG 1002C,BC(0,6)(66,10) @*);
            (* @CC_LG 3004 @*)
            obl_node `8 : DOCUMENT;
            (* @CC_LG 6007(360,224)(95,21) @*)
            `6 -> `8 : Stores;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7008(8,8)(485,299) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(216,28)(57,19) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(120,84)(57,19) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(324,84)(57,19) @*)
            obl_node 4' = `4;
            (* @CC_LG 6007(120,154)(57,19) @*)
            obl_node 5' = `5;
            (* @CC_LG 6007(324,154)(57,19) @*)
            obl_node 6' = `6;
            (* @CC_LG 6007(324,238)(57,19) @*)
            obl_node 7' = `7;
            (* @CC_LG 6007(120,238)(57,19) @*)
            1' -> 2' : Administrates (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            1' -> 3' : Controls (* @CC_LG 1002C,BC(0,-9)(72,10) @*);
            (* @CC_LG 3004 @*)
            2' -> 3' : Integrates (* @CC_LG 1002C,BC(0,-9)(60,10) @*);
            (* @CC_LG 3004 @*)
            2' -> 4' : Contains (* @CC_LG 1002C,BC(0,-9)(48,10) @*);
            (* @CC_LG 3004 @*)
            3' -> 5' : Stores (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            4' -> 5' : Describes (* @CC_LG 1002C,BC(0,-9)(60,10) @*);
            (* @CC_LG 3004 @*)
            7' -> 6' : Accesses (* @CC_LG 1002C,BC(0,6)(66,10) @*);
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7009(8,8)(485,257) @*)
         condition (`1.Name = CIMManagerName) and (`2.Name = IntegrationBaseName) and (`3.Name = PublicBaseName) and (`4.Name = DocumentName)
                                                 and (`5.Name = DocumentName) and (`6.Name = PrivateBaseName) and (`7.Name = ApplicationName)
                                                 and (`8.Name = DocumentName) and (`4.Locked)                                                ;
         transfer 5'.File := `5.File;
                  4'.Locked := false;
      end;

      production CreateDependency ( CIMManagerName : string ; IntegrationBaseName : string ; SlaveDocument, MasterDocument : string
                                     )
      =
         begin
            obl_node `1 : CIM_MANAGER;
            (* @CC_LG 6007(180,28)(113,21) @*)
            obl_node `2 : INTEGRATION_BASE;
            (* @CC_LG 6007(180,84)(155,21) @*)
            obl_node `3 : META_DOCUMENT;
            (* @CC_LG 6007(72,154)(125,21) @*)
            obl_node `4 : META_DOCUMENT;
            (* @CC_LG 6007(300,154)(125,21) @*)
            `1 -> `2 : Administrates;
            (* @CC_LG 3004 @*)
            `2 -> `3 : Contains;
            (* @CC_LG 3004 @*)
            `2 -> `4 : Contains;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7008(8,8)(377,173) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(192,28)(57,19) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(192,84)(57,19) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(120,154)(57,19) @*)
            obl_node 4' = `4;
            (* @CC_LG 6007(276,154)(57,19) @*)
            1' -> 2' : Administrates (* @CC_LG 1002C,BC(0,-9)(54,10) @*);
            (* @CC_LG 3004 @*)
            2' -> 3' : Contains (* @CC_LG 1002C,BC(0,-9)(48,10) @*);
            (* @CC_LG 3004 @*)
            2' -> 4' : Contains (* @CC_LG 1002C,BC(0,-9)(48,10) @*);
            (* @CC_LG 3004 @*)
            4' -> 3' : DependsOn;
            (* @CC_LG 3004 @*)
         end
         (* @CC_LG 7009(8,8)(377,173) @*)
         condition (`1.Name = CIMManagerName) and (`2.Name = IntegrationBaseName) and (`3.Name = MasterDocument) and (`4.Name = SlaveDocument);
      end;
   end;

   section Example

      node_type SUN : COMPUTER end;

      node_type VAX : COMPUTER end;

      node_type IBM : COMPUTER end;

      node_type HP : COMPUTER end;

      node_type CIMManager : CIM_MANAGER end;

      node_type CADSystem : APPLICATION end;

      node_type CAPSystem : APPLICATION end;

      node_type NCPSystem : APPLICATION end;

      node_type IntegrationBase : INTEGRATION_BASE end;

      node_type PublicCADBase : PUBLIC_BASE end;

      node_type PublicCAPBase : PUBLIC_BASE end;

      node_type PublicNCPBase : PUBLIC_BASE end;

      node_type PrivateCADBase : PRIVATE_BASE end;

      node_type PrivateCAPBase : PRIVATE_BASE end;

      node_type PrivateNCPBase : PRIVATE_BASE end;

      node_type Drawing : DOCUMENT end;

      node_type WorkPlan : DOCUMENT end;

      node_type NCProgram : DOCUMENT end;

      node_type MetaDrawing : META_DOCUMENT end;

      node_type MetaWorkPlan : META_DOCUMENT end;

      node_type MetaNCProgram : META_DOCUMENT end;

      transaction MAIN =
           CreateComputer ( "Trinidad", SUN )
         & CreateComputer ( "Corea", VAX )
         & CreateComputer ( "Einstein", IBM )
         & CreateComputer ( "Nordkap", HP )
         & CreateProgram ( "Sukits", CIMManager, "Trinidad" )
         & CreateProgram ( "Euclid", CADSystem, "Corea" )
         & CreateProgram ( "PC-CAP", CAPSystem, "Einstein" )
         & CreateProgram ( "Exapt", NCPSystem, "Nordkap" )
         & CreateIntegrationBase ( "Wheel", IntegrationBase, "Sukits", "Trinidad" )
         & CreatePrivateBase ( "CADBase", PrivateCADBase, "Euclid", "Corea" )
         & CreatePrivateBase ( "CAPBase", PrivateCAPBase, "PC-CAP", "Einstein" )
         & CreatePrivateBase ( "NCPBase", PrivateNCPBase, "Exapt", "Nordkap" )
         & CreatePublicBase ( "WheelCAD", PublicCADBase, "Sukits", "Wheel", "Corea" )
         & CreatePublicBase ( "WheelCAP", PublicCAPBase, "Sukits", "Wheel", "Einstein" )
         & CreatePublicBase ( "WheelNCP", PublicNCPBase, "Sukits", "Wheel", "Nordkap" )
         & CreateDocument ( "WheelDrawing", Drawing, MetaDrawing, "Sukits", "Wheel", "WheelCAD" )
         & CreateDocument ( "WheelWorkPlan", WorkPlan, MetaWorkPlan, "Sukits", "Wheel", "WheelCAP" )
         & CreateDependency ( "Sukits", "Wheel", "WheelWorkPlan", "WheelDrawing" )
         & FetchDocument ( "WheelDrawing", true, "Euclid", "Sukits", "Wheel", "WheelCAD", "CADBase" )
         & StoreDocument ( "WheelDrawing", "Euclid", "Sukits", "Wheel", "WheelCAD", "CADBase" )
         & FetchDocument ( "WheelWorkPlan", true, "PC-CAP", "Sukits", "Wheel", "WheelCAP", "CAPBase" )
         & StoreDocument ( "WheelWorkPlan", "PC-CAP", "Sukits", "Wheel", "WheelCAP", "CAPBase" )
         & CreateDocument ( "WheelNCProgram1", NCProgram, MetaNCProgram, "Sukits", "Wheel", "WheelNCP" )
         & CreateDocument ( "WheelNCProgram2", NCProgram, MetaNCProgram, "Sukits", "Wheel", "WheelNCP" )
         & CreateDependency ( "Sukits", "Wheel", "WheelNCProgram1", "WheelWorkPlan" )
         & CreateDependency ( "Sukits", "Wheel", "WheelNCProgram2", "WheelWorkPlan" )
         & FetchDocument ( "WheelNCProgram1", true, "Exapt", "Sukits", "Wheel", "WheelNCP", "NCPBase" )
         & StoreDocument ( "WheelNCProgram1", "Exapt", "Sukits", "Wheel", "WheelNCP", "NCPBase" )
         & FetchDocument ( "WheelNCProgram2", true, "Exapt", "Sukits", "Wheel", "WheelNCP", "NCPBase" )
         & StoreDocument ( "WheelNCProgram2", "Exapt", "Sukits", "Wheel", "WheelNCP", "NCPBase" )
      end;
   end;

end.
