spec Reals (* This specification demonstrates how the language
              PROGRES can be extended by importing user
              types and operations implemented externally in C. 
              Here, we introduce a floating point type Real (double in C),
              for mathematical calculations. (ajw, 07/07/97) *)

   from RealNumbers (* Real is an imported type which is 
                     * implemented with the double in C *) import

      types
         Real;

      functions
         R_0            : -> Real (* 0.0 *),
         R_Pi           : -> Real (* 3.14159..    *),
         R_e            : -> Real (* 2.71828..    *);
      (* constants *)

      functions
         R_Minus        : ( Real) -> Real (* unary minus: -x *);
      (* unary operator *)

      functions
         `+'            : ( Real, Real) -> Real (* x+y *),
         `-'            : ( Real, Real) -> Real (* x-y *),
         `*'            : ( Real, Real) -> Real (* x*y *),
         `/'            : ( Real, Real) -> Real (* x/y *),
         `='            : ( Real, Real) -> boolean
         (* x=y *)                                ;
      (* binary infix(!) operators *
      * don't forget the quotes, *
      * e.g. r := (R_0 `+' R_IntToReal(2)) *)

      functions
         R_RealToInt    : ( Real) -> integer,
         R_Ceil         : ( Real) -> integer,
         R_Floor        : ( Real) -> integer,
         R_IntToReal    : ( integer) -> Real,
         R_InitReal     : ( integer, integer, integer) -> Real (*
                                                               <int>.<fract>E<exp> *);
      (* integer/Real conversion operations *)

      functions
         R_Square       : ( Real) -> Real,
         R_Sqrt         : ( Real) -> Real,
         R_Pow          : ( Real, Real) -> Real (* x^y *),
         R_Exp          : ( Real) -> Real (* e^x *),
         R_Log          : ( Real) -> Real (* ln(x) *),
         R_Sin          : ( Real) -> Real (* sin(x) *),
         R_Cos          : ( Real) -> Real (* cos(x) *),
         R_Tan          : ( Real) -> Real (* tan(x) *);
      (* useful math functions *)

   end;

   transaction Main =
      use r1, r2 : Real;
          i : integer;
          s : string
          (* This transactions shows some examples of how to use the
             imported type and functions. *)
      do
           r1 := ((R_e `*' R_Pi `-' R_0) `/' R_Sqrt ( R_e ))
         & r2 := R_InitReal ( 1, 2345, 16 )
         & i := R_RealToInt ( r2 )
         & (R_IntToReal ( 1 ) `=' R_Exp ( R_0 ))
         & s := (string ( r1 ) & " * " & string ( r2 ) & " = " & string ( r1 `*' r2 ))
         & skip
      end
   end;

end.
