spec MedicalDiagnosis

   section BasicScheme

      node_class ITEM
         intrinsic
            index Name : string := string ( self.type );
         derived
            index GRAS_BROWSER_LABEL : string = self.Name;
      end;

      node_class LIST end;
      (* @CC_LG 6007(240,336)(97,29) @*)

      node_class LIST_ELEM
         derived
            No : integer = [ self.<-lnext-.No
                           | 1 ]             ;
      end;
      (* @CC_LG 6007(228,154)(97,29) @*)

      edge_type lelem : LIST -> LIST_ELEM [1:1];

      edge_type llast : LIST [0:1] -> LIST_ELEM [0:1];

      edge_type lnext : LIST_ELEM [0:1] -> LIST_ELEM [0:1];

      production CreateItem( nodeType : type_in ITEM) =
         (* @CC_LG 7008(8,8)(27,33) @*)
         ::=
         begin
            1' : nodeType;
            (* @CC_LG 6011(60,28)(89,21) @*)
         end
         (* @CC_LG 7009(8,8)(111,47) @*)

      end;

      section WorkingData

         node_class PRACTICE is_a ITEM end;

         edge_type selem : PRACTICE -> PERSON;

         node_class PERSON is_a ITEM end;

         node_class DOCTOR is_a PERSON end;

         edge_type does : DOCTOR [1:1] -> TREATMENT;

         node_class PATIENT is_a PERSON, LIST end;

         node_class BILL
            derived
               CostSum : integer;
         end;

         node_class TREATMENT is_a LIST_ELEM, BILL
            derived
               Date : integer;
            redef_derived
               CostSum = self.toDiagnosis.CostSum + self.toPrescription.CostSum;
         end;

         edge_type toDiagnosis : TREATMENT [1:1] -> DIAGNOSIS [1:1];

         edge_type toPrescription : TREATMENT [1:1] -> PRESCRIPTION [1:1];

         node_class DIAGNOSIS is_a BILL
            redef_derived
               CostSum = AddCosts ( 0, all self.=Clear=> );
         end;

         function AddCosts : ( sum : integer ; aNode : BILL_POSITION) -> integer =
            sum + aNode.Costs
         end;

         node_class BILL_POSITION
            meta
               Costs : integer := 0;
         end;

         edge_type yes : DIAGNOSIS -> EVIDENCE;

         edge_type no : DIAGNOSIS -> EVIDENCE;

         node_class PRESCRIPTION is_a BILL end;

         edge_type gets : PRESCRIPTION -> THERAPY;

         section WorkingTypes

            node_type LocalPractice : PRACTICE end;

            node_type Treatment : TREATMENT end;

            node_type Diagnosis : DIAGNOSIS end;

            node_type Prescription : PRESCRIPTION end;

            node_type Doctor : DOCTOR end;

            node_type Patient : PATIENT end;

         end;

      end;

      section Knowledge

         node_class EVIDENCE is_a ITEM, BILL_POSITION end;
         (* @CC_LG 6007(120,84)(97,29) @*)

         node_type Evidence : EVIDENCE end;
         (* @CC_LG 6013(96,336)(61,43) @*)

         node_class DISEASE is_a EVIDENCE end;

         node_type Disease : DISEASE end;

         node_class SYMPTOM is_a EVIDENCE end;

         node_type Symptom : SYMPTOM end;

         node_class EXAMINATION is_a EVIDENCE end;

         node_type Examination : EXAMINATION end;

         edge_type pro : EVIDENCE -> EVIDENCE;

         production AddProEdge( kind, target : string) =
            begin
               obl_node `1 : EVIDENCE;
               (* @CC_LG 6007(60,28)(95,21) @*)
               obl_node `2 : EVIDENCE;
               (* @CC_LG 6007(228,28)(95,21) @*)
            end
            (* @CC_LG 7008(8,8)(291,47) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(95,19) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(228,28)(95,19) @*)
               1' -> 2' : pro;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(291,47) @*)
            condition `1.Name = kind;
                      `2.Name = target;

         end;

         edge_type contra : EVIDENCE -> EVIDENCE;

         production AddContraEdge( kind, target : string) =
            begin
               obl_node `1 : EVIDENCE;
               (* @CC_LG 6007(60,28)(95,21) @*)
               obl_node `2 : EVIDENCE;
               (* @CC_LG 6007(228,28)(95,21) @*)
            end
            (* @CC_LG 7008(8,8)(291,47) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(95,19) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(228,28)(95,19) @*)
               1' -> 2' : contra;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(291,47) @*)
            condition `1.Name = kind;
                      `2.Name = target;

         end;

         node_class THERAPY is_a ITEM end;

         node_type Therapy : THERAPY end;

         edge_type needs : EVIDENCE -> THERAPY;

         production AddNeedsEdge( kind, target : string) =
            begin
               obl_node `1 : EVIDENCE;
               (* @CC_LG 6007(60,28)(95,21) @*)
               obl_node `2 : THERAPY;
               (* @CC_LG 6007(228,28)(95,21) @*)
            end
            (* @CC_LG 7008(8,8)(291,47) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(95,19) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(228,28)(95,19) @*)
               1' -> 2' : needs;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(291,47) @*)
            condition `1.Name = kind;
                      `2.Name = target;

         end;

         edge_type excludes : EVIDENCE -> THERAPY;

         production AddExcludesEdge( kind, target : string) =
            begin
               obl_node `1 : EVIDENCE;
               (* @CC_LG 6007(60,28)(95,21) @*)
               obl_node `2 : THERAPY;
               (* @CC_LG 6007(228,28)(89,21) @*)
            end
            (* @CC_LG 7008(8,8)(291,47) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(95,19) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(228,28)(95,19) @*)
               1' -> 2' : excludes;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(291,47) @*)
            condition `1.Name = kind;
                      `2.Name = target;

         end;

         section Base

            node_type HasCough : SYMPTOM end;
            (* @CC_LG 6013(60,364)(61,43) @*)

            node_type HighAnitHistamenica : SYMPTOM end;
            (* @CC_LG 6013(120,364)(61,43) @*)

            node_type IsChild : SYMPTOM end;
            (* @CC_LG 6013(168,350)(61,43) @*)

            node_type PseudoCroup : SYMPTOM end;
            (* @CC_LG 6013(204,364)(61,43) @*)

            node_type Stomachache : SYMPTOM end;
            (* @CC_LG 6013(228,322)(61,43) @*)

            node_type InHypogastrium : SYMPTOM end;

            node_type InEpigastrum : SYMPTOM end;

            node_type Diarrhoe : DISEASE end;

            node_type Catarrh : DISEASE end;

            node_type Influenza : DISEASE end;

            node_type AllergicTest : EVIDENCE end;

            node_type HayFever : DISEASE end;

            node_type AvoidPollen : THERAPY end;

            node_type IsSmoker : EVIDENCE end;

            node_type Alcoholic : DISEASE end;

            node_type Fibrinolyse : THERAPY end;

            node_type Makoma : THERAPY end;

            node_type Histaminika : THERAPY end;

            node_type NoCalorieDiet : THERAPY end;

         end;

      end;

   end;

   section Operations

      section BuildingOperations

         section BuildKnowledge

            transaction AddKnowledge =
                 AddTonsilitis
               & AddMorbusPfeifer
            end;

            transaction AddMoreKnowledge =
                 AddCough
               & AddAsthma
               & AddCardiacInfarction
            end;

            section Tonsilitis_Knowledge

               node_type AntiPyretic : THERAPY end;
               (* @CC_LG 6013(144,294)(61,43) @*)

               node_type SwollenTonsils : SYMPTOM end;
               (* @CC_LG 6013(60,56)(61,43) @*)

               node_type SoreThroat : SYMPTOM end;
               (* @CC_LG 6013(144,56)(61,43) @*)

               node_type Croup : SYMPTOM end;
               (* @CC_LG 6013(228,56)(61,43) @*)

               node_type Tonsilitis : SYMPTOM end;
               (* @CC_LG 6013(60,112)(61,43) @*)

               node_type Diphteria : SYMPTOM end;

               node_type HerpesSimplex : SYMPTOM end;

               node_type Cough : DISEASE end;

               node_type TakeTemperature : EXAMINATION end;

               node_type Fever : DISEASE end;

               node_type Antibiotic : THERAPY end;

               node_type BloodTest : EXAMINATION end;

               node_type VariedBloodTest : SYMPTOM end;

               production AddTonsilitis =
                  (* @CC_LG 7008(8,8)(291,33) @*)
                  ::=
                  begin
                     1' : SoreThroat;
                     (* @CC_LG 6011(60,42)(101,21) @*)
                     2' : SwollenTonsils;
                     (* @CC_LG 6011(96,97)(125,21) @*)
                     1' -> 2' : pro;
                     (* @CC_LG 3004C @*)
                     3' : Fever;
                     (* @CC_LG 6011(168,56)(71,21) @*)
                     4' : Tonsilitis;
                     (* @CC_LG 6011(168,196)(101,21) @*)
                     2' -> 4' : pro;
                     (* @CC_LG 3004 @*)
                     3' -> 4' : pro (* @CC_LG 1002(-12,28)(18,10) @*);
                     (* @CC_LG 3004MC @*)
                     5' : VariedBloodTest;
                     (* @CC_LG 6011(264,154)(149,21) @*)
                     5' -> 4' : contra (* @CC_LG 1002C,LC(30,6)(36,10) @*);
                     (* @CC_LG 3004C @*)
                     6' : Croup;
                     (* @CC_LG 6011(48,196)(71,21) @*)
                     7' : Cough;
                     (* @CC_LG 6011(48,140)(71,21) @*)
                     7' -> 6' : pro;
                     (* @CC_LG 3004 @*)
                     8' : Diphteria;
                     (* @CC_LG 6011(96,252)(95,21) @*)
                     6' -> 8' : pro;
                     (* @CC_LG 3004 @*)
                     4' -> 8' : pro;
                     (* @CC_LG 3004 @*)
                     9' : HerpesSimplex;
                     (* @CC_LG 6011(252,252)(119,21) @*)
                     4' -> 9' : pro;
                     (* @CC_LG 3004 @*)
                     10' : Antibiotic;
                     (* @CC_LG 6011(168,308)(107,21) @*)
                     8' -> 10' : needs;
                     (* @CC_LG 3004 @*)
                     12' : AntiPyretic;
                     (* @CC_LG 6011(264,98)(113,21) @*)
                     3' -> 12' : needs;
                     (* @CC_LG
                     3004C3
                     264,56 @*)
                  end
                  (* @CC_LG 7009(8,8)(351,327) @*)

               end;

            end;
            (* @CC_LG 7013(24,182)(291,435) @*)

            section MorbusPfeifer_Knowledge

               node_type SwollenSpleen : SYMPTOM end;

               node_type SwollenLiver : SYMPTOM end;

               node_type SwollenLymphNodes : SYMPTOM end;

               node_type MorbusPfeifer : SYMPTOM end;

               production AddMorbusPfeifer =
                  begin
                     obl_node `1 : SwollenTonsils;
                     (* @CC_LG 6007(84,28)(131,21) @*)
                     obl_node `2 : VariedBloodTest;
                     (* @CC_LG 6007(108,84)(155,21) @*)
                     obl_node `3 : Antibiotic;
                     (* @CC_LG 6007(144,126)(107,21) @*)
                     obl_node `4 : Fever;
                     (* @CC_LG 6007(204,28)(77,21) @*)
                  end
                  (* @CC_LG 7008(8,8)(351,145) @*)
                  ::=
                  begin
                     obl_node 1' = `1;
                     (* @CC_LG 6007(70,28)(57,19) @*)
                     obl_node 2' = `2;
                     (* @CC_LG 6007(34,84)(57,19) @*)
                     obl_node 3' = `3;
                     (* @CC_LG 6007(48,168)(57,19) @*)
                     5' : MorbusPfeifer;
                     (* @CC_LG 6011(132,126)(149,21) @*)
                     5' -> 3' : excludes;
                     (* @CC_LG
                     3004C3
                     132,168 @*)
                     2' -> 5' : pro;
                     (* @CC_LG 3004C @*)
                     obl_node 4' = `4;
                     (* @CC_LG 6007(156,28)(57,19) @*)
                     4' -> 5' : pro;
                     (* @CC_LG 3004C @*)
                     1' -> 5' : pro;
                     (* @CC_LG 3004C @*)
                     6' : SwollenSpleen;
                     (* @CC_LG 6011(228,70)(119,21) @*)
                     6' -> 5' : pro;
                     (* @CC_LG 3004C @*)
                     7' : SwollenLiver;
                     (* @CC_LG 6011(288,98)(113,21) @*)
                     7' -> 5' : pro (* @CC_LG 1002C,TC(0,10)(18,10) @*);
                     (* @CC_LG
                     3004U3
                     288,126 @*)
                     8' : SwollenLymphNodes;
                     (* @CC_LG 6011(276,168)(143,21) @*)
                     8' -> 5' : pro (* @CC_LG 1002C,TC(-10,10)(18,10) @*);
                     (* @CC_LG
                     3004C3
                     192,154 @*)
                     9' : SuperSonics;
                     (* @CC_LG 6011(288,28)(107,21) @*)
                     9' -> 6' : pro (* @CC_LG 1002C,BC(-21,-1)(18,10) @*);
                     (* @CC_LG 3004 @*)
                     9' -> 7' : pro (* @CC_LG 1002C,TC(0,10)(18,10) @*);
                     (* @CC_LG
                     3004M3
                     312,42 @*)
                  end
                  (* @CC_LG 7009(8,8)(351,187) @*)

               end;

            end;

            node_type CoughsSputum : SYMPTOM end;

            node_type InTheMorning : SYMPTOM end;

            node_type Lasting3Month : SYMPTOM end;

            node_type Smoker : DISEASE end;

            node_type InhalesNoxiousStuff : SYMPTOM end;

            node_type ChronicalBronchitis : DISEASE end;

            node_type NoSmoking : THERAPY end;

            node_type SeaClimate : THERAPY end;

            node_type AvoidNoxiousStuff : THERAPY end;

            production AddCough =
               begin
                  obl_node `1 : Cough;
                  (* @CC_LG 6007(84,28)(77,21) @*)
                  obl_node `2 : Fever;
                  (* @CC_LG 6007(204,28)(77,21) @*)
               end
               (* @CC_LG 7008(8,8)(363,47) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(96,27)(57,19) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(312,112)(57,19) @*)
                  3' : Lasting3Month;
                  (* @CC_LG 6011(72,111)(119,21) @*)
                  1' -> 3' : pro;
                  (* @CC_LG 3004C @*)
                  4' : CoughsSputum;
                  (* @CC_LG 6011(156,83)(113,21) @*)
                  1' -> 4' : pro;
                  (* @CC_LG 3004C @*)
                  5' : InTheMorning;
                  (* @CC_LG 6011(192,140)(113,21) @*)
                  4' -> 5' : pro;
                  (* @CC_LG 3004C @*)
                  6' : ChronicalBronchitis;
                  (* @CC_LG 6011(204,224)(155,21) @*)
                  5' -> 6' : pro (* @CC_LG 1002C,TC(-4,7)(18,10) @*);
                  (* @CC_LG 3004C @*)
                  7' : Smoker;
                  (* @CC_LG 6011(312,182)(77,21) @*)
                  2' -> 6' : contra (* @CC_LG 1002(-36,28)(36,10) @*);
                  (* @CC_LG 3004C @*)
                  8' : NoSmoking;
                  (* @CC_LG 6011(276,280)(95,21) @*)
                  6' -> 8' : needs (* @CC_LG 1002(23,4)(30,10) @*);
                  (* @CC_LG 3004C @*)
                  7' -> 8' : needs (* @CC_LG 1002C,TC(0,10)(30,10) @*);
                  (* @CC_LG 3004C @*)
                  9' : InhalesNoxiousStuff;
                  (* @CC_LG 6011(84,182)(155,21) @*)
                  9' -> 6' : pro;
                  (* @CC_LG
                  3004U3
                  84,224 @*)
                  10' : AvoidNoxiousStuff;
                  (* @CC_LG 6011(84,280)(149,21) @*)
                  6' -> 10' : needs (* @CC_LG 1002(-72,14)(30,10) @*);
                  (* @CC_LG 3004 @*)
                  11' : SeaClimate;
                  (* @CC_LG 6011(192,322)(131,21) @*)
                  6' -> 11' : needs (* @CC_LG 1002C,BC(10,-1)(30,10) @*);
                  (* @CC_LG 3004 @*)
                  3' -> 6' : pro (* @CC_LG 1002C,BC(-57,-10)(18,10) @*);
                  (* @CC_LG
                  3004U4
                  96,154
                  180,168 @*)
               end
               (* @CC_LG 7009(8,8)(363,341) @*)

            end;

            node_type HasAnhelations : SYMPTOM end;

            node_type InAttacks : SYMPTOM end;

            node_type LungRattles : SYMPTOM end;

            node_type CongenitallyAfflicted : SYMPTOM end;

            node_type IsAcute : SYMPTOM end;

            node_type CompliansOk : SYMPTOM end;

            node_type ResistanceOk : SYMPTOM end;

            node_type LungBreatheStertorously : SYMPTOM end;

            node_type HasAsthmaAttack : SYMPTOM end;

            node_type Asthma : DISEASE end;

            node_type HasAllergies : SYMPTOM end;

            node_type Theophillin : THERAPY end;

            node_type Fenotioral : THERAPY end;

            node_type Cortison : THERAPY end;

            node_type SecondaryEffects : DISEASE end;

            production AddAsthma =
               begin
                  obl_node `1 : ChronicalBronchitis;
                  (* @CC_LG 6007(180,28)(161,21) @*)
               end
               (* @CC_LG 7008(8,8)(399,47) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(336,224)(57,19) @*)
                  2' : HasAnhelations;
                  (* @CC_LG 6011(192,27)(125,21) @*)
                  3' : IsAcute;
                  (* @CC_LG 6011(60,168)(83,21) @*)
                  4' : InAttacks;
                  (* @CC_LG 6011(204,83)(95,21) @*)
                  2' -> 4' : pro;
                  (* @CC_LG 3004C @*)
                  5' : HasAsthmaAttack;
                  (* @CC_LG 6011(84,224)(131,21) @*)
                  3' -> 5' : pro;
                  (* @CC_LG 3004C @*)
                  6' : Asthma;
                  (* @CC_LG 6011(192,322)(77,21) @*)
                  5' -> 6' : pro (* @CC_LG 1002(-24,4)(18,10) @*);
                  (* @CC_LG
                  3004MC3
                  132,238 @*)
                  4' -> 6' : pro (* @CC_LG 1002(5,5)(18,10) @*);
                  (* @CC_LG
                  3004C3
                  180,140 @*)
                  7' : HasAllergies;
                  (* @CC_LG 6011(72,266)(113,21) @*)
                  7' -> 6' : pro (* @CC_LG 1002C,BC(-36,-7)(18,10) @*);
                  (* @CC_LG 3004MC @*)
                  8' : CongenitallyAfflicted;
                  (* @CC_LG 6011(96,112)(167,21) @*)
                  8' -> 6' : pro (* @CC_LG 1002(-24,4)(18,10) @*);
                  (* @CC_LG
                  3004MC3
                  144,126 @*)
                  9' : CompliansOk;
                  (* @CC_LG 6011(264,126)(107,21) @*)
                  9' -> 6' : pro (* @CC_LG 1002(-48,0)(18,10) @*);
                  (* @CC_LG
                  3004C3
                  192,154 @*)
                  9' -> 1' : pro (* @CC_LG 1002(34,0)(18,10) @*);
                  (* @CC_LG
                  3004C3
                  336,154 @*)
                  10' : ResistanceOk;
                  (* @CC_LG 6011(264,182)(119,21) @*)
                  10' -> 6' : contra (* @CC_LG 1002C,LC(10,0)(36,10) @*);
                  (* @CC_LG 3004C @*)
                  10' -> 1' : contra (* @CC_LG 1002C,TC(-12,0)(36,10) @*);
                  (* @CC_LG 3004C @*)
                  11' : LungRattles;
                  (* @CC_LG 6011(324,84)(113,21) @*)
                  11' -> 1' : contra (* @CC_LG 1002(0,18)(36,10) @*);
                  (* @CC_LG
                  3004C3
                  360,140 @*)
                  12' : LungBreatheStertorously;
                  (* @CC_LG 6011(300,266)(185,21) @*)
                  12' -> 6' : pro;
                  (* @CC_LG 3004 @*)
                  12' -> 1' : contra;
                  (* @CC_LG 3004 @*)
                  13' : SecondaryEffects;
                  (* @CC_LG 6011(324,322)(143,21) @*)
                  14' : Cortison;
                  (* @CC_LG 6011(348,392)(95,21) @*)
                  13' -> 14' : excludes (* @CC_LG 1002C,BC(10,50)(48,10) @*);
                  (* @CC_LG 3004 @*)
                  6' -> 14' : needs (* @CC_LG 1002C,BC(-15,-10)(30,10) @*);
                  (* @CC_LG 3004 @*)
                  15' : Fenotioral;
                  (* @CC_LG 6011(240,392)(107,21) @*)
                  6' -> 15' : needs (* @CC_LG 1002C,BC(2,14)(30,10) @*);
                  (* @CC_LG 3004 @*)
                  16' : SeaClimate;
                  (* @CC_LG 6011(84,392)(131,21) @*)
                  17' : Theophillin;
                  (* @CC_LG 6011(72,308)(113,21) @*)
                  18' : Histaminika;
                  (* @CC_LG 6011(72,364)(113,21) @*)
                  6' -> 16' : needs (* @CC_LG 1002(-12,32)(30,10) @*);
                  (* @CC_LG
                  3004U3
                  168,378 @*)
                  6' -> 18' : needs (* @CC_LG 1002C,BC(-15,8)(30,10) @*);
                  (* @CC_LG
                  3004U3
                  144,364 @*)
                  6' -> 17' : needs (* @CC_LG 1002C,BC(-23,8)(30,10) @*);
                  (* @CC_LG
                  3004U3
                  144,336 @*)
               end
               (* @CC_LG 7009(8,8)(399,411) @*)

            end;

            section Stomach_Ache_Knowledge

               node_type StomachAche : SYMPTOM end;

               node_type InUpperStomach : SYMPTOM end;

               node_type InLowerStomach : SYMPTOM end;

               node_type Diarrhoea : SYMPTOM end;

               node_type ElectroCardioGramm : EXAMINATION end;

               node_type ShowsAsphyxiaT : SYMPTOM end;

               node_type CreatininkinaseFound : SYMPTOM end;

               node_type CardiacInfarction : DISEASE end;

               node_type LipaseAmylaseEnzymHigh : SYMPTOM end;

               node_type TissueTypePlasminogenActivator : THERAPY end;

               node_type Aspirin : THERAPY end;

               production AddCardiacInfarction =
                  (* @CC_LG 7008(8,8)(291,33) @*)
                  ::=
                  begin
                     1' : StomachAche;
                     (* @CC_LG 6011(144,28)(107,21) @*)
                     2' : InUpperStomach;
                     (* @CC_LG 6011(72,84)(125,21) @*)
                     3' : InLowerStomach;
                     (* @CC_LG 6011(252,84)(125,21) @*)
                     1' -> 2' : pro;
                     (* @CC_LG 3004 @*)
                     1' -> 3' : pro;
                     (* @CC_LG 3004 @*)
                     4' : Diarrhoea;
                     (* @CC_LG 6011(156,140)(95,21) @*)
                     3' -> 4' : pro;
                     (* @CC_LG 3004 @*)
                     2' -> 4' : pro;
                     (* @CC_LG 3004 @*)
                     5' : CardiacInfarction;
                     (* @CC_LG 6011(240,252)(143,21) @*)
                     4' -> 5' : contra (* @CC_LG 1002C,LC(10,0)(36,10) @*);
                     (* @CC_LG 3004 @*)
                     2' -> 5' : pro;
                     (* @CC_LG
                     3004C3
                     96,154 @*)
                     6' : ElectroCardioGramm;
                     (* @CC_LG 6011(288,140)(149,21) @*)
                     7' : ShowsAsphyxiaT;
                     (* @CC_LG 6011(288,196)(125,21) @*)
                     6' -> 7' : pro;
                     (* @CC_LG 3004 @*)
                     7' -> 5' : pro;
                     (* @CC_LG 3004 @*)
                     8' : BloodTest;
                     (* @CC_LG 6011(60,182)(95,21) @*)
                     9' : CreatininkinaseFound;
                     (* @CC_LG 6011(84,224)(161,21) @*)
                     8' -> 9' : pro;
                     (* @CC_LG 3004 @*)
                     9' -> 5' : pro;
                     (* @CC_LG
                     3004U3
                     132,252 @*)
                     10' : Fibrinolyse;
                     (* @CC_LG 6011(72,294)(113,21) @*)
                     5' -> 10' : needs (* @CC_LG 1002C,BC(-21,-1)(30,10) @*);
                     (* @CC_LG 3004 @*)
                     11' : TissueTypePlasminogenActivator;
                     (* @CC_LG 6011(156,322)(227,21) @*)
                     5' -> 11' : needs (* @CC_LG 1002C,BC(-15,14)(30,10) @*);
                     (* @CC_LG 3004 @*)
                     12' : Aspirin;
                     (* @CC_LG 6011(324,308)(89,21) @*)
                     5' -> 12' : needs;
                     (* @CC_LG 3004 @*)
                  end
                  (* @CC_LG 7009(8,8)(375,341) @*)

               end;

            end;

            section Gastritis_Knowledge

               node_type Gastrosil : THERAPY end;

               node_type Gastritis : SYMPTOM end;

               node_type GastrosScopy : EXAMINATION end;

               node_type ShowsInflammation : SYMPTOM end;

               production AddGrastritis =
                  begin
                     obl_node `1 : Diarrhoea;
                     (* @CC_LG 6007(132,28)(101,21) @*)
                  end
                  (* @CC_LG 7008(8,8)(291,47) @*)
                  ::=
                  begin
                     obl_node 1' = `1;
                     (* @CC_LG 6007(264,28)(57,21) @*)
                     2' : GastrosScopy;
                     (* @CC_LG 6011(84,28)(119,21) @*)
                     3' : ShowsInflammation;
                     (* @CC_LG 6011(96,84)(143,21) @*)
                     2' -> 3' : pro;
                     (* @CC_LG 3004 @*)
                     4' : Gastritis;
                     (* @CC_LG 6011(216,126)(95,21) @*)
                     1' -> 4' : contra;
                     (* @CC_LG 3004 @*)
                     3' -> 4' : pro;
                     (* @CC_LG
                     3004U3
                     120,126 @*)
                  end
                  (* @CC_LG 7009(8,8)(303,145) @*)

               end;

            end;

            section Ulcer_Knowledge

               node_type ShowsMucousMembaneDefect : SYMPTOM end;

               node_type BloodInExcrements : SYMPTOM end;

               node_type GastricUlcer : SYMPTOM end;

               production AddUlcus =
                  begin
                     obl_node `1 : GastrosScopy;
                     (* @CC_LG 6007(72,28)(125,21) @*)
                     obl_node `2 : Diarrhoea;
                     (* @CC_LG 6007(240,28)(101,21) @*)
                  end
                  (* @CC_LG 7008(8,8)(303,47) @*)
                  ::=
                  begin
                     obl_node 1' = `1;
                     (* @CC_LG 6007(60,28)(57,19) @*)
                     obl_node 2' = `2;
                     (* @CC_LG 6007(240,28)(57,19) @*)
                     4' : GastricUlcer;
                     (* @CC_LG 6011(240,126)(113,21) @*)
                     2' -> 4' : contra (* @CC_LG 1002C,BC(8,-10)(36,10) @*);
                     (* @CC_LG
                     3004C3
                     264,84 @*)
                     5' : ShowsMucousMembaneDefect;
                     (* @CC_LG 6011(132,70)(185,21) @*)
                     5' -> 4' : pro;
                     (* @CC_LG 3004C @*)
                     1' -> 5' : pro (* @CC_LG 1002C,BC(10,0)(18,10) @*);
                     (* @CC_LG 3004C @*)
                     3' : BloodInExcrements;
                     (* @CC_LG 6011(84,126)(143,21) @*)
                     3' -> 4' : pro;
                     (* @CC_LG
                     3004U3
                     120,98 @*)
                  end
                  (* @CC_LG 7009(8,8)(303,145) @*)

               end;

            end;

            section Pankreatitis_Knowledge

               node_type SuperSonics : EXAMINATION end;

               node_type PancreasIsThick : SYMPTOM end;

               node_type Pancreatitis : DISEASE end;

               node_type FoodPoisoning : SYMPTOM end;

               node_type LipaseAmylaseEnzymeHigh : SYMPTOM end;

               production AddPankreatitis =
                  begin
                     obl_node `1 : BloodInExcrements;
                     (* @CC_LG 6007(84,28)(149,21) @*)
                     obl_node `2 : Diarrhoea;
                     (* @CC_LG 6007(228,56)(101,21) @*)
                     obl_node `3 : BloodTest;
                     (* @CC_LG 6007(228,28)(101,21) @*)
                     obl_node `4 : SuperSonics;
                     (* @CC_LG 6007(96,56)(113,21) @*)
                  end
                  (* @CC_LG 7008(8,8)(303,75) @*)
                  ::=
                  begin
                     obl_node 1' = `1;
                     (* @CC_LG 6007(168,224)(57,19) @*)
                     obl_node 2' = `2;
                     (* @CC_LG 6007(276,84)(57,19) @*)
                     5' : Pancreatitis;
                     (* @CC_LG 6011(72,182)(113,21) @*)
                     6' : FoodPoisoning;
                     (* @CC_LG 6011(252,182)(119,21) @*)
                     2' -> 6' : pro (* @CC_LG 1002C,TC(0,10)(18,10) @*);
                     (* @CC_LG 3004 @*)
                     obl_node 3' = `3;
                     (* @CC_LG 6007(264,28)(57,19) @*)
                     8' : LipaseAmylaseEnzymHigh;
                     (* @CC_LG 6011(96,28)(173,21) @*)
                     3' -> 8' : pro;
                     (* @CC_LG 3004 @*)
                     8' -> 5' : pro;
                     (* @CC_LG 3004 @*)
                     1' -> 6' : contra (* @CC_LG 1002C,LC(10,0)(36,10) @*);
                     (* @CC_LG 3004 @*)
                     obl_node 4' = `4;
                     (* @CC_LG 6007(168,70)(57,19) @*)
                     7' : PancreasIsThick;
                     (* @CC_LG 6011(168,126)(131,21) @*)
                     4' -> 7' : pro;
                     (* @CC_LG 3004 @*)
                     7' -> 5' : pro;
                     (* @CC_LG 3004 @*)
                  end
                  (* @CC_LG 7009(8,8)(315,243) @*)

               end;

            end;

         end;

         production CreatePractice( practiceName : string ; doctorName : string ; out practice : PRACTICE ; out doctor : DOCTOR)
         =
            (* @CC_LG 7008(8,8)(291,33) @*)
            ::=
            begin
               1' : LocalPractice;
               (* @CC_LG 6011(72,28)(119,21) @*)
               2' : Doctor;
               (* @CC_LG 6011(240,28)(83,21) @*)
               1' -> 2' : selem;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(291,47) @*)
            transfer 1'.Name := practiceName;
                     2'.Name := doctorName;
            return practice := 1';
                   doctor := 2';

         end;

         production AddPatient( practice : PRACTICE ; patientName : string ; out patient : PATIENT)
         =
            begin
               obl_node `1 = practice;
               (* @CC_LG 6007(72,28)(97,21) @*)
            end
            (* @CC_LG 7008(8,8)(291,47) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(57,19) @*)
               2' : Patient;
               (* @CC_LG 6011(228,28)(113,21) @*)
               1' -> 2' : selem;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(291,47) @*)
            transfer 2'.Name := patientName;
            return patient := 2';

         end;

         production AddTreatment( patient : PATIENT ; doctor : DOCTOR ; out examination : DIAGNOSIS)
         =
            begin
               obl_node `1 = patient;
               (* @CC_LG 6007(60,26)(89,21) @*)
               obl_node `2 = doctor;
               (* @CC_LG 6007(228,26)(83,21) @*)
               opt_node `3 : Treatment;
               (* @CC_LG 6008(60,84)(101,21) @*)
               `1 -> `3 : llast;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7008(8,8)(291,103) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(57,19) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(228,28)(57,19) @*)
               6' : Treatment;
               (* @CC_LG 6011(156,84)(95,21) @*)
               1' -> 6' : lelem (* @CC_LG 1002C,BC(-23,0)(24,10) @*);
               (* @CC_LG 3004C @*)
               2' -> 6' : does (* @CC_LG 1002C,BC(29,0)(24,10) @*);
               (* @CC_LG 3004C @*)
               4' : Diagnosis;
               (* @CC_LG 6011(72,140)(95,21) @*)
               5' : Prescription;
               (* @CC_LG 6011(216,140)(113,21) @*)
               6' -> 4' : toDiagnosis (* @CC_LG 1002C,BC(-21,-8)(66,10) @*);
               (* @CC_LG 3004C @*)
               6' -> 5' : toPrescription (* @CC_LG 1002C,BC(27,-10)(6,10) @*);
               (* @CC_LG 3004C @*)
               opt_node 3' = `3;
               (* @CC_LG 6008(36,84)(57,19) @*)
               3' -> 6' : lnext;
               (* @CC_LG 3004C @*)
               1' -> 6' : llast;
               (* @CC_LG
               3004C3
               144,42 @*)
            end
            (* @CC_LG 7009(8,8)(291,159) @*)
            return examination := 4';

         end;

         production SetCurrentTopic( diagnosis : DIAGNOSIS ; evidence : type_in EVIDENCE)
         =
            begin
               obl_node `1 = diagnosis;
               (* @CC_LG 6007(72,28)(101,21) @*)
               obl_node `2 : evidence;
               (* @CC_LG 6007(228,70)(95,21) @*)
               `1 #> `2 : Clear;
               (* @CC_LG 1002C,BC(10,-8)(18,10) @*)
            end
            (* @CC_LG 7008(8,8)(291,89) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(48,28)(57,19) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(240,84)(57,19) @*)
               3' : Current;
               (* @CC_LG 6011(144,56)(83,21) @*)
               1' -> 3' : f;
               (* @CC_LG
               3004U3
               48,56 @*)
               3' -> 2' : t;
               (* @CC_LG
               3004U3
               144,84 @*)
            end
            (* @CC_LG 7009(8,8)(291,103) @*)

         end;

         transaction RecordEvidence( examination : DIAGNOSIS ; result : string) =
            choose
               when (result = "yes")
               then
                  RecordYesEvidence ( examination )
            else
               when (result = "no")
               then
                  RecordNoEvidence ( examination )
            end
         end;

         production RecordYesEvidence( diagnosis : DIAGNOSIS) =
            begin
               obl_node `1 = diagnosis;
               (* @CC_LG 6007(60,28)(101,21) @*)
               obl_node `2 : EVIDENCE;
               (* @CC_LG 6007(312,28)(95,21) @*)
               obl_node `3 : Current;
               (* @CC_LG 6007(192,28)(89,21) @*)
               `1 -> `3 : f;
               (* @CC_LG 3004C @*)
               `3 -> `2 : t;
               (* @CC_LG 3004C @*)
            end
            (* @CC_LG 7008(8,8)(375,47) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(57,21) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(312,28)(57,21) @*)
               1' -> 2' : yes;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(375,47) @*)

         end;

         production RecordNoEvidence( diagnosis : DIAGNOSIS) =
            begin
               obl_node `1 = diagnosis;
               (* @CC_LG 6007(60,28)(101,21) @*)
               obl_node `2 : EVIDENCE;
               (* @CC_LG 6007(312,28)(95,21) @*)
               obl_node `3 : Current;
               (* @CC_LG 6007(192,28)(89,21) @*)
               `1 -> `3 : f;
               (* @CC_LG 3004C @*)
               `3 -> `2 : t;
               (* @CC_LG 3004C @*)
            end
            (* @CC_LG 7008(8,8)(375,47) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(60,28)(57,21) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(312,28)(57,21) @*)
               1' -> 2' : no;
               (* @CC_LG 3004 @*)
            end
            (* @CC_LG 7009(8,8)(375,47) @*)

         end;

         section GuideDiagnosis

            path Clear : DIAGNOSIS -> EVIDENCE =
                   -yes->
               or  -no->
            end;

            path Indicates : EVIDENCE -> EVIDENCE =
                   -pro->
               or  -contra->
            end;

            path Preconditions : EVIDENCE -> EVIDENCE =
               (     <-pro-
                 or  <-contra- )
            end;

            path Conclusion : DIAGNOSIS -> EVIDENCE =
                   (    -yes->
                     &  -pro-> )
               or  (    -no->
                     &  -contra-> )
            end;

            path Exclusion : DIAGNOSIS -> EVIDENCE =
                   (    -yes->
                     &  -contra-> )
               or  (    -no->
                     &  -pro-> )
            end;

            path Conjecture : DIAGNOSIS -> EVIDENCE =
                        Conclusion
               but_not  (     Exclusion
                          or  Clear     )
            end;

            node_class MARKER is_a ITEM end;
            (* @CC_LG 6007(144,84)(97,29) @*)

            node_type Current : MARKER end;

            edge_type f : DIAGNOSIS -> MARKER [0:1];

            edge_type t : MARKER -> EVIDENCE [1:1];

            transaction MarkCurrentTopic( diagnosis : DIAGNOSIS) =
                 MarkConjecture ( diagnosis )
               & loop
                    MarkPreCondition ( diagnosis )
                 end
            end;

            production MarkConjecture( diagnosis : DIAGNOSIS) =
               begin
                  obl_node `1 = diagnosis;
                  (* @CC_LG 6007(60,28)(101,21) @*)
                  obl_node `2 : EVIDENCE;
                  (* @CC_LG 6007(240,28)(101,21) @*)
                  `1 => `2 : Conjecture;
                  (* @CC_LG 3006 @*)
               end
               (* @CC_LG 7008(8,8)(303,47) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(60,28)(59,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(240,28)(59,21) @*)
                  3' : Current;
                  (* @CC_LG 6011(156,70)(89,21) @*)
                  1' -> 3' : f;
                  (* @CC_LG
                  3004C3
                  60,70 @*)
                  3' -> 2' : t;
                  (* @CC_LG
                  3004U3
                  240,70 @*)
               end
               (* @CC_LG 7009(8,8)(303,89) @*)

            end;

            production MarkPreCondition( diagnosis : DIAGNOSIS) =
               begin
                  obl_node `1 = diagnosis;
                  (* @CC_LG 6007(108,28)(101,21) @*)
                  obl_node `2 : EVIDENCE;
                  (* @CC_LG 6007(168,126)(95,21) @*)
                  obl_node `3 : EVIDENCE;
                  (* @CC_LG 6007(60,182)(95,21) @*)
                  `1 #> `3 : Clear;
                  (* @CC_LG 3007C @*)
                  `2 => `3 : Preconditions;
                  (* @CC_LG 3006C @*)
                  obl_node `4 : Current;
                  (* @CC_LG 6007(168,70)(89,21) @*)
                  `1 -> `4 : f;
                  (* @CC_LG 3004C @*)
                  `4 -> `2 : t (* @CC_LG 1002C,RC(-10,0)(6,10) @*);
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(224,204) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(60,28)(57,19) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(132,112)(57,19) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(240,84)(57,19) @*)
                  4' : Current;
                  (* @CC_LG 6011(156,56)(77,21) @*)
                  1' -> 4' : f;
                  (* @CC_LG 3004 @*)
                  4' -> 3' : t (* @CC_LG 1002C,BC(-4,6)(6,10) @*);
                  (* @CC_LG 3004 @*)
               end
               (* @CC_LG 7009(8,8)(291,131) @*)

            end;

         end;

         section Mark_Therapies

            path ToTherapies : DIAGNOSIS -> THERAPY =
               `1 => `3 in
               begin
                  obl_node `1 : DIAGNOSIS;
                  (* @CC_LG 6007(60,28)(101,21) @*)
                  obl_node `2 : EVIDENCE;
                  (* @CC_LG 6007(60,112)(95,21) @*)
                  obl_node `3 : THERAPY;
                  (* @CC_LG 6007(240,112)(89,21) @*)
                  `1 -> `2 : yes (* @CC_LG 1002(-23,4)(18,10) @*);
                  (* @CC_LG 3004C @*)
                  `2 -> `3 : needs (* @CC_LG 1002C,BC(-16,6)(30,10) @*);
                  (* @CC_LG 3004C @*)
                  `1 #> `3 : ToExcluded;
               end
               (* @CC_LG 7008(8,8)(291,131) @*)
            end;

            path ToExcluded : DIAGNOSIS -> THERAPY =
                  -yes->
               &  -excludes->
            end;

            production MarkTherapies( diagnosis : DIAGNOSIS) =
               begin
                  obl_node `1 = diagnosis;
                  (* @CC_LG 6007(84,98)(101,21) @*)
                  opt_set `2 : THERAPY;
                  (* @CC_LG 6009(288,98)(89,21) @*)
                  `1 => `2 : ToTherapies;
                  (* @CC_LG 3006C @*)
                  obl_node `3 : TREATMENT;
                  (* @CC_LG 6007(84,28)(101,21) @*)
                  `3 -> `1 : toDiagnosis;
                  (* @CC_LG 3004C @*)
                  obl_node `4 : PRESCRIPTION;
                  (* @CC_LG 6007(300,28)(119,21) @*)
                  `3 -> `4 : toPrescription;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(375,117) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(96,98)(57,19) @*)
                  opt_set 2' = `2;
                  (* @CC_LG 6009(264,98)(57,19) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(96,28)(57,19) @*)
                  3' -> 1' : toDiagnosis (* @CC_LG 1002C,BC(0,-10)(6,10) @*);
                  (* @CC_LG 3004C @*)
                  obl_node 4' = `4;
                  (* @CC_LG 6007(264,28)(57,19) @*)
                  3' -> 4' : toPrescription (* @CC_LG 1002C,BC(0,-10)(6,10) @*);
                  (* @CC_LG 3004C @*)
                  4' -> 2' : gets;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(327,117) @*)

            end;

            transaction YesAndMark( diagnosis : DIAGNOSIS) =
                 RecordYesEvidence ( diagnosis )
               & choose
                    MarkCurrentTopic ( diagnosis )
                 else
                    skip
                 end
            end;

            transaction NoAndMark( diagnosis : DIAGNOSIS) =
                 RecordNoEvidence ( diagnosis )
               & choose
                    MarkCurrentTopic ( diagnosis )
                 else
                    skip
                 end
            end;

         end;

      end;

   end;

   transaction Main =
      use practice : PRACTICE;
          patient : PATIENT;
          doctor : DOCTOR;
          diagnosis : DIAGNOSIS;
          evidence : type_in EVIDENCE
      do
           CreatePractice ( "General Hospital", "Dr. M. Welby", out practice, out doctor )
         & AddPatient ( practice, "W. Johnson", out patient )
         & AddTreatment ( patient, doctor, out diagnosis )
         & AddKnowledge
         & SetCurrentTopic ( diagnosis, SoreThroat )
         & YesAndMark ( diagnosis )
         & loop
              begin
                    YesAndMark ( diagnosis )
                 or NoAndMark ( diagnosis )
              end
           end
         & choose
              MarkTherapies ( diagnosis )
           else
              skip
           end
         & choose
              skip
           else
                AddPatient ( practice, "A. Smith", out patient )
              & AddMoreKnowledge
           end
      end
   end;

end.
