spec PNets

   section Scheme (* @CC_LG 1002(71,14)(36,10) @*)

      node_class ANY
         intrinsic
            index name : string := "";
            (* @CC_LG
            3011XC2
            0,42 @*)
         derived
            label : string = self.name;
            (* @CC_LG
            3013XC2
            96,0 @*)
            icon : string = string ( self.type );
            (* @CC_LG
            3013XC2
            -96,0 @*)
      end;
      (* @CC_LG 6007(288,308)(97,29) @*)

      node_type Task : ANY (* @CC_LG 3005C @*)
         intrinsic
            duration (* @CC_LG 1002TC,BC(-4,-6)(48,10) @*) : integer := 0;
            (* Estimated duration of task *)
            (* @CC_LG
            3011XC2
            72,42 @*)

         derived
            EPST (* @CC_LG 1002TC,RC(-21,-7)(24,10) @*) : integer
                                                          (* @CC_LG 4005LC,RC(-12,0)(42,12) @*)
               = max ( 0, all self.pred.EPET );
            (* Earliest possible start time *)
            (* @CC_LG
            3013XC2
            -48,-70 @*)

            EPET (* @CC_LG 1002TC(0,8)(24,10) @*) : integer = self.EPST + self.duration;
            (* Earliest possible end time *)
            (* @CC_LG
            3013XC2
            -96,-42 @*)
            (* @CC_LT VertWithIndentAndSpace @*)

            LPET (* @CC_LG 1002TC(0,14)(24,10) @*) : integer
                                                     (* @CC_LG 4005LC,RC(-12,0)(42,12) @*) = [ self is not with succ :: self.EPET
                                                                                             | min ( MaxDuration, all self.succ.LPST ) ];
            (* latest possible end time *)
            (* @CC_LG
            3013XC2
            -96,42 @*)
            (* @CC_LT VertWithIndentAndSpace @*)

            LPST (* @CC_LG 1002TC,RC(-8,12)(24,10) @*) : integer = self.LPET - self.duration;
            (* Latest possible start time *)
            (* @CC_LG
            3013XC2
            -60,70 @*)
            (* @CC_LT VertWithIndentAndSpace @*)

            TotalFloat (* @CC_LG 1002(10,7)(60,10) @*) : integer = self.LPET - self.EPST - self.duration;
            (* The maximum amount of delay if all preceding *)
            (* tasks start as early as possible and all     *)
            (* successors start as late as possible.        *)
            (* @CC_LG
            3013XC2
            -119,0 @*)
            (* @CC_LT VertWithIndentAndSpace @*)
         (* @CC_LT VertWithSpace @*)

         redef_derived
            label = self.name & "\n" & "Start: " & string ( self.EPST ) & "-" & string ( self.LPST ) & "\n" & "Time:  "
                              & string ( self.duration )                                                               ;
         (* @CC_LT VertWithIndentAndSpace @*)
      end;
      (* @CC_LG 6013(288,168)(97,29) @*)

      edge_type Prec (* @CC_LG 1002C,TC(0,9)(24,10) @*) : Task -> Task;
      (* precedence relationship *)
      (* @CC_LG
      3004M5
      321,126
      361,126
      361,154 @*)

      edge_type For (* @CC_LG 1002C,BC(0,-10)(18,10) @*) : Resource -> Task;
      (* @CC_LG 3004C @*)

      node_type Resource : ANY (* @CC_LG 3005C @*)
         intrinsic
            no : integer := 0;
            (* @CC_LG
            3011XC2
            0,42 @*)
            (* @CC_LG 3005C @*)
         redef_derived
            label = self.name & "\n" & "No: " & string ( self.no );
      end;
      (* @CC_LG 6013(492,168)(97,29) @*)

   end;
   (* @CC_LG 7013(84,84)(553,393) @*)

   section TaskOperations

      constraint
         for
               obl_node `1 : Task;
               (* @CC_LG 6007(48,28)(71,21) @*)
               obl_node `2 : Task;
               (* @CC_LG 6007(168,28)(71,21) @*)
               `1 -> `2 : Prec;
               (* @CC_LG 3004 @*)
            (* @CC_LG 7008C(219,61) @*)
         ensure
               old_obl_node `1;
               (* @CC_LG 6007(36,28)(29,19) @*)
               old_obl_node `2;
               (* @CC_LG 6007(156,28)(29,19) @*)
               `2 #> `1 : Prec +;
               (* @CC_LG 3007C @*)
            (* @CC_LG 7008C(183,61) @*)
      end;

      function MaxDuration : -> integer =
         100
      end;

      function max : ( v1, v2 : integer) -> integer =
         [ v1 < v2 :: v2
         | v1 ]
      end;

      function min : ( v1, v2 : integer) -> integer =
         [ v1 > v2 :: v2
         | v1 ]
      end;

      path succ : Task -> Task =
         -Prec->
      end;

      path pred : Task -> Task =
         <-Prec-
      end;

      production EnterDuration( Name : string ; Duration : integer) =
         begin
            obl_node `1 : Task;
            (* @CC_LG 6007(47,20)(71,21) @*)
         end
         (* @CC_LG 7008(8,8)(95,41) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(41,20)(59,21) @*)
         end
         (* @CC_LG 7009(8,8)(83,41) @*)
         condition `1.name = Name;
         transfer 1'.duration := Duration;
      end;

      production ChangeDuration( T : Task ; Duration : integer) =
         begin
            obl_node `1 = T;
            (* @CC_LG 6007(38,20)(53,21) @*)
         end
         (* @CC_LG 7008(8,8)(77,41) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(41,20)(59,21) @*)
         end
         (* @CC_LG 7009(8,8)(83,41) @*)
         transfer 1'.duration := Duration;
      end;

      production ChangeName( T : Task ; Name : string) =
         begin
            obl_node `1 = T;
            (* @CC_LG 6007(38,20)(53,21) @*)
         end
         (* @CC_LG 7008(8,8)(77,41) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(41,20)(59,21) @*)
         end
         (* @CC_LG 7009(8,8)(83,41) @*)
         transfer 1'.name := Name;
      end;

      production CreateDiagram( StartName, StopName : string) =
         (<OptLeftSideList>)
         (* @CC_LG 7008(8,8)(24,24) @*)
         ::=
         begin
            1' : Task;
            (* @CC_LG 6011(47,20)(71,21) @*)
            2' : Task;
            (* @CC_LG 6011(47,104)(71,21) @*)
            1' -> 2' : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(95,125) @*)
         transfer 1'.name := StartName;
                  2'.name := StopName;
      end;

      production CreateTask( Name : string) =
         (<OptLeftSideList>)
         (* @CC_LG 7008(8,8)(24,24) @*)
         ::=
         begin
            1' : Task;
            (* @CC_LG 6011(47,20)(71,21) @*)
         end
         (* @CC_LG 7009(8,8)(95,41) @*)
         transfer 1'.name := Name;
      end;

      production InsertTask( FromName, NewName, ToName : string) =
         begin
            obl_node `1 : Task;
            (* @CC_LG 6007(47,20)(71,21) @*)
            obl_node `2 : Task;
            (* @CC_LG 6007(47,84)(71,21) @*)
            `1 -> `2 : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(95,105) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(47,20)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(47,140)(59,21) @*)
            3' : Task;
            (* @CC_LG 6011(47,84)(71,21) @*)
            1' -> 3' : Prec;
            (* @CC_LG 3004C @*)
            3' -> 2' : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(95,161) @*)
         condition `1.name = FromName;
                   `2.name = ToName;
         transfer 3'.name := NewName;
      end;

      production ParallelTask( FromName, NewName, ToName : string) =
         begin
            obl_node `1 : Task;
            (* @CC_LG 6007(47,20)(71,21) @*)
            obl_node `2 : Task;
            (* @CC_LG 6007(47,84)(71,21) @*)
            `1 -> `2 : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(95,105) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(119,20)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(120,140)(59,21) @*)
            3' : Task;
            (* @CC_LG 6011(47,84)(71,21) @*)
            1' -> 3' : Prec (* @CC_LG 1002C,RC(-10,-1)(24,10) @*);
            (* @CC_LG 3004C @*)
            3' -> 2' : Prec (* @CC_LG 1002C,RC(-10,0)(24,10) @*);
            (* @CC_LG 3004C @*)
            1' -> 2' : Prec (* @CC_LG 1002C,TC(0,10)(24,10) @*);
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(166,161) @*)
         condition `1.name = FromName;
                   `2.name = ToName;
         transfer 3'.name := NewName;
      end;

      production CreatePrecedence( From, To : Task) =
         begin
            obl_node `1 = From;
            (* @CC_LG 6007(47,28)(71,21) @*)
            obl_node `2 = To;
            (* @CC_LG 6007(47,94)(71,21) @*)
            `2 #> `1 : Prec +
                       (* @CC_LG 4005C,TC(0,15)(36,12) @*);
            (* @CC_LG 3007C @*)
         end
         (* @CC_LG 7008(8,8)(115,117) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(46,20)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(46,84)(59,21) @*)
            1' -> 2' : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(88,105) @*)
      end;

      production DeleteTask( T : Task) =
         begin
            opt_set `1 : Task;
            (* @CC_LG 6009(48,42)(71,21) @*)
            obl_node `2 = T;
            (* @CC_LG 6007(156,42)(53,21) @*)
            opt_set `3 : Task;
            (* @CC_LG 6009(264,42)(71,21) @*)
            `1 -> `2 : Prec;
            (* @CC_LG 3004C @*)
            `2 -> `3 : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(315,77) @*)
         ::=
         begin
            opt_set 1' = `1;
            (* @CC_LG 6009(45,42)(59,21) @*)
            opt_set 3' = `3;
            (* @CC_LG 6009(168,42)(59,21) @*)
            1' -> 3' : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(223,77) @*)
      end;

      test CriticalPath( out TSet : Task [1:n]) =
         begin
            obl_set `1 : Task;
            (* @CC_LG 6010(93,56)(71,21) @*)
            => `1 : valid (self.TotalFloat = 0);
            (* @CC_LG
            3009XC2
            93,25 @*)
         end
         (* @CC_LG 7008(8,8)(186,77) @*)
         return TSet := `1;

      end;

      test Duration( out T : integer) =
         begin
            obl_node `1 : Task;
            (* @CC_LG 6007(47,20)(71,21) @*)
            not_node `2 : Task;
            (* @CC_LG 6012(47,84)(71,21) @*)
            `1 -> `2 : Prec;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(95,105) @*)
         return T := `1.EPET;

      end;

      query Analyse( out Path : Task [1:n] ; out Time : integer) =
           CriticalPath ( out Path )
         & Duration ( out Time )
      end;

   end;
   (* @CC_LG 7013(432,42)(156,30) @*)

   section ResourceOperations

      path overlap : Task -> Task =
         `1 => `2 in
         begin
            obl_node `1 : Task;
            (* @CC_LG 6007(47,20)(71,21) @*)
            obl_node `2 : Task;
            (* @CC_LG 6007(167,20)(71,21) @*)
         end
         (* @CC_LG 7008(8,8)(215,41) @*)
         condition (`1.EPET > `2.LPST) and (`2.EPET > `1.LPST);
      end;

      path equal : Resource -> Resource =
         `1 => `2 in
         begin
            obl_node `1 : Resource;
            (* @CC_LG 6007(59,20)(95,21) @*)
            obl_node `2 : Resource;
            (* @CC_LG 6007(203,20)(95,21) @*)
         end
         (* @CC_LG 7008(8,8)(263,41) @*)
         condition `1.name = `2.name;
      end;

      production CreateResource( Name : string ; No : integer) =
         (<OptLeftSideList>)
         (* @CC_LG 7008(8,8)(24,24) @*)
         ::=
         begin
            1' : Resource;
            (* @CC_LG 6011(59,20)(95,21) @*)
         end
         (* @CC_LG 7009(8,8)(119,41) @*)
         transfer 1'.name := Name;
                  1'.no := No;
      end;

      production DeleteResource( R : Resource) =
         begin
            obl_node `1 = R;
            (* @CC_LG 6007(38,20)(53,21) @*)
         end
         (* @CC_LG 7008(8,8)(77,41) @*)
         ::=
         (<OptRightSideList>)
         (* @CC_LG 7009(8,8)(24,24) @*)
      end;

      production ScheduleResource( ResourceName, TaskName : string) =
         begin
            obl_node `1 : Resource;
            (* @CC_LG 6007(59,20)(95,21) @*)
            obl_node `2 : Task;
            (* @CC_LG 6007(179,20)(71,21) @*)
         end
         (* @CC_LG 7008(8,8)(227,41) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(41,20)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(41,84)(59,21) @*)
            1' -> 2' : For;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(83,105) @*)
         condition `1.name = ResourceName;
                   `2.name = TaskName;
      end;

      test FindConflict( out R : Resource ; out Task1, Task2 : Task) =
         begin
            obl_node `1 : Resource;
            (* @CC_LG 6007(119,20)(95,21) @*)
            obl_node `2 : Task;
            (* @CC_LG 6007(47,104)(71,21) @*)
            obl_node `3 : Task;
            (* @CC_LG 6007(190,104)(71,21) @*)
            `1 -> `2 : For;
            (* @CC_LG 3004C @*)
            `1 -> `3 : For;
            (* @CC_LG 3004C @*)
            `2 => `3 : overlap;
            (* @CC_LG 3006C @*)
         end
         (* @CC_LG 7008(8,8)(238,125) @*)
         return R := `1;
                Task1 := `2;
                Task2 := `3;

      end;

      transaction RemoveConflict( R : Resource ; Task1, Task2 : Task) =
         choose
            Reschedule ( R, Task1, Task2 )
         else
            CreatePrecedence ( Task1, Task2 )
         end
      end;

      production Reschedule( R : Resource ; Task1, Task2 : Task) =
         begin
            obl_node `1 = R;
            (* @CC_LG 6007(59,29)(53,21) @*)
            obl_node `2 : Task;
            (* @CC_LG 6007(58,113)(71,21) @*)
            obl_node `3 : Task;
            (* @CC_LG 6007(201,113)(71,21) @*)
            `1 -> `2 : For;
            (* @CC_LG 3004C @*)
            `1 -> `3 : For (* @CC_LG 1002C,BC(1,-10)(18,10) @*);
            (* @CC_LG 3004C @*)
            `2 => `3 : overlap;
            (* @CC_LG 3006C @*)
            obl_node `4 : Resource;
            (* @CC_LG 6007(203,29)(95,21) @*)
            `4 +> `3 : For;
            (* @CC_LG 3008C @*)
            not_node `5 : Task;
            (* @CC_LG 6012(332,113)(71,21) @*)
            `3 => `5 : overlap;
            (* @CC_LG 3006C @*)
            `4 -> `5 : For (* @CC_LG 1002C,BC(6,-10)(18,10) @*);
            (* @CC_LG 3004C @*)
            `1 => `4 : equal;
            (* @CC_LG 3006C @*)
         end
         (* @CC_LG 7008(8,8)(380,134) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(42,20)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(41,104)(59,21) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(184,104)(59,21) @*)
            1' -> 2' : For (* @CC_LG 1002C,BC(0,-10)(18,10) @*);
            (* @CC_LG 3004C @*)
            obl_node 4' = `4;
            (* @CC_LG 6007(186,20)(59,21) @*)
            4' -> 3' : For;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(228,125) @*)
         condition `2 in (Task1 or Task2);
                   `3 in (Task1 or Task2);
      end;

   end;
   (* @CC_LG 7013(216,42)(180,30) @*)

   section Test

      transaction CreatePlanForGantryProject =
           CreateDiagram ( "Mark out",
                            "Erect span" (* @CC_LT VertWithIndent @*) )
         & InsertTask ( "Mark out",
                         "Complete base A" (* @CC_LT VertWithIndent @*),
                         "Erect span" (* @CC_LT VertWithIndent @*) )
         & ParallelTask ( "Mark out", "Dig A",
                           "Complete base A" (* @CC_LT VertWithIndent @*) )
         & InsertTask ( "Dig A", "Concrete A",
                         "Complete base A" (* @CC_LT VertWithIndent @*) )
         & InsertTask ( "Mark out", "Dig B",
                         "Complete base A" (* @CC_LT VertWithIndent @*) )
         & InsertTask ( "Dig B", "Concrete B",
                         "Complete base A" (* @CC_LT VertWithIndent @*) )
         & InsertTask ( "Concrete B",
                         "Erect plinth" (* @CC_LT VertWithIndent @*),
                         "Complete base A" (* @CC_LT VertWithIndent @*) )
         & ParallelTask ( "Complete base A",
                           "Erect tower A" (* @CC_LT VertWithIndent @*),
                           "Erect span" (* @CC_LT VertWithIndent @*) )
         & InsertTask ( "Complete base A",
                         "Erect tower B" (* @CC_LT VertWithIndent @*),
                         "Erect span" (* @CC_LT VertWithIndent @*) )
         & EnterDuration ( "Mark out", 1 )
         & EnterDuration ( "Dig A", 2 )
         & EnterDuration ( "Dig B", 4 )
         & EnterDuration ( "Concrete A", 1 )
         & EnterDuration ( "Concrete B", 3 )
         & EnterDuration ( "Erect plinth", 2 )
         & EnterDuration ( "Complete base A", 1 )
         & EnterDuration ( "Erect tower A", 1 )
         & EnterDuration ( "Erect tower B", 1 )
         & EnterDuration ( "Erect span", 1 )
      end;

      transaction AssignResourcesToProject =
           CreateResource ( "Excavator", 1 )
         & CreateResource ( "Concrete-Mixer", 1 )
         & CreateResource ( "Crane", 1 )
         & ScheduleResources
      end;

      transaction ScheduleResources =
           ScheduleResource ( "Excavator", "Dig A" )
         & ScheduleResource ( "Excavator", "Dig B" )
         & ScheduleResource ( "Concrete-Mixer", "Concrete A" )
         & ScheduleResource ( "Concrete-Mixer", "Concrete B" )
         & ScheduleResource ( "Concrete-Mixer",
                               "Complete base A" (* @CC_LT VertWithIndent @*) )
         & ScheduleResource ( "Crane",
                               "Erect plinth" (* @CC_LT VertWithIndent @*) )
         & ScheduleResource ( "Crane",
                               "Erect tower A" (* @CC_LT VertWithIndent @*) )
         & ScheduleResource ( "Crane",
                               "Erect tower B" (* @CC_LT VertWithIndent @*) )
         & ScheduleResource ( "Crane",
                               "Erect span" (* @CC_LT VertWithIndent @*) )
      end;

   end;
   (* @CC_LG 7013(84,42)(96,30) @*)

   transaction MAIN =
        CreatePlanForGantryProject
      & AssignResourcesToProject
      & use CriticalPath : Task [0:n];
            Time : integer;
            R : Resource;
            Task1, Task2 : Task
        do
             Analyse ( out CriticalPath,
                        out Time
                        (* @CC_LT VertWithIndent @*) )
           & loop
                  FindConflict ( out R, out Task1, out Task2 )
                & RemoveConflict ( R, Task1, Task2 )
             end
           & Analyse ( out CriticalPath, out Time )
        end
   end;

end.
(* @CC_LG 7012(8,8)(651,490) @*)
