spec PhascumCuspidatum (* The specification modells the first five *)
                       (* steps of the development process of the  *)
                       (* moss Phascum Cuspidatum. This is one of  *)
                       (* the popular examples in the literature   *)
                       (* about so-called L(indenmeyer) systems.   *)
                       (* L systems are closely related to graph   *)
                       (* grammars and usually used to model the   *)
                       (* development process of plants.           *)
                       (*                                          *)
                       (* Introduced node and edge types have the  *)
                       (* following meaning:                       *)
                       (*                                          *)
                       (* P = primary cell                         *)
                       (* S = secondary cell (six types of cells)  *)
                       (* T = tertiary cell                        *)
                       (*                                          *)
                       (* midv = middle vein (of leaf)             *)
                       (* branch = branch of middle vein           *)
                       (* peri = periphery (border) of leaf        *)

   node_class ANY end;

   node_type P : ANY end;

   node_type S : ANY end;

   node_type S1 : ANY end;

   node_type S2 : ANY end;

   node_type S3 : ANY end;

   node_type S4 : ANY end;

   node_type S5 : ANY end;

   node_type T : ANY end;

   edge_type midv : ANY -> ANY;

   edge_type peri : ANY -> ANY;

   edge_type branch : ANY -> ANY;

   production Create (* @CC_LG 7008(8,8)(405,45) @*)
                     (* @CC_LG 7008(8,8)(20,45) @*)
                     (* @CC_LG 7008(8,8)(20,45) @*)
                     (* @CC_LG 7008(8,8)(20,45) @*)
                     (* @CC_LG 7008(8,8)(20,45) @*)
                     (* @CC_LG 7008(8,8)(20,45) @*)  =
      (* @CC_LG 7008(8,8)(36,38) @*)
      ::=
      begin
         1' : P;
         (* @CC_LG 6011(38,20)(53,21) @*)
      end
      (* @CC_LG 7009(8,8)(77,41) @*)

   end;

   production p1 * =
      begin
         obl_node `1 : P;
         (* @CC_LG 6007(136,20)(59,21) @*)
         opt_set `3 : S2;
         (* @CC_LG 6009(230,84)(59,21) @*)
         opt_set `4 : S1;
         (* @CC_LG 6009(38,84)(53,21) @*)
         `4 -> `1 : peri;
         (* @CC_LG 3004C @*)
         `3 -> `1 : peri;
         (* @CC_LG 3004C @*)
         opt_set `5 : ANY;
         (* @CC_LG 6009(134,84)(65,21) @*)
         `5 -> `1 : midv;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7008(8,8)(275,105) @*)
      ::=
      begin
         opt_set 3' = `3;
         (* @CC_LG 6009(223,140)(59,21) @*)
         opt_set 4' = `4;
         (* @CC_LG 6009(41,140)(59,21) @*)
         1' : P;
         (* @CC_LG 6011(132,40)(53,21) @*)
         2' : S;
         (* @CC_LG 6011(175,84)(53,21) @*)
         3' -> 2' : peri;
         (* @CC_LG 3004C @*)
         4' -> 1' : peri;
         (* @CC_LG 3004C @*)
         2' -> 1' : midv;
         (* @CC_LG 3004C @*)
         2' -> 1' : peri (* @CC_LG 1002C,LC(27,0)(24,10) @*);
         (* @CC_LG 
         3004C3
         192,56 @*)
         opt_set 5' = `5;
         (* @CC_LG 6009(127,140)(59,21) @*)
         5' -> 2' : midv;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7009(8,8)(268,161) @*)
      folding { `4, `5 }, { `3, `5 };

   end;

   production p2 * =
      begin
         obl_node `1 : S;
         (* @CC_LG 6007(144,56)(53,21) @*)
         opt_set `2 : ANY;
         (* @CC_LG 6009(44,28)(65,21) @*)
         opt_set `3 : ANY;
         (* @CC_LG 6009(44,84)(65,21) @*)
         opt_set `4 : ANY;
         (* @CC_LG 6009(240,23)(65,21) @*)
         opt_set `5 : ANY;
         (* @CC_LG 6009(240,84)(65,21) @*)
         `2 -> `1 : midv;
         (* @CC_LG 3004C @*)
         `1 -> `4 : midv;
         (* @CC_LG 3004C @*)
         `3 -> `1 : peri;
         (* @CC_LG 3004C @*)
         `1 -> `5 : peri;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7008(8,8)(289,105) @*)
      ::=
      begin
         1' : S1;
         (* @CC_LG 6011(144,51)(59,21) @*)
         opt_set 2' = `2;
         (* @CC_LG 6009(41,23)(59,21) @*)
         opt_set 3' = `3;
         (* @CC_LG 6009(41,79)(59,21) @*)
         opt_set 4' = `4;
         (* @CC_LG 6009(240,23)(59,21) @*)
         opt_set 5' = `5;
         (* @CC_LG 6009(240,79)(59,21) @*)
         2' -> 1' : midv;
         (* @CC_LG 3004C @*)
         1' -> 4' : midv;
         (* @CC_LG 3004C @*)
         3' -> 1' : peri;
         (* @CC_LG 3004C @*)
         1' -> 5' : peri;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7009(8,8)(285,100) @*)
      folding { `2, `3 }, { `4, `5 };

   end;

   production p3 * =
      begin
         obl_node `1 : S1;
         (* @CC_LG 6007(148,56)(59,21) @*)
         opt_set `2 : ANY;
         (* @CC_LG 6009(44,28)(65,21) @*)
         `2 -> `1 : peri;
         (* @CC_LG 3004C @*)
         opt_set `3 : ANY;
         (* @CC_LG 6009(236,23)(65,21) @*)
         `1 -> `3 : peri;
         (* @CC_LG 3004C @*)
         opt_set `4 : ANY;
         (* @CC_LG 6009(44,84)(65,21) @*)
         `4 -> `1 : midv;
         (* @CC_LG 3004C @*)
         opt_set `5 : ANY;
         (* @CC_LG 6009(236,84)(65,21) @*)
         `1 -> `5 : midv;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7008(8,8)(285,105) @*)
      ::=
      begin
         1' : S2;
         (* @CC_LG 6011(194,63)(59,21) @*)
         6' : T;
         (* @CC_LG 6011(103,84)(53,21) @*)
         6' -> 1' : branch (* @CC_LG 1002C,BC(14,16)(36,10) @*);
         (* @CC_LG 3004C @*)
         opt_set 2' = `2;
         (* @CC_LG 6009(103,28)(59,21) @*)
         2' -> 1' : peri;
         (* @CC_LG 3004C @*)
         opt_set 3' = `3;
         (* @CC_LG 6009(295,23)(59,21) @*)
         1' -> 3' : peri;
         (* @CC_LG 
         3004C3
         271,63 @*)
         opt_set 4' = `4;
         (* @CC_LG 6009(41,140)(59,21) @*)
         4' -> 6' : midv;
         (* @CC_LG 
         3004C3
         92,140 @*)
         opt_set 5' = `5;
         (* @CC_LG 6009(175,140)(59,21) @*)
         6' -> 5' : midv;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7009(8,8)(340,161) @*)
      folding { `2, `4 }, { `3, `5 };

   end;

   production p4 * =
      begin
         obl_node `1 : S2;
         (* @CC_LG 6007(142,26)(59,21) @*)
         opt_set `2 : ANY;
         (* @CC_LG 6009(188,84)(65,21) @*)
         `2 -> `1 : branch (* @CC_LG 1002(-34,-25)(36,10) @*);
         (* @CC_LG 3004C @*)
         opt_set `3 : ANY;
         (* @CC_LG 6009(44,84)(65,21) @*)
         opt_set `4 : ANY;
         (* @CC_LG 6009(44,26)(65,21) @*)
         `3 -> `1 : peri;
         (* @CC_LG 3004C @*)
         `1 -> `4 : peri;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7008(8,8)(237,105) @*)
      ::=
      begin
         1' : S3;
         (* @CC_LG 6011(151,26)(59,21) @*)
         5' : S3;
         (* @CC_LG 6011(151,84)(59,21) @*)
         5' -> 1' : peri;
         (* @CC_LG 3004C @*)
         opt_set 2' = `2;
         (* @CC_LG 6009(271,83)(59,21) @*)
         2' -> 1' : branch;
         (* @CC_LG 3004C @*)
         2' -> 5' : branch;
         (* @CC_LG 3004C @*)
         opt_set 3' = `3;
         (* @CC_LG 6009(51,112)(59,21) @*)
         3' -> 5' : peri;
         (* @CC_LG 3004C @*)
         opt_set 4' = `4;
         (* @CC_LG 6009(41,28)(59,21) @*)
         1' -> 4' : peri (* @CC_LG 1002C,BC(0,-10)(24,10) @*);
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7009(8,8)(316,133) @*)

   end;

   production p5 * =
      begin
         obl_node `1 : S3;
         (* @CC_LG 6007(58,84)(59,21) @*)
         opt_set `2 : ANY;
         (* @CC_LG 6009(58,23)(65,21) @*)
         `2 -> `1 : branch;
         (* @CC_LG 3004C @*)
         opt_set `3 : ANY;
         (* @CC_LG 6009(144,23)(65,21) @*)
         opt_set `4 : ANY;
         (* @CC_LG 6009(58,145)(65,21) @*)
         `3 -> `1 : peri;
         (* @CC_LG 3004C @*)
         `1 -> `4 : peri;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7008(8,8)(193,166) @*)
      ::=
      begin
         1' : S4;
         (* @CC_LG 6011(58,84)(59,21) @*)
         opt_set 2' = `2;
         (* @CC_LG 6009(58,23)(59,21) @*)
         opt_set 3' = `3;
         (* @CC_LG 6009(144,23)(59,21) @*)
         opt_set 4' = `4;
         (* @CC_LG 6009(58,140)(59,21) @*)
         3' -> 1' : peri;
         (* @CC_LG 3004C @*)
         1' -> 4' : peri;
         (* @CC_LG 3004C @*)
         2' -> 1' : branch;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7009(8,8)(189,161) @*)

   end;

   production p6 * =
      begin
         obl_node `1 : S4;
         (* @CC_LG 6007(156,26)(59,21) @*)
         opt_set `2 : ANY;
         (* @CC_LG 6009(244,86)(65,21) @*)
         `2 -> `1 : branch;
         (* @CC_LG 3004C @*)
         opt_set `3 : ANY;
         (* @CC_LG 6009(52,84)(65,21) @*)
         `3 -> `1 : peri;
         (* @CC_LG 3004C @*)
         opt_set `4 : ANY;
         (* @CC_LG 6009(44,26)(65,21) @*)
         `1 -> `4 : peri;
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7008(8,8)(293,107) @*)
      ::=
      begin
         1' : S5;
         (* @CC_LG 6011(166,27)(59,21) @*)
         opt_set 2' = `2;
         (* @CC_LG 6009(285,112)(59,21) @*)
         opt_set 3' = `3;
         (* @CC_LG 6009(41,25)(59,21) @*)
         3' -> 1' : peri;
         (* @CC_LG 3004C @*)
         opt_set 4' = `4;
         (* @CC_LG 6009(45,84)(59,21) @*)
         1' -> 4' : peri;
         (* @CC_LG 3004C @*)
         5' : T;
         (* @CC_LG 6011(285,56)(53,21) @*)
         5' -> 1' : branch;
         (* @CC_LG 3004C @*)
         2' -> 5' : branch (* @CC_LG 1002C,TC(0,10)(36,10) @*);
         (* @CC_LG 3004C @*)
      end
      (* @CC_LG 7009(8,8)(342,133) @*)

   end;

   transaction MAIN (* The main transaction which regulates *)
                    (* the application of all productions.  *)
                    (* It's implementation is a little bit  *)
                    (* due to the following problem:        *)
                    (*                                      *)
                    (* The original version of the spec. in *)
                    (*                                      *)
                    (* Manfred Nagl: Graphgrammatiken       *)
                    (* Vieweg Verlag 1979                   *)
                    (*                                      *)
                    (* requires the parallel application of *)
                    (* all rules to all occurrences of left *)
                    (* hand sides. PROGRES supports the     *)
                    (* application of a single rule to all  *)
                    (* occurrences of its left hand side    *)
                    (* (via the * annotation in the rule's  *)
                    (* declaration), but not the parallel   *)
                    (* application of a set of rules.       *)
                    (* Therefore, we have to take care that *)
                    (* rule x rewrites in development step  *)
                    (* n only nodes created by rule y in    *)
                    (* development step n-1. The simplest   *)
                    (* way to guarantee such a behaviour of *)
                    (* rewrite rules is the reverse sequ.   *)
                    (* application of rules p1 through p6.  *) =
        Create
      & loop
             p6
           & p5
           & p4
           & p3
           & p2
           & p1
        end
   end;

end.
