spec RedBlackTrees

   section scheme

      node_class RBNODE
         intrinsic
            key Value : integer;
            GRAPH_BROWSER_LABEL : string;
         derived
            Height : integer = max ( self.lHeight, self.rHeight );
            lHeight = [ self.-bl->.Height + 1
                      | self.-rl->.Height
                      | 0 ]                  ;
            rHeight = [ self.-br->.Height + 1
                      | self.-rr->.Height
                      | 0 ]                  ;
      end;
      (* @CC_LG 6007(205,77)(97,29) @*)

      edge_type bl : RBNODE [0:1] -> RBNODE [0:1];
      (* @CC_LG 
      3004M5
      274,78
      274,112
      234,112 @*)

      edge_type br : RBNODE [0:1] -> RBNODE [0:1];
      (* @CC_LG 
      3004M5
      274,78
      274,112
      234,112 @*)

      edge_type rl : RBNODE [0:1] -> RBNODE [0:1];
      (* @CC_LG 
      3004M5
      274,78
      274,112
      234,112 @*)

      edge_type rr : RBNODE [0:1] -> RBNODE [0:1];
      (* @CC_LG 
      3004M5
      274,78
      274,112
      234,112 @*)

      static path right : RBNODE [0:1] -> RBNODE [0:1] =
         [ -br->
         | -rr-> ]
      end;
      (* @CC_LG 
      3006M5
      274,78
      274,112
      234,112 @*)

      static path left : RBNODE [0:1] -> RBNODE [0:1] =
         [ -bl->
         | -rl-> ]
      end;
      (* @CC_LG 
      3006M5
      274,78
      274,112
      234,112 @*)

      path parent : RBNODE [0:1] -> RBNODE [0:1] =
         [ <=left=
         | <=right= ]
      end;
      (* @CC_LG 
      3006M5
      274,78
      274,112
      234,112 @*)

      path down( Val : integer) : RBNODE -> RBNODE [0:1] =
         [ valid (Val < self.Value) :: =left=>
         | valid (Val > self.Value) :: =right=> ]
      end;
      (* @CC_LG 
      3006M5
      274,78
      274,112
      234,112 @*)

      function df : ( Src : RBNODE ; Val : integer) -> RBNODE [0:1] =
         [ Val < Src.Value :: Src.=left=>
         | Val > Src.Value :: Src.=right=> ]
      end;

      restriction balanced : RBNODE =
         valid (self.lHeight = self.rHeight)
      end;
      (* @CC_LG 
      3009XC2
      205,115 @*)

      restriction correctColoured : RBNODE =
         not with (    (     -rl->
                         or  -rr-> )
                    &  (     -rl->
                         or  -rr-> ) )
      end;
      (* @CC_LG 
      3009XC2
      205,115 @*)

      node_type RBNode : RBNODE end;
      (* @CC_LG 6013(181,77)(47,29) @*)

      function max : ( x : integer ; y : integer) -> integer =
         [ x <= y :: y
         | x ]
      end;

      test GetRoot( out nn : RBNODE) =
         begin
            obl_node `1 : RBNODE;
            (* @CC_LG 6007(87,104)(83,21) @*)
            => `1 : not with (     <-bl-
                               or  <-rl-
                               or  <-br-
                               or  <-rr- );
            (* @CC_LG 
            3009XC2
            78,61 @*)
         end
         (* @CC_LG 7008(8,8)(156,125) @*)
         return nn := `1;

      end;

   end;
   (* @CC_LG 7013(12,27)(417,142) @*)

   section main

      transaction MAIN =
         use i := 1
         do
              InsertFirst ( 20 )
            & loop
                   Insert ( i )
                 & i := (i + 1)
              end
         end
      end;

      transaction Rotate( n : RBNODE ; out nn : RBNODE) =
           nn := n
         & loop
                 RotateLR ( n, out nn )
              or RotateRL ( n, out nn )
              or RotateLL ( n, out nn )
              or RotateRR ( n, out nn )
           end
      end;

      test FindInconsistencies( out S : RBNODE [1:n]) =
         begin
            obl_node `X : RBNODE;
            (* @CC_LG 6007(84,73)(83,21) @*)
            => `X :     not balanced
                    or  not correctColoured;
            (* @CC_LG 
            3009XC2
            81,37 @*)
         end
         (* @CC_LG 7008(8,8)(162,94) @*)
         return S := `X;

      end;

      transaction Insert( Val : integer) =
         use n : RBNODE
         do
              GetRoot ( out n )
            & loop
                 choose
                      Split ( n, out n )
                    & Rotate ( n, out n )
                 else
                    n := (n.down ( Val ))
                 end
              end
            & begin
                    InsertRight ( n, Val, out n )
                 or InsertLeft ( n, Val, out n )
              end
            & Rotate ( n, out n )
         end
      end;

      production InsertFirst( Val : integer) =
         (<OptPrecondDecl>)
         (* @CC_LG 7008(8,8)(291,145) @*)
         (<OptLeftSideList>)
         (* @CC_LG 7008(8,8)(24,24) @*)
         ::=
         begin
            x' : RBNode;
            (* @CC_LG 6011(53,20)(83,21) @*)
         end
         (* @CC_LG 7009(8,8)(107,41) @*)
         transfer x'.Value := Val;
                  x'.GRAPH_BROWSER_LABEL := string ( Val );
      end;

      production Split( n : RBNODE ; out nn : RBNODE) =
         begin
            obl_node `x = n;
            (* @CC_LG 6007(101,90)(53,21) @*)
            obl_node `y : RBNODE;
            (* @CC_LG 6007(53,160)(83,21) @*)
            obl_node `z : RBNODE;
            (* @CC_LG 6007(161,160)(83,21) @*)
            `x -> `y : rl;
            (* @CC_LG 3004C @*)
            `x -> `z : rr;
            (* @CC_LG 3004C @*)
            opt_node `a : RBNODE;
            (* @CC_LG 6008(269,20)(83,21) @*)
            opt_node `b : RBNODE;
            (* @CC_LG 6008(101,20)(83,21) @*)
            `b -> `x : bl;
            (* @CC_LG 3004C @*)
            `a -> `x : br;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(323,181) @*)
         ::=
         begin
            opt_node a' = `a;
            (* @CC_LG 6008(89,20)(59,21) @*)
            opt_node b' = `b;
            (* @CC_LG 6008(257,20)(59,21) @*)
            obl_node x' = `x;
            (* @CC_LG 6007(89,90)(59,21) @*)
            obl_node y' = `y;
            (* @CC_LG 6007(149,160)(59,21) @*)
            obl_node z' = `z;
            (* @CC_LG 6007(41,160)(59,21) @*)
            a' -> x' : rr;
            (* @CC_LG 3004C @*)
            b' -> x' : rl;
            (* @CC_LG 3004C @*)
            x' -> y' : bl;
            (* @CC_LG 3004C @*)
            x' -> z' : br;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(299,181) @*)
         return nn := n;
      end;

      production InsertRight( n : RBNODE ; Val : integer ; out nn : RBNODE) =
         begin
            obl_node `x = n;
            (* @CC_LG 6007(185,20)(53,21) @*)
            => `x : not with =right=>;
            (* @CC_LG 
            3009XC2
            117,20 @*)
         end
         (* @CC_LG 7008(8,8)(224,41) @*)
         ::=
         begin
            obl_node x' = `x;
            (* @CC_LG 6007(53,20)(59,21) @*)
            y' : RBNode;
            (* @CC_LG 6011(53,90)(83,21) @*)
            x' -> y' : rr;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(107,111) @*)
         condition Val > `x.Value;
         transfer y'.Value := Val;
                  y'.GRAPH_BROWSER_LABEL := string ( Val );
         return nn := y';
      end;

      production InsertLeft( n : RBNODE ; Val : integer ; out nn : RBNODE) =
         begin
            obl_node `x = n;
            (* @CC_LG 6007(71,61)(53,21) @*)
            => `x : not with =left=>;
            (* @CC_LG 
            3009XC2
            60,25 @*)
         end
         (* @CC_LG 7008(8,8)(120,82) @*)
         ::=
         begin
            obl_node x' = `x;
            (* @CC_LG 6007(53,20)(59,21) @*)
            y' : RBNode;
            (* @CC_LG 6011(53,90)(83,21) @*)
            x' -> y' : rl;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(107,111) @*)
         condition Val < `x.Value;
         transfer y'.Value := Val;
                  y'.GRAPH_BROWSER_LABEL := string ( Val );
         return nn := y';
      end;

      production RotateLL( n : RBNODE ; out nn : RBNODE) =
         begin
            obl_node `y : RBNode;
            (* @CC_LG 6007(86,90)(83,21) @*)
            obl_node `a = n;
            (* @CC_LG 6007(38,160)(53,21) @*)
            opt_node `b : RBNode;
            (* @CC_LG 6008(146,160)(83,21) @*)
            obl_node `x : RBNode;
            (* @CC_LG 6007(86,20)(83,21) @*)
            `x -> `y : rl;
            (* @CC_LG 3004C @*)
            `y -> `b : br;
            (* @CC_LG 3004C @*)
            `y -> `a : rl;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(200,181) @*)
         ::=
         begin
            obl_node y' = `y;
            (* @CC_LG 6007(89,20)(59,21) @*)
            obl_node a' = `a;
            (* @CC_LG 6007(41,90)(59,21) @*)
            y' -> a' : rl;
            (* @CC_LG 3004C @*)
            obl_node x' = `x;
            (* @CC_LG 6007(149,90)(59,21) @*)
            opt_node b' = `b;
            (* @CC_LG 6008(149,160)(59,21) @*)
            x' -> b' : bl;
            (* @CC_LG 3004C @*)
            y' -> x' : rr;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(191,181) @*)
         embedding redirect <-br-, <-bl- from `x to y';
         return nn := n;
      end;

      production RotateRR( n : RBNODE ; out nn : RBNODE) =
         begin
            obl_node `y : RBNode;
            (* @CC_LG 6007(101,90)(83,21) @*)
            obl_node `x : RBNode;
            (* @CC_LG 6007(101,20)(83,21) @*)
            `x -> `y : rr;
            (* @CC_LG 3004C @*)
            opt_node `a : RBNode;
            (* @CC_LG 6008(53,160)(83,21) @*)
            obl_node `b = n;
            (* @CC_LG 6007(161,160)(53,21) @*)
            `y -> `a : bl;
            (* @CC_LG 3004C @*)
            `y -> `b : rr;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(200,181) @*)
         ::=
         begin
            obl_node y' = `y;
            (* @CC_LG 6007(89,20)(59,21) @*)
            obl_node x' = `x;
            (* @CC_LG 6007(149,90)(59,21) @*)
            y' -> x' : rl;
            (* @CC_LG 3004C @*)
            obl_node b' = `b;
            (* @CC_LG 6007(41,90)(59,21) @*)
            opt_node a' = `a;
            (* @CC_LG 6008(149,160)(59,21) @*)
            y' -> b' : rr;
            (* @CC_LG 3004C @*)
            x' -> a' : br;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(191,181) @*)
         embedding redirect <-br-, <-bl- from `x to y';
         return nn := n;
      end;

      production RotateLR( n : RBNODE ; out nn : RBNODE) =
         begin
            obl_node `x : RBNode;
            (* @CC_LG 6007(101,20)(83,21) @*)
            obl_node `y : RBNode;
            (* @CC_LG 6007(101,90)(83,21) @*)
            obl_node `z = n;
            (* @CC_LG 6007(101,160)(53,21) @*)
            opt_node `a : RBNode;
            (* @CC_LG 6008(53,230)(83,21) @*)
            opt_node `b : RBNode;
            (* @CC_LG 6008(161,230)(83,21) @*)
            `x -> `y : rl;
            (* @CC_LG 3004C @*)
            `y -> `z : rr;
            (* @CC_LG 3004C @*)
            `z -> `a : bl;
            (* @CC_LG 3004C @*)
            `z -> `b : br;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(215,251) @*)
         ::=
         begin
            obl_node z' = `z;
            (* @CC_LG 6007(89,20)(59,21) @*)
            obl_node x' = `x;
            (* @CC_LG 6007(149,90)(59,21) @*)
            obl_node y' = `y;
            (* @CC_LG 6007(41,90)(59,21) @*)
            opt_node a' = `a;
            (* @CC_LG 6008(41,160)(59,21) @*)
            opt_node b' = `b;
            (* @CC_LG 6008(149,160)(59,21) @*)
            z' -> y' : rl;
            (* @CC_LG 3004C @*)
            x' -> b' : bl;
            (* @CC_LG 3004C @*)
            z' -> x' : rr;
            (* @CC_LG 3004C @*)
            y' -> a' : br;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(191,181) @*)
         embedding redirect <-br-, <-bl- from `x to z';
         return nn := y';
      end;

      production RotateRL( n : RBNODE ; out nn : RBNODE) =
         begin
            obl_node `x : RBNode;
            (* @CC_LG 6007(101,20)(83,21) @*)
            obl_node `y : RBNode;
            (* @CC_LG 6007(101,90)(83,21) @*)
            obl_node `z = n;
            (* @CC_LG 6007(101,160)(53,21) @*)
            opt_node `a : RBNode;
            (* @CC_LG 6008(53,230)(83,21) @*)
            opt_node `b : RBNode;
            (* @CC_LG 6008(161,230)(83,21) @*)
            `x -> `y : rr;
            (* @CC_LG 3004C @*)
            `y -> `z : rl;
            (* @CC_LG 3004C @*)
            `z -> `a : bl;
            (* @CC_LG 3004C @*)
            `z -> `b : br;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(215,251) @*)
         ::=
         begin
            obl_node z' = `z;
            (* @CC_LG 6007(89,20)(59,21) @*)
            obl_node y' = `y;
            (* @CC_LG 6007(149,90)(59,21) @*)
            obl_node x' = `x;
            (* @CC_LG 6007(41,90)(59,21) @*)
            opt_node a' = `a;
            (* @CC_LG 6008(41,160)(59,21) @*)
            opt_node b' = `b;
            (* @CC_LG 6008(149,160)(59,21) @*)
            z' -> x' : rl;
            (* @CC_LG 3004C @*)
            x' -> a' : br;
            (* @CC_LG 3004C @*)
            z' -> y' : rr;
            (* @CC_LG 3004C @*)
            y' -> b' : bl;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(191,181) @*)
         embedding redirect <-br-, <-bl- from `x to z';
         return nn := y';
      end;

   end;
   (* @CC_LG 7013(141,36)(96,30) @*)

end.
(* @CC_LG 7012(8,8)(441,179) @*)
