/***************/
/* ExecSystem.c */
/***************/

/**************************************************/
/* implementation of imported types and functions */
/**************************************************/

/***************************************************************************/
/*                                                                         */
/*  ATTENTION: please SET environment Variable "IMPORTLIB" according       */
/*             to the library containing the object code of the            */
/*             imported types and functions.                               */
/*                                                                         */
/***************************************************************************/

/* Note further that an imported type needs to be provided with at 
   least three pre-defined functions that are needed for PROGRES built-in 
   functionality.
   * (let USERTYPE be the type to be imported)
   *  - USERTYPE_Size returns the size of the type in the parameter
   *  - USERTYPE_ValueToString writes the contents of the imported type to a string, 
   *    that will be returned in the first parameter
   *  - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
   *    parameter
   * Remark: The formerly needed procedures WriteConst and ReadConst are obsolete!

   * Further user-defined functions for an imported type have the following rules: 
   * Assume that you want to import a function in the specification:
   *   MyImportedFunction : (T1, ..., Tn) -> Tr
   * where types T1, ..., Tn, Tr are imported or built-in types.
   * The procedure implementing the function above must then look like:
   *   PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
   * p1, ..., pn are the in-paramters and pr is the result of the function.
   *
   * Note: All parameters must be passed by address (for technical reasons)
 */


#include <stdio.h>
#include <stdlib.h>

/* use type definition of text in the header file */
#include "CVSUtils.h"


/* header of local functions */


struct RevisionInfo {
	char*	filename;
	char*	revision;
};

struct RevisionInfo* revisionTable = NULL; 

char*	basedir = NULL;

/*****************************************************/
/* implementation of user-defined imported functions */
/*****************************************************/

int searchTable(char* filename)

{
	static int	current_size = 0;
	static int	alloc_size = 200;
	int i;

	if (revisionTable == NULL)
		revisionTable = malloc(sizeof(struct RevisionInfo) * alloc_size);

	for (i=0; i<current_size; i++) {
		if (strcmp(filename, revisionTable[i].filename) == 0)
			break;
	}

	if (i == current_size) {			/* element does not yet exist? */
		current_size++;
		if (current_size == alloc_size) {
			alloc_size *= 2;
			revisionTable = realloc(revisionTable, sizeof(struct RevisionInfo) * alloc_size);
		}
	
		revisionTable[i].filename = malloc(sizeof(char) * (strlen(filename)+1));
		strcpy(revisionTable[i].filename, filename);
		revisionTable[i].revision = NULL;	
	}
	return i;
}

void setRevisionNr(char* current, char* new)

{
	if (current != NULL) free(current);

	current = malloc(sizeof(char) * (strlen(new)+1));
	strcpy(current, new);
}

/* ------------------------------------------------------------------- *
 *  CheckOut an revision, if not already done                          *
 * ------------------------------------------------------------------- */

void CheckOut (BOOLEAN* retval, PGCMString* pfilename, PGCMString* prevision)

{
	char*	filename;
	char*	revision;
	char	command[500];
	int		pos;
	/* basedir will be stored in ADL_SYSTEM (new intrinsic attribute) */

	filename = PGCMStrToStr(*pfilename);
	revision = PGCMStrToStr(*prevision);
	
	pos = searchTable(filename);

	if (strcmp(revision, revisionTable[pos].revision) == 0)
		return;			/* no need for checkout */

	setRevisionNr(revisionTable[pos].revision, revision);
	printf("checking out: %s\n", filename);
	sprintf(command, "cd %s; cvs checkout -p -r %s %s > %s\n", basedir,
		revision, filename, filename);
	system(command);
}

/* ------------------------------------------------------------------- *
 *  retrieve the revision number from file (using cvs status)          *
 * ------------------------------------------------------------------- */

getRevision(PGCMString* prevision, PGCMString* pfilename)

{
	FILE* fin;

	char command[400], dummy;
	char* filename;
	char buffer[400];
	int pos;
	filename = PGCMStrToStr(*pfilename);

	pos = searchTable(filename);
			
	sprintf(command,
		"cd %s; cvs status %s > .cvs.info", basedir, filename);
	system(command);

	sprintf(buffer, "%s/.cvs.info", basedir);
	fin=fopen(buffer, "r");

	if (fin == NULL) {
		printf("could not open file %s\n", command);
		return;
	}

	buffer[0] = 0;
	while (!feof(fin)) {
		if (fscanf(fin, " Working%c", &dummy) != 0) {
			fscanf(fin, "revision: %s", buffer);
			break;
		}
		fscanf(fin, "%*s");
    }
	fclose(fin);

	system("rm -f .cvs.info");

	if (strlen(buffer) == 0) {
		printf("could not get revision number for %s\n", filename);
		strcpy(PGCMStrToStr(*prevision), "");
	}
	else {
		if (strcmp(buffer, "No") == 0)
			strcpy(PGCMStrToStr(*prevision), "n/a");
		else
			strcpy(PGCMStrToStr(*prevision), buffer);
	}
}

/* ------------------------------------------------------------------- *
 *  commit a revision                                                  *
 * ------------------------------------------------------------------- */

commitRevision(PGCMString* prevision, PGCMString* pfilename)

{
	char* filename;
	char command[500];
	int pos;
	filename = PGCMStrToStr(*pfilename);
	
	pos = searchTable(filename);
			
	sprintf(command,
		"cd %s; cvs commit -m \"progres commit\" %s", basedir, filename);

	if (system(command) != 0) {
		printf("commit failed!\n", filename);
		strcpy(PGCMStrToStr(*prevision), "");
		return;
	}

	// get revision	nr. after checkout
	getRevision(prevision, pfilename);	
}

/* ------------------------------------------------------------------- *
 *  set the root directory                                             *
 * ------------------------------------------------------------------- */

setBaseDir(BOOLEAN* retval, PGCMString* pbasedir)

{
	if (basedir != NULL) free(basedir);

	basedir = malloc(sizeof(char) * (strlen(PGCMStrToStr(*pbasedir))+1));

	strcpy(basedir, PGCMStrToStr(*pbasedir));
	*retval = TRUE;
}
