#ifndef C_Local_h_
#define C_Local_h_

# ifdef __cplusplus
extern "C" {
# endif

/**************************************************************************/
/*                                                                        */
/*  PROGRES environment: an integrated set of tools for PROgramming       */
/*                       with Graph REwriting Systems.                    */
/*                                                                        */
/*  Copyright (C) 1987-1992  Lehrstuhl Informatik III, RWTH Aachen        */
/*                                                                        */
/*  This library is free software; you can redistribute it and/or         */
/*  modify it under the terms of the GNU Library General Public           */
/*  License as published by the Free Software Foundation; either          */
/*  version 2 of the License, or (at your option) any later version.      */
/*                                                                        */
/*  This library is distributed in the hope that it will be useful,       */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     */
/*  Library General Public License for more details.                      */
/*                                                                        */
/*  You should have received a copy of the GNU Library General Public     */
/*  License along with this library; if not, write to the Free            */
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.    */
/*                                                                        */
/*  Contact Adresses:                                                     */
/*                                                                        */
/*      Dr. Andy Sch"urr (or Cornelia Weigmann),                          */
/*      Lehrstuhl f"ur Informatik III,                                    */
/*      University of Technology Aachen (RWTH Aachen),                    */
/*      Ahornstr. 55,                                                     */
/*      D-5100 Aachen                                                     */
/*                                                                        */
/*  Email to                                                              */
/*                                                                        */
/*      andy@rwthi3.informatik.rwth-aachen.de                             */
/*      cornelia@rwthi3.informatik.rwth-aachen.de (for technical support) */
/*                                                                        */
/**************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*                         Locale Typdeklarationen			     */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/*
   $Author: muench $
   $Revision: 1.16.2.1 $
   $Date: 1999/10/21 15:53:19 $
   $State: Exp $
   $Log: C_Local.h,v $
   Revision 1.16.2.1  1999/10/21 15:53:19  muench
   Bereits gemachte Bugfixes, die bei der Umstellung auf eine aeltere Version
   unter ~progres verloren gingen, nachgezogen. [mm]

   Revision 1.17  1998/11/12 19:33:33  chrisp
   In diesem und allen folgenden Aenderungen, die ich heute abend einchecke,
   ist habe ich Progres die Funktionen ord und char beigebracht, die vorher
   abgefangen wurden und einen "Not yet implemented" erzeugten.[cp]

   Revision 1.16  1998/07/01 08:15:06  marita
   Die Beschreibung der Mengenklassen wird nun in PGC durchgefuehrt. Dadurch
   sind die SETof...-Dateien ueberfluessig.
   Zusaetzlich wird noch der Typ 'UniqueIDAttr' und die zugehoerige
   Menge eingefuehrt. (mb)

   Revision 1.15  1998/05/20 16:03:21  prom3
   PGCMString jetzt wie er sein soll.

   Revision 1.14  1998/05/19 15:01:45  roetschk
   Typecasts in Makros eingebaut[tr].

   Revision 1.13  1998/05/19 14:45:02  roetschk
   Neue Funktion CopyStrToPGCMString eingefuehrt.

   Revision 1.12  1998/05/18 15:34:42  prom3
   Neues Makro PGCStrToString eingefuehrt. Dadurch ist die direkte
   Konvertierung in einen C++-String moeglich[tr].

   Revision 1.11  1998/05/18 13:00:08  roetschk
   Wie gesagt, length heit das Ding, nicht size.

   Revision 1.10  1998/05/18 12:55:45  roetschk
   PGCMStringLength war runtergefallen, sorry [tr].

   Revision 1.9  1998/05/18 10:44:43  roetschk
   - Einheitliche Verwendung von PGCMString.
   - Neues Makro PGCMStrToStr.
   [tr]

   Revision 1.8  1998/05/05 14:32:22  roetschk
   MaxTypeSize auf 4 KB erhht. Lnge von PGCString in der C-Schnittstelle
   korrigiert[tr].

   Revision 1.7  1997/11/11 10:01:39  roetschk
   berflssige Konstantendeklarationen entfernt [tr].

   Revision 1.6  1997/11/11 09:50:11  roetschk
   Alle Parameter die C-seitig als char[] mit fester Lnge deklariert sind, d. h.
   NameStringType, PoolName, GraphName, ApplicationName, AGString, PGCMString,
   mssen Modula-3-seitig als READONLY oder VAR deklariert werden. Ansonsten
   funktioniert die C-Schnittstelle unter Linux nicht, weil C Pointer bergibt,
   wo Modula-3 Werte, d. h. Arrays erwartet [tr].

   Revision 1.5  1997/10/23 11:33:17  prom3
   Geaendert aber nicht eingecheckt. Hoffe es ist richtig! -ajw

   Revision 1.4  1997/09/29 12:33:53  roetschk
   Aufzaehlungstyp ChoiceStatusType ergaenzt[tr].

   Revision 1.3  1997/09/18 12:01:56  prom3
   Neue Resource GetInContextOfNode [tr].

   Revision 1.2  1997/08/28 15:14:31  roetschk
   Integration der C-Modula-3-Schnittstelle, Initialisierung der M3-Laufzeit-
   bibliothek, Ausgabe von Fehlermeldungen bei Exceptions, Portierung von
   TAssignElemSet aus Pro92 [tr].

   Revision 1.1  1997/07/07 16:57:33  roetschk
   Weitgehend automatische Generierung von *.c und *.h-Files aus Modula-3
   Interfaces. Prototypgenerierung noch nicht getestet. [tr]

*/
/*****************************************************************************/


#include <ansi.h>
#include <string.h>
#include <stdio.h>
#include <rgglobal.h>
#include <CTypesForM3.h>
#include "C_Internal.h"

/****************************************************************************/
/*                                                                          */ 
/*  SECTION PGCGlobal                                                       */ 
/*                                                                          */ 
/****************************************************************************/
#define DontKnowNumberOfOperands -1
    /* Im PGCode gibt es einige Befehle mit variabler Anzahl von Operanden */
    /* (z.B.: Call).  Fuer diese Befehle liefert die Methode NoOfOperands */
    /* DontKnowNumberOfOperands zurueck. */
 
#define MaxTypeSize 4096

typedef BYTE LevelDiffType; /* = [0..255]; */ 
 
typedef AttributeNumber CellType;
    /* Die Nummer einer Speicherzelle in einem Prozedurframe. */
typedef SHORTCARD OffsetType;
    /* Offset beim Zugriff auf Komponenten einer Variable in der PGCMaschine.*/
 
    /*
    Die Elemente dieses Aufzaehlungstyps sind Standardoperanden, die von dem
    Befehl 'ArithEval' verarbeitet werden koennen.
    */

typedef enum ArithmeticOperators {
      Plus,        GPlus,        GPlusWithRightAll,     GPlusWithLeftAll,  
      Minus,        GMinus,       GMinusWithRightAll,    GMinusWithLeftAll,
      Times,        GTimes,       GTimesWithRightAll,    GTimesWithLeftAll,
      Concat,       GConcat,      GConcatWithRightAll,   GConcatWithLeftAll,
      Div,          GDiv,         GDivWithRightAll,      GDivWithLeftAll,
      Mod,          GMod,         GModWithRightAll,      GModWithLeftAll,
      Negate,
      And,
      Or,
      Not,
      Equal,        StrEqual,
      NotEqual,
      Greater,
      Less,
      GrEqual,
      LeEqual,
      EmptySet,           GEmptySet,
                          GInitSetOfNSurrogates,
      ElemToSet,          GElemToSet,
      SetToElem,
      Card,               GCard,
      Def,                GDef,
      AddElemToSet,
      DeleteElemFromSet,
      GetAnyElemOfSet,
      GElemIsInSet,        ElemIsInSet,
      Union,              GUnion,
      Intersection,       GIntersection,
      Difference,         GDifference,
      Size,
      IsEmpty,            GIsEmpty,
      IsNotEmpty,         GIsNotEmpty,
      SetEqual,           GSetEqual,
      SetInclusion,       GSetInclusion, 
      None,
      Self,
      TestElemOut,
      SetOut,
      ElemOut,
      TestElemIn,
      SetIn,
      ElemIn,
      TypeOf, ElemTypeOf, 
      GetTypesOfClass,
      NodeIsInstanceOf,
      InstanceOfClause,
      TypeInstanceOfClause,
      IsOfClass,
      TypeIsOfClass,
      IsOfTypeSet,
      NodeIsOfClass,
      AllInSetIsOfClass,
      NodeIsOfType,
      AllInSetIsOfType,
      NegateBtInfo,
      SetBtInfo,
      SetBtInfoFailed,
      SetBtInfoInitial,
      SetBtInfoContinueOld,
      Substr, GSubstr,
      StatIsSuccess,
      StatIsContinueOld,
      TypeCheck,
      TypeCheckStandard,
      Length, GLength,
      Char, GChar,
      Ord, GOrd
} ArithmeticOperators;
  
typedef enum ControlFlowEdgeType {
      CFEToSelf,
      CFEToAllDeclarationUnits,
      CFEToAttCond,
      CFEToBody,
      CFEToNext,
      CFEToCalled,
      CFEToPlusRelCallId,
      CFEToMinusRelCallId,
      CFEToMain,
      CFEToBlock,
      CFEToElseBlock,
      CFEToFirst,
      CFEToNewNodeDecls,
      CFEToNewEdgeDecls,
      CFEToListElems,
      CFEFirstSchemeDecl,
      CFENextSchemeDecl,
      CFEToNil
} ControlFlowEdgeType;
    /* Aufzaehlungstyp mit den moeglichen Kontrollfluss-Kanten. */

typedef char PGCString[MaxTypeSize]; /*  = ARRAY [0..4095] OF CHAR; */
    /* String used for internal error messages */
 
#define PGCMStringLength MaxRecordLength
#define StrToPGCMStr(x) (PGCMString) { x }
#define PGCMStrToStr(x) (char *) (x).s
#define PGCMStrToString(x) String((char *) &x.s[0])

typedef struct { 
    AGString  s;
} PGCMString;
    /* Built in string type of the PROGRES-GraphCodeMachine. */

PGCMString CopyStrToPGCMStr(char *aString);
 
typedef NodeNumber Framenumber;
    /* Typ eines Aktivierungsblocks in der Memory */
 
typedef struct AbsAddressType {
    Framenumber FrameNo;
    CellType CellNo;
    OffsetType Offset;
} AbsAddressType;
    /* Typ einer absoluten Adresse einer Variablen in der Memory */
 
    /* Typen, die als Grundlage fuer einige StandardTypes benoetigt werden: */
 
typedef struct SetDescription {
    SimpleSet SetNumber;
    GraphNumber SetGraph;
} SetDescription;
 
typedef struct RelDescription {
    RelSet RelNumber;
    GraphNumber RelGraph;
} RelDescription;
 
typedef Surrogate SchemeComponentNumber;
 
typedef enum ChoiceStatusType {
    CSTInitial,
    CSTChoose,
    CSTContinueOld,
    CSTSuccess,
    CSTFail,
    CSTExecInnerCode
} ChoiceStatusType;
 
typedef struct ChoiceRec {
    SHORTCARD No;
    ChoiceStatusType Stat;
    Surrogate Node;
    SimpleSet Set;
    SHORTCARD SubNo;
    BOOLEAN Innermost;
} ChoiceRec;
 
typedef struct HostGraphNosType {
    GraphNumber HostGraphNo;
    GraphNumber ChoiceGraphNo;
} HostGraphNosType;
 
#define GrOneGroup FALSE
#define GrNoWait FALSE
 
#define GrNoSaveGroup "ReadOrg"
  
#define WorkGrGroup     SaveGroup
#define WorkGrGroupMode GetGroup
#define WorkGrGroupType Session /* Operation */
 
#define HostGrGroup     SaveGroup
#define HostGrGroupMode GetGroup
#define HostGrGroupType Operation /* Operation */
#define ChoiceGrGroup     SaveGroup
#define ChoiceGrGroupMode GetGroup
#define ChoiceGrGroupType Operation /* Operation */
 
     /* Consts fuer Edge-Events: alex */
     /* EventEdge_xxx sind Konstanten, die IN GraphEvent-Variablen
        eingetragen werden koennen.
        EventNode_xxx sind Konstanten, die bei Aufrufen von 
        AGRaiseEvent als Parameter fuer Node verwendet werden */
     /* xxx_GraphModification signalisieren Edge, dass der Browser
        die Anzeige an den neuen Graphzustand anpassen soll */
#define EventEdge_GraphModification 55
#define EventNode_GraphModification 33
     
     /* Sag Edge: Gib dir die Kugel, Kleines !!! */
#define EventEdge_CloseEdge 56
#define EventNode_CloseEdge 33
 
     /* Event fuer den Unparser; 
     Kind: OutgoingEdgeDeleted, NewOutgoingEdge */
#define EventEdge_EApplToDecl 116
     /* EventEdge_DeleteXXX signalisiert dem Graphbrowser von Prototypen,
        dass ein Knoten oder eine Kante geloescht wurde,
        EventEdge_AddXXX signalisiert, dass ein Knoten oder eine Kante
        eingefuegt wurde;
        wird IN PGCExecutionBasics bei Loeschen und Erzeugen
        von Knoten und Kanten ausgeloest */     
     
#define EventEdge_DeleteNode 60
#define EventEdge_DeleteEdge 61
     
#define EventEdge_AddNode 62
#define EventEdge_AddEdge 63
     
 
     /* EventEdge_Redisplay signalisiert dem Graphbrowser von Prototypen,
        dass der aktuell angezeigte Graph ungueltig ist und neu von GRAS 
        eingelesen werden muss. Wird bei Backtracking ausgeloest */
     /* Flag, ob Events fuer den Graphbrowser IN Prototypen ausgeloest 
        werden sollen, s.o.. */     
#define RaiseChangedEvent TRUE

/****************************************************************************/
/*                                                                          */ 
/*  SECTION ChoiceGraphs                                                    */ 
/*                                                                          */ 
/****************************************************************************/

/****************************************************************************/
/*                                                                          */ 
/*  SECTION GenericSets                                                     */ 
/*                                                                          */ 
/****************************************************************************/

typedef BYTE SetCompareResultType;

/*
#define SetLess    0
#define SetEqual   1
#define SetGreater 2
*/

    /* Einen Pointer wie er im Buche steht ??? */
    /* Ergebniswerte, die eine Mengenelementvergleichsprozedur liefern muss */
    /* Prozedurtyp zum Vergleich zweier Mengenelemente. Kompatibilitaet mit
    allen Modula-2- Datentypen durch ADDRESS-Benutzung. ABSOLUT NOTWENDIG ist
    dafuer die Verwendung von Call-By-Reference bei beiden Parametern, also
    stets ein VAR benutzen!!!  Fuer Anwendungsprozeduren wird eine TypeCast mit
    SetCompareProcType vorgeschlagen. */
typedef SetCompareResultType (*SetCompareProcType)(ADDRESS,
                                                   ADDRESS);


typedef BYTE *APtr[MaxTypeSize]; /* = POINTER TO ARRAY TypeIndexType OF [0..255]; */


/****************************************************************************/
/* Der Versuch den opaken generischen Mengentyp aus Modula-2 zu simulieren  */
/* ist leider gescheitert.                                                  */
/* Daher jetzt halt als offener Typ.                                        */

  /* Record fuer die Daten, die zu einem Mengenelement gespeichert werden */
typedef struct Element_Type {
    ADDRESS DataPtr;       /* - Zeiger auf die Datenbytes          */
    BOOLEAN Deleted;       /* - TRUE, falls Element geloescht      */
    struct Element_Type *Left;   /* - Zeiger auf linken Teilsuchbaum     */
    struct Element_Type *Right;  /* - Zeiger auf rechten Teilsuchbaum    */
    struct Element_Type *Thread; /* - Nachfolger in der linearen Liste   */
} Element_Type;
				   
typedef Element_Type *Element_TypePtr;    /* Zeiger auf ein Mengenelement */

typedef struct Setinfo {
    Element_TypePtr     TreeHead;
    Element_TypePtr     ListThread;
    Element_TypePtr     Current;
    CARDINAL           DeletedN;
    CARDINAL           N;
    SetCompareProcType Cmp;
    CARDINAL           Bytes;
} SetInfo;

typedef SetInfo *Set_Type;

/****************************************************************************/
/*                                                                          */ 
/*  SECTION PGC                                                             */ 
/*                                                                          */ 
/****************************************************************************/

typedef struct EdgeDescription {
    UniqueID   Source;
    UniqueID   Target;
    TypeNumber Type;
} EdgeDescription;

typedef struct UniqueIDAttr {
    UniqueID  ID;
    Surrogate Attr;
} UniqueIDAttr;

/****************************************************************************/
/*                                                                          */ 
/*  SECTION StandardTypes                                                   */ 
/*                                                                          */ 
/****************************************************************************/
typedef enum StandardType {
      _dummy,
      _boolean,
      _shortcard,
      _integer,
      _real,
      _char,
      _string,
      AGGlobal_Graphnumber,
      AGGlobal_Nodenumber,
      AGGlobal_Surrogate,
      AGGlobal_Attributetype,
      AGGlobal_ExternNumber,
      AGGlobal_TStatus,
      AGGlobal_GraphType,
      AGGlobal_AttNumberType,
      AGGlobal_Edgelabel,
      AGGlobal_Cardinality, 
      AGGlobal_IndexProperties, 
      PGCGlobal_SetDescription,
      PGCGlobal_ArithmeticOperators,
      PGCGlobal_DeclSchemeComponentNumber,
      PGCGlobal_SchemeComponentNumber,
      PGCGlobal_AbsAddressType,
      PGCOperandTypes_OperandType,
      PGCVirtualCodeAddress_VirtualCodeAddressType,
      PGCGlobal_ControlFlowEdgeType,
      BacktrackTypes_BacktrackInfo
} StandardType;
 
typedef enum STChoiceStatusType {
       STCSTInitial,
       STCSTContinueOld,
       STCSTChooseNext,
       STCSTChooseLast,
       STCSTExecInitial,
       STCSTExecBody,
       STCSTsuccess,
       STCSTTestSuccess,
       STCSTFailed
} STChoiceStatusType;
 
typedef struct STBacktrackInfo {
    Surrogate No;
    STChoiceStatusType Stat;
    GraphNumber Graph;
} STBacktrackInfo;

typedef struct STChoiceRec {
    Surrogate Node;
    SimpleSet Set;
    STBacktrackInfo SubBtI;
} STChoiceRec;
 
typedef BOOLEAN *BOOLEANPtr;
typedef INTEGER *INTEGERPtr;
typedef REAL *REALPtr;
typedef SHORTCARD *SHORTCARDPtr;
typedef PGCMString *STRINGPtr;
typedef GraphNumber *GraphNumberPtr;
typedef NodeNumber *NodeNumberPtr;
typedef Surrogate *SurrogatePtr;
typedef AttributeKind *AttributeKindPtr;
typedef ExternNumber *ExternNumberPtr;
typedef TStatus *TStatusPtr;
typedef GraphType *GraphTypePtr;
typedef AttributeNumber *AttributeNumberPtr;
typedef TypeNumber *TypeNumberPtr;
typedef Cardinality *CardinalityPtr;
typedef IndexProperties *IndexPropertiesPtr;
typedef SetDescription *SetDescriptionPtr;

typedef ArithmeticOperators *ArithmeticOperatorsPtr;

typedef SchemeComponentNumber *SchemeComponentNumberPtr;
typedef AbsAddressType *AbsAddressTypePtr;
typedef ControlFlowEdgeType *ControlFlowEdgeTypePtr;
typedef STBacktrackInfo *BacktrackInfoPtr;

/****************************************************************************/
/*                                                                          */ 
/*  SECTION PGCBacktrackTypes                                               */ 
/*                                                                          */ 
/****************************************************************************/

/* Aufzaehlungstyp fuer Choice - Zustaende */
 
typedef STChoiceStatusType BTChoiceStatusType;
 
typedef STBacktrackInfo BacktrackInfo;
 
typedef STChoiceRec BTChoiceRec;
 
void  InitBacktrackInfo(BacktrackInfo    *BtI,
                        ChoiceStatusType Stat,
                        GraphNumber      Graph);
 
ChoiceStatusType  GetBacktrackInfoStatus(BacktrackInfo BtI);
 
void  SetBacktrackInfoStatus(BacktrackInfo    *BtI,
                             ChoiceStatusType Stat);

/****************************************************************************/
/*                                                                          */ 
/*  SECTION PGCExecutionBasics                                              */ 
/*                                                                          */ 
/****************************************************************************/
typedef PGCMString* PtrToPGCMString;

/****************************************************************************/
/*                                                                          */ 
/*  SECTION PGCSets                                                         */ 
/*                                                                          */ 
/****************************************************************************/
  
typedef struct { BOOLEAN IsAGSet ;
                 union { SetDescription AGSet;
                         Set_Type        GenSet;
                       }
                       var;
               } 
               PGCSetType;
 
/****************************************************************************/
/*                                                                          */ 
/*  SECTION PermanentSets                                                   */ 
/*                                                                          */ 
/****************************************************************************/
  
/****************************************************************************/
/*                                                                          */ 
/*  SECTION SCGlobal                                                                */ 
/*                                                                          */ 
/****************************************************************************/
#define SchemeVar 1      /* Schemanummer im Host-Graphen */
#define GraphVar 2       /* interne Graphnummer des Host-Graphen */
#define ExtNoVar 3       /* externe Graphnummer der Host-Graphen */
#define StatusVar 4      /* Graphstatus */
#define PoolNameVar 5    /* Name des Graphpools fuer den Host-Graphen */
#define GraphNameVar 6   /* Name des Host-Graphen */
 #define TypeVar 7        /* Typ des Host-Graphen */
    
#define NodeParVar 8     /* aktueller Knoten in Berechnungsfunktionen */
#define ReturnParVar 9   /* Rueckgabewert in (Berechnungs)funktionen */
#define AttParVar 10     /* aktuelles Attribut in Berechnungsfunktionen */
 
#define ChoiceInfoVar 11  /* Info fuer den Choicegraphen */
 
#define NodeSetVar 12    /* SetDescription zum Abspeichern schon bearbeiteter 
                          Pfade bei Berechnung der Attributabhaengigkeiten */
    
#define CodeUnitClass 200        /* Knotentypen, die Code tragen */
#define DummyUnitClass 201       /* Knotentypen, die das Dummy-Attribut tragen */
#define ExpressionUnitClass 202  /* Knotentypen, die in Ausdruecken vorkommen */
#define MetaClass 203            /* Knotentypen des Mets-Schemas */
 
#define NumberOfAny 1    /* Nummer der Klasse ANY */
    
#define CompilerName "Prog:CompileUnit" 

/****************************************************************************/
/*                                                                          */ 
/*  SECTION StringOperations                                                */ 
/*                                                                          */ 
/****************************************************************************/

/****************************************************************************/
/*                                                                          */ 
/*  SECTION Hacker`s Corner                                                 */
/*                                                                          */ 
/****************************************************************************/
void DummyModuleInit();
   
	       
# ifdef __cplusplus
}
# endif

#endif /* C_Local_h_ */

