/*********************************************************************/
/*                                                                   */
/* Copyright 1994/95/96 Carsten Friedrich, University of Passau.     */
/*                                                                   */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/*********************************************************************/
// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)
    based on code by Marc Shapiro (shapiro@sor.inria.fr)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _MPlex_h
#ifdef __GNUG__
#pragma interface
#endif
#define _MPlex_h 1


#include "MFPlex.h"


// Number of bits per long, used in MChunk bit map operations

#define _MAP_BITS  32

template<class Type>
class MChunk : public MFIChunk<Type>
{
protected:

  unsigned long* map;          // bitmap of slots
  int            unused;       // number of unused internal slots 

  void           mark(int);    // bitmap operations
  void           free(int);
  int            valid(int) const;

public:

                 MChunk(Type*    d,        // ptr to array of elements
                        int      base_idx, // initial indices
                        int      low_idx,  // & initially clear map
                        int      fence_idx,
                        int      top_idx);

                 ~MChunk();

// virtuals

  int            first_index() const;
  int            last_index() const;
  int            succ(int idx) const;
  int            pred(int idx) const;
  Type*           first_pointer() const;                 
  Type*           last_pointer() const;                 
  Type*           succ(Type*) const;                 
  Type*           pred(Type*) const; 
  int            empty() const;
  int            full() const;
  int            valid_index(int i) const;
  int            valid_pointer(const Type* p) const;
  Type*           grow_high ();
  Type*           grow_low ();  
  void           shrink_high ();
  void           shrink_low ();     
  void           clear(int);
  void           cleardown(int);
  int            OK() const;

// extensions

  int            unused_indices() const;   // how many free slot in low..fence?

  int            unused_index() const;     // return index of free slot

  int            del(int i);         // delete data indexed by i
                                     // return true if was present
  int            undel(int idx);     // un-delete data indexed by i
                                     // return true if already present

  void           reset_low();        // reset low = lowest valid index; 
  void           reset_high();       // same for high

};

template<class Type>
class MPlex: public MFPlex<Type>
{
  MChunk<Type>*       ch;          // cached chunk
  int              unused;      // # of free slots between low & fence

  void             make_initial_chunks(int up = 1);
  void             cache(int idx) const;
  void             cache(const Type* p) const;
  int              dopred(int) const;
  int              dosucc(int) const;

  void             set_cache(const MChunk<Type>* t) const; // logically, 
                                               // not physically const

public:
                   MPlex();                 // set low = 0;
                                               // fence = 0;
                                               // csize = default

                   MPlex(int ch_size);      // low = 0; 
                                               // fence = 0;
                                               // csize = ch_size

                   MPlex(int lo,            // low = lo; 
                            int ch_size);      // fence=lo
                                               // csize = ch_size

                   MPlex(int lo,            // low = lo
                            int hi,            // fence = hi+1
                            const Type  initval,// fill with initval,
                            int ch_size = 0);  // csize= ch_size
                                               // or fence-lo if 0

                   MPlex(const MPlex&);

  void             operator= (const MPlex&);

// virtuals 

  Type&             high_element ();
  Type&             low_element ();
  const Type&       high_element () const;
  const Type&       low_element () const;

  Pix              first() const;
  Pix              last() const ;
  void             prev(Pix& ptr) const;
  void             next(Pix& ptr) const;
  int              owns(Pix p) const;
  Type&             operator () (Pix p);
  const Type&       operator () (Pix p) const;

  int              low() const; 
  int              high() const;
  int              valid(int idx) const;
  void             prev(int& idx) const;
  void             next(int& x) const;
  Type&             operator [] (int index);
  const Type&       operator [] (int index) const;
    
  int              Pix_to_index(Pix p) const;
  Pix              index_to_Pix(int idx) const;

  int              can_add_high() const;
  int              can_add_low() const;
  int              full() const;

  int              add_high(const Type  elem);
  int              del_high ();
  int              add_low (const Type  elem);
  int              del_low ();
  void             clear();

  int              OK () const; 

// extensions

  int              count() const;             // # valid elements
  int              available() const;         // # deleted elements

  int              unused_index()const;       // return index of a deleted elem
  Pix              unused_Pix() const;        // return Pix of a deleted elem

  int              del_index(int idx);        // logically delete at idx;
                                              // return true if was present
  int              del_Pix(Pix p);            // delete at p

  void             undel_index(int idx);      // undelete at idx;
  void             undel_Pix(Pix p);          // undelete at p;

  void             adjust_bounds();           // reset lo, hi to lowest &
                                              // highest valid indices

  int              add(const Type  elem);      // add anywhere
};

#endif
