/*********************************************************************/
/*                                                                   */
/* Copyright 1994/95/96 Carsten Friedrich, University of Passau.     */
/*                                                                   */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/*********************************************************************/
/*
// Copyright 1994 Carsten Friedrich, Universitaet Passau
//
// $Author: friedric $
// $Date: 1995/12/18 12:22:07 $
// $RCSfile: FFEdgeClass.h,v $
// $Revision: 1.2 $
// $State: Exp $
//
*/

#ifndef FF_EDGE_CLASS
#define FF_EDGE_CLASS

#include "globdefs.h"
#include <container/MPlex.h>
#include "FFEdgeLabel.h"
#include "FFGraph.h"

class FFNodeClass;
class FFEdgeClass;
class FFGraphClass;

class FFGraphicEdge;
class FFGraphWindow;

class FFEdgeClass
{
  FFNodeClass *sourceNode, *destNode;
  FFGraphClass *derGraph;
  
  MPlex<FFEdgeLabel *> *label;
  int value;
  FFCoord weight;
  
	String name;
	
  void copyDataFrom(FFEdgeClass *);
  
  FFEdgeClass(MPlex<FFEdgeLabel *>*, FFNodeClass *, FFNodeClass *);
  ~FFEdgeClass();  

	friend FFGraphClass;
	void addLabel(const FFEdgeLabel *); 
	void delLabel(int, BOOL);

	void _sendEvent(labelEvent, labelEventStruct *);
	void init(FFGraphClass *);
	
 public:

  FFEdgeClass *copy();
  void setValue(int );
  int getValue();
  void setWeight (FFCoord x){weight =x;}
  FFCoord getWeight(){return weight;}
  void setName(const String& s);
  const String& getName(){return name;}

	FFGraph getGraph(){return FFGraph(derGraph);}
  FFNodeClass *source();
  FFNodeClass *target();
  void revert();
	void setTarget(FFNodeClass *);
	void setSource(FFNodeClass *);

  void assignLabel(FFEdgeLabel *l, int i=-1);
  void setLabel(FFEdgeLabel *l, int i=-1, BOOL doFree=YES);
  void assignLabel(MPlex<FFEdgeLabel *>&);
  void setLabel(MPlex<FFEdgeLabel *>&, BOOL doFree=YES);
  int getLabelIndex(const String&, FFSet<int>*set=NULL) const;
  FFEdgeLabel *getLabel(int i)
					{if(label && label->valid(i)) return (*label)[i];
					 return NULL;}  
  MPlex<FFEdgeLabel *> *getLabels() {return label;}  

  void show();
  void hideGraphic() ;
  void showGraphic() ;
  
  inline FFGraphicEdge *getGraphicEdge()
		{if(derGraph->getGraphicEdgeIndex()>=0)
			return (FFGraphicEdge *)((*label)[derGraph->getGraphicEdgeIndex()]);
		 else
			return NULL;}
	inline FFGraphicEdge *operator()() 
		{if(derGraph->getGraphicEdgeIndex()>=0)
			return (FFGraphicEdge *)((*label)[derGraph->getGraphicEdgeIndex()]);
		 else
			return NULL;}
	inline FFGraphicEdge *operator->() 
		{if(derGraph->getGraphicEdgeIndex()>=0)
			return (FFGraphicEdge *)((*label)[derGraph->getGraphicEdgeIndex()]);
		 else
			return NULL;}
};

typedef FFEdgeClass * FFEdge;

#endif

