/*********************************************************************/
/*                                                                   */
/* Copyright 1994/95/96 Carsten Friedrich, University of Passau.     */
/*                                                                   */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/*********************************************************************/
/*
// Copyright 1994 Carsten Friedrich, Universitaet Passau
//
// $Author: friedric $
// $Date: 1996/05/06 14:24:45 $
// $RCSfile: FFNodeClass.h,v $
// $Revision: 1.3 $
// $State: Exp $
//
*/

#ifndef FF_NODE_CLASS
#define FF_NODE_CLASS

#include "globdefs.h"

#include <container/AVLSet.h>
#include <container/VOHSet.h>
#include <container/VHSet.h>
#include <container/MPlex.h>
#include "String.h"
#include "FFGraphClass.h"
#include "FFPoint.h"

class FFGraph;
class FFGrahClass;
class FFEdgeClass;

class FFGraphicNode;
class FFGraphWindow;
class FFNodeLabel;

class FFNodeClass
{
protected:
  FFSet<FFEdgeClass *> transFrom;      // Gehen Rein
  FFSet<FFEdgeClass *> transTo;        // Gehen Raus
  
  int value;
  MPlex<FFNodeLabel *> *label;
  int attribut;

  class FFGraphClass *derGraph;
  String name;

  void copyDataFrom(FFNodeClass *n);  
  
  friend FFGraphClass;
  void addLabel(const FFNodeLabel *);
  void delLabel(int, BOOL);

  void _sendEvent(labelEvent, labelEventStruct *);

public:
  
  inline int getValue() const {return value;}
  inline void setValue(int i) {value=i;}
  inline int getAttribut() const {return attribut;} 
  inline void setAttribut(int i) {attribut = i;}

  FFNodeClass *copy();
  
  FFNodeClass(MPlex<FFNodeLabel *> *);
  ~FFNodeClass();
  void init(FFGraphClass *);
  
  inline int indeg() const {return transFrom.length();}
  inline int outdeg() const {return transTo.length();}
  void eliminateParallelEdges();

  void assignLabel(FFNodeLabel *l, int i=-1);
  void assignLabel(MPlex<FFNodeLabel *>&);

  void setLabel(FFNodeLabel *l, int i=-1, BOOL doFree=YES);
  void setLabel(MPlex<FFNodeLabel *>&, BOOL doFree=YES);

  int getLabelIndex(const String&, FFSet<int>*set=NULL) const;
  FFNodeLabel *getLabel(int i)
  {if(label && label->valid(i)) return (*label)[i]; return NULL;}  
  inline const MPlex<FFNodeLabel *> *getLabels() const {return label;}  

  inline FFGraphClass *getGraph() {return derGraph;}

  void show();
  
  FFSet<FFNodeClass *> succ();
  FFSet<FFNodeClass *> pred();
  inline const FFSet<FFEdgeClass *>& transToSucc() const 
    {return transTo;}
  inline const FFSet<FFEdgeClass *>& transFromPred() const 
    {return transFrom;}

  void succ(FFSet<FFNodeClass *> &tmpS);
  void pred(FFSet<FFNodeClass *> &tmpS);
  
  void setName(const String& s);
  inline const String& getName() const {return name;}
  
  void hideGraphic();
  void showGraphic();
  
  inline FFGraphicNode *getGraphicNode()
  {if(derGraph->getGraphicNodeIndex()>=0)
    return (FFGraphicNode *)((*label)[derGraph->getGraphicNodeIndex()]);
  else
    return NULL;}
  inline FFGraphicNode *operator()()
  {if(derGraph->getGraphicNodeIndex()>=0)
    return (FFGraphicNode *)((*label)[derGraph->getGraphicNodeIndex()]);
  else
    return NULL;}
  inline FFGraphicNode *operator->()
  {if(derGraph->getGraphicNodeIndex()>=0)
    return (FFGraphicNode *)((*label)[derGraph->getGraphicNodeIndex()]);
  else
    return NULL;}
};

typedef FFNodeClass * FFNode;

#endif
