/*********************************************************************/
/*                                                                   */
/* Copyright 1994/95/96 Carsten Friedrich, University of Passau.     */
/*                                                                   */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/*********************************************************************/
#ifndef LAYOUT_ALGORITHMEN
#define LAYOUT_ALGORITHMEN


#include <stream.h>
#include <string.h>
#include "FFGraphClass.h"
#include "FFNodeClass.h"
#include "FFEdgeClass.h"
#include "FFGraphicNode.h"
#include "FFGraphicEdge.h"
#include "FFsugiyama.h"
#include "FFGraph.h"
#include "FFPoint.h"

#include <container/AVLSet.h>
#include <container/VOHSet.h>
#include <container/VHSet.h>
#include <container/AVLMap.h>
#include "SLList.h"
#include "ffError.h"


////////////////////////////////////////////////////////////////////////////
//                                                                        //
// Struktur zur Speicherung der Knoten + zusaetzlicher Informationen      //
// (schneller Zugriff fuer den Sugiyama-Algorithmus).                     //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

struct FSNode
{
    FFNode  node;
    int     indeg;
    int     outdeg;
    int     attrib;
    int     level;
    int     dfsNumber;
    int     dummy;    // 0 - kein dummy-Knoten
                      // 1 - dummy-Knoten fuer reflexive Kante
                      // 2 -       ""     fuer parallele Kante
                      // 3 -       ""     nach Hierarchisierung
    int     vertUnit; // zusaetzlicher vertikaler Abstand fuer Layout
                      // auf Grund der Anzahl der Kanten von/zu einem Knoten
    FFPoint point;
    BOOL    leaf;     // ist Knoten Blatt eines Baumes
    FFCoord height;   // Hoehe und Breite eines Knotens
    FFCoord width;   
};


////////////////////////////////////////////////////////////////////////////
//                                                                        //
// Knoten werden numeriert, indem in "value" die Knotennummer gespeichert  //
// wird. Alte Werte koennen fuer einen Graphen in einem int-Array gemerkt //
// und mit "write_back_values" spaeter zurueckgeschrieben werden.            //
//                                                                        //
// Allgemeine Funktionen fuer Springembedder- und Sugiyama-Algorithmus.   //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

void    number_nodes (FFGraph& graph, int* oldValues);
void    number_edges (FFGraph& graph, int* oldValues);
void    write_back_values (FFSet <FFNode>& allNodes, int* oldValues);
void    write_back_edge_values (FFSet <FFEdge>& allEdges, int* oldValues);
FFEdge  get_edge (FFSet <FFEdge>& allEdges, int source, int target);
FFEdge  get_not_used_edge (FFSet <FFEdge>& allEdges, int source, int target, 
			   FFSet <FFEdge>& usedEdges);
FFEdge  get_edge_same_number_not_used (FFSet <FFEdge>& allEdges, 
				       int source, int target, int number,
				       FFSet <FFEdge>& usedEdges);
void    mark_connected_nodes (FFNode& node);

void    init_array (int *array, int size);
void    init_nodearray (FFSet <FFNode>& allNodes, FSNode *nodes);
void    reset_node_attribut (FFSet <FFNode>& allNodes);

FFCoord get_max_x (FFSet <FFNode>& allNodes);
FFCoord get_max_y (FFSet <FFNode>& allNodes);
FFCoord get_min_x (FFSet <FFNode>& allNodes);
FFCoord get_min_y (FFSet <FFNode>& allNodes);
FFNode  get_node_min_x (FFSet <FFNode>& allNodes, FSNode *nodes);
FFNode  get_node_x_y (SLList <FFNode>& allNodes, FFCoord x, FFCoord y, 
		      FSNode *nodes);

int     get_max_level (FSNode *nodes, int anzN);
void    move_succ (FFNode &node, int diff);
void    compute_edges_of_level (FSNode *nodes, int anzN, int maxLevel, 
				FFCoord yDist);
void    make_succ_set (FFSet <FFNode>& allSucc, FFNode node, 
		       FSNode *nodes);
void    make_pred_set (FFSet <FFNode>& allPred, FFNode node, 
		       FSNode *nodes);
void    make_succ_list (SLList <FFNode>& allSucc, FFNode node, 
			FSNode *nodes);
void    make_pred_list (SLList <FFNode>& allPred, FFNode node, 
			FSNode *nodes);

void    set_positions (FSNode *nodes, int anzN, FFCoord yoff, FSSugiInfo info);
void    set_positions3D (FSNode *nodes, int anzN);

void    remove_reflexive_edges (FFGraph &graph, FFSet <FFEdge>& allEdges,
				AVLMap <int, int>& reflexiveEdges, 
				int &refEdges);
void    back_reflexive_edges (FFGraph &graph, AVLMap <int, int>& 
			      reflexiveEdges, int refEdges);

#endif




